@@ -1,34 +1,39 @@
 // used in src/Languages.cpp, encoding=UTF8
-const wxString langNames[] = {
-	wxString("Հայոց լեզու", wxConvUTF8),
-	wxString("Česky", wxConvUTF8),
-	wxString("Dansk", wxConvUTF8),
-	wxString("Deutsch", wxConvUTF8),
-	wxString("English", wxConvUTF8),
-	wxString("Español", wxConvUTF8),
-	wxString("Euskara", wxConvUTF8),
-	wxString("Français", wxConvUTF8),
-	wxString("Ελληνικά", wxConvUTF8),
-	wxString("Italiano", wxConvUTF8),
-	wxString("日本語 ", wxConvUTF8),
-	wxString("한국어", wxConvUTF8),
-	wxString("Magyar", wxConvUTF8),
-	wxString("Македонски", wxConvUTF8),
-	wxString("Nederlands", wxConvUTF8),
-	wxString("Norsk", wxConvUTF8),
-	wxString("Polski", wxConvUTF8),
-	wxString("Português", wxConvUTF8),
-	wxString("Português (Brasil)", wxConvUTF8),
-	wxString("Pyccкий", wxConvUTF8),
-	wxString("Slovenčina", wxConvUTF8),
-	wxString("Slovenščina", wxConvUTF8),
-	wxString("Srpski", wxConvUTF8),
-	wxString("Svenska", wxConvUTF8),
-	wxString("tiếng Việt", wxConvUTF8)
+const char* langNames[] = {
+	"Հայոց լեզու",
+	"简体字",
+	"繁體字",
+	"Česky",
+	"Dansk",
+	"Deutsch",
+	"English",
+	"Español",
+	"Euskara",
+	"Français",
+	"Ελληνικά",
+	"Italiano",
+	"日本語 ",
+	"한국어",
+	"Magyar",
+	"Македонски",
+	"Nederlands",
+	"Norsk",
+	"Polski",
+	"Português",
+	"Português (Brasil)",
+	"Pyccкий",
+	"Slovenčina",
+	"Slovenščina",
+	"Srpski",
+	"Svenska",
+	"Украї́нська",
+	"tiếng Việt"
 };
 
 const wxLanguage langIds[] = {
 	wxLANGUAGE_ARMENIAN,
+	wxLANGUAGE_CHINESE_SIMPLIFIED,
+	wxLANGUAGE_CHINESE_TRADITIONAL,
 	wxLANGUAGE_CZECH,
 	wxLANGUAGE_DANISH,
 	wxLANGUAGE_GERMAN,
@@ -52,10 +57,12 @@ const wxLanguage langIds[] = {
 	wxLANGUAGE_SLOVENIAN,
 	wxLANGUAGE_SERBIAN,
 	wxLANGUAGE_SWEDISH,
+	wxLANGUAGE_UKRAINIAN,
 	wxLANGUAGE_VIETNAMESE
 };
 
 #include "bitmaps/armenia.png.h"
+#include "bitmaps/china.png.h"
 #include "bitmaps/czech.png.h"
 #include "bitmaps/denmark.png.h"
 #include "bitmaps/germany.png.h"
@@ -79,12 +86,15 @@ const wxLanguage langIds[] = {
 #include "bitmaps/slovenia.png.h"
 #include "bitmaps/serbia.png.h"
 #include "bitmaps/sweden.png.h"
+#include "bitmaps/ukraine.png.h"
 #include "bitmaps/vietnam.png.h"
 
 BitmapArray s_langBitmapArray;
 const BitmapArray& GetLangBitmapArray() {
 	if (s_langBitmapArray.size() == 0) {
 		s_langBitmapArray.Add(wxBITMAP_FROM_MEMORY(armenia));
+		s_langBitmapArray.Add(wxBITMAP_FROM_MEMORY(china));
+		s_langBitmapArray.Add(wxBITMAP_FROM_MEMORY(china));
 		s_langBitmapArray.Add(wxBITMAP_FROM_MEMORY(czech));
 		s_langBitmapArray.Add(wxBITMAP_FROM_MEMORY(denmark));
 		s_langBitmapArray.Add(wxBITMAP_FROM_MEMORY(germany));
@@ -108,6 +118,7 @@ const BitmapArray& GetLangBitmapArray() {
 		s_langBitmapArray.Add(wxBITMAP_FROM_MEMORY(slovenia));
 		s_langBitmapArray.Add(wxBITMAP_FROM_MEMORY(serbia));
 		s_langBitmapArray.Add(wxBITMAP_FROM_MEMORY(sweden));
+		s_langBitmapArray.Add(wxBITMAP_FROM_MEMORY(ukraine));
 		s_langBitmapArray.Add(wxBITMAP_FROM_MEMORY(vietnam));
 	}
 	return s_langBitmapArray;
