@@ -3,7 +3,7 @@
 // Purpose:     About dialog
 // Author:      Alex Thuering
 // Created:     6.11.2003
-// RCS-ID:      $Id: About.cpp,v 1.65 2010/05/24 14:44:05 ntalex Exp $
+// RCS-ID:      $Id: About.cpp,v 1.69 2011/02/10 20:50:46 ntalex Exp $
 // Copyright:  (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -100,46 +100,34 @@ About::About(wxWindow* parent): wxDialog(parent, -1, wxEmptyString,
 	wxHtmlWindow* authorsPanel = new wxHtmlWindow(notebook, -1,
 	  wxDefaultPosition, wxDefaultSize, wxSUNKEN_BORDER|wxHW_SCROLLBAR_AUTO);
 	page = _T("<html><body>");
-	page += _T("<table width='100%' cellspacing='1' cellpadding='0' border='0' valign='top'>");
+	page += _T("<table width='100%' cellspacing='2' cellpadding='2' border='0' valign='top'>");
 	
-	page += _T("<tr><td colspan=3><b>");
-	page += _("Author and Maintainer");
-	page += _T("</b></td></tr>");
-	page += _T("<tr><td width='16px'>&nbsp;</td><td colspan='2'>");
+	page += _T("<tr><td colspan='3'><b>") + wxString(_("Author and Maintainer")) + _T("</b></td></tr>");
+	page += _T("<tr><td>&nbsp;</td><td colspan='2'>");
 	page += FixEmail(_T("Alex Th&uuml;ring <alex at thuering.biz>"));
 	page += _T("</td></tr>");
 	
-	page += _T("<tr><td colspan='3'>&nbsp;</td></tr>");
-	
-	page += _T("<tr><td colspan='3'><b>");
-	page += _("Doc Writer");
-	page += _T("</b></td></tr>");
+	page += _T("<tr><td colspan='3'><b>") + wxString(_("Doc Writer")) + _T("</b></td></tr>");
 	page += _T("<tr><td>&nbsp;</td><td colspan='2'>");
 	page += FixEmail(_T("Don Brett <dlbrett at zoominternet.net>"));
 	page += _T("</td></tr>");
 	
-	page += _T("<tr><td colspan='3'>&nbsp;</td></tr>");
-	
-	page += _T("<tr><td colspan='3'><b>");
-	page += _("Packager (.deb)");
-	page += _T("</b></td></tr>");
+	page += _T("<tr><td colspan='3'><b>") + wxString(_("Packager (.deb)")) + _T("</b></td></tr>");
 	page += _T("<tr><td>&nbsp;</td><td colspan='2'>");
 	page += FixEmail(_T("Fabrice Coutadeur <coutadeurf at gmail.com>"));
 	page += _T("</td></tr>");
 	
-	page += _T("<tr><td colspan='3'>&nbsp;</td></tr>");
-	
-	page += _T("<tr><td colspan='3'><b>");
-	page += _("Translations");
-	page += _T("</b></td></tr>");
+	page += _T("<tr><td colspan='3'><b>") + wxString(_("Translations")) + _T("</b></td></tr>");
 	
 	const wxString translations[][2] = {
 		{ wxT("Armenian"), wxT("Hrant Ohanyan <hrantohanyan at mail.am>") },
+		{ wxT("Chinese (Simplified)"), wxString(wxT("程序民工 <lyj8063 at sohu.com>"), wxConvUTF8) },
+		{ wxT("Chinese (Traditional)"), wxString(wxT("紀明村 <tw.chi.ming at gmail.com>"), wxConvUTF8) },
 		{ wxT("Basque"), wxT("Piarres Beobide <pi at beobide.net>") },
 		{ wxT("Czech"), wxString(wxT("Vlastimil Skácel <vlastiks at gmail.com>"), wxConvUTF8) },
 		{ wxT("Danish"), wxT("Patrick Fust <patrick_fust at hotmail.com>, Jørgen Rasmussen <l10n at joergenr.dk>") },
 		{ wxT("Dutch"), wxT("Maikel Egberink <maikel.egberink at gmail.com>, Ron Vennix <ron.vennix at gmail.com>") },
-		{ wxT("French"), wxT("Laurent MIALARET <laurent at arom.fr>, Philippe MACE <philippe.mace at kioskeo.fr>") },
+		{ wxT("French"), wxT("Philippe MACE <philippe.mace at kioskeo.fr>, Pierre Buard <pierre.buard at gmail.com>") },
 		{ wxT("German"), wxT("Alex Th&uuml;ring <alex at thuering.biz>") },
 		{ wxT("Greek"), wxT("Aristotelis Pitaridis <pitaridis at gmail.com>") },
 		{ wxT("Hungarian"), wxString(wxT("Fáczán László <faczan at freemail.hu>"), wxConvUTF8) },
@@ -149,14 +137,15 @@ About::About(wxWindow* parent): wxDialog(parent, -1, wxEmptyString,
 		{ wxT("Macedonian"), wxT("Damjan Dimitrioski <damjandimitrioski at gmail.com>") },
 		{ wxT("Norwegian"), wxString(wxT("Olav Inge Frøynes <olavinge at gmail.com>"), wxConvUTF8) },
 		{ wxT("Polish"), wxString(wxT("Michal Jusyk <drewnozlasu at o2.pl>, Michał Trzebiatowski <hippie_1968 at hotmail.com>"), wxConvUTF8) },
-		{ wxT("Portuguese"), wxT("Henrique Lopes Barone <henriquebarone at quadrochave.com>") },
-		{ wxT("Portuguese (Brazilian)"), wxT("Henrique Lopes Barone <henriquebarone at quadrochave.com>, Marco de Freitas <marcodefreitas at gmail.com>") },
+		{ wxT("Portuguese"), wxString(wxT("João Frade / / 100 NOME PT <100nome.portugal at gmail.com>"), wxConvUTF8) },
+		{ wxT("Portuguese (Brazilian)"), wxT("Henrique Lopes Barone <henriquebarone at quadrochave.com>") },
 		{ wxT("Russian"), wxT("Alex Th&uuml;ring <alex at thuering.biz>") },
 		{ wxT("Slovenian"), wxT("Martin Srebotnjak <miles at filmsi.net>") },
 		{ wxT("Serbian"), wxT("CICIC Ivana <ozzii23 at gmail.com>") },
 		{ wxT("Slovak"), wxT("Michal Kovac <deadsock at deadsock.co.uk>") },
 		{ wxT("Spanish"), wxT("Redy Rodriguez <redy at telefonica.net>, Fabrice Coutadeur <coutadeurf at gmail.com>") },
 		{ wxT("Swedish"), wxT("Daniel Nylander <po at danielnylander.se>") },
+		{ wxT("Ukrainian"), wxT("Федір Зінчук <getthesite at gmail.com>") },
 		{ wxT("Vietnamese"), wxT("Phan Anh <ppanhh at gmail.com>") },
 	};
 	
@@ -164,11 +153,7 @@ About::About(wxWindow* parent): wxDialog(parent, -1, wxEmptyString,
 		page += wxT("<tr><td>&nbsp;</td><td>") + translations[i][0] + wxT("</td><td width='100%'>")
 				+ FixEmail(translations[i][1]) + wxT("</td></tr>");
 	
-	page += _T("<tr><td colspan='3'>&nbsp;</td></tr>");
-	
-	page += _T("<tr><td colspan='3'><b>");
-	page += _("Libraries and Tools");
-	page += _T("</b></td></tr>");
+	page += _T("<tr><td colspan='3'><b>") + wxString(_("Libraries and Tools")) + _T("</b></td></tr>");
 	page += _T("<tr><td>&nbsp;</td><td>wxWidgets</td><td>Julian Smart, Robert Roebling and other</td></tr>");
 	page += _T("<tr><td>&nbsp;</td><td>dvdauthor</td><td>Scott Smith</td></tr>");
 	page += _T("<tr><td>&nbsp;</td><td>cdrtools</td><td>Joerg Schilling</td></tr>");
