@@ -3,7 +3,7 @@
 // Purpose:     The class to store a DVD Structure (Titles/Menus)
 // Author:      Alex Thuering
 // Created:	29.01.2003
-// RCS-ID:      $Id: DVD.cpp,v 1.80 2010/05/22 14:12:36 ntalex Exp $
+// RCS-ID:      $Id: DVD.cpp,v 1.95 2010/10/10 18:03:10 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -24,6 +24,53 @@
 #define DATA_FILE(fname) wxFindDataFile(_T("data") + wxString(wxFILE_SEP_PATH) + fname)
 #define TEMPLATES_DIR wxFindDataDirectory(_T("templates"))
 
+//////////////////////////////// Cell ////////////////////////////////////
+Cell::Cell() {
+	m_chapter = false;
+	m_program = false;
+	m_pause = 0;
+}
+
+wxSvgXmlNode* Cell::GetXML(DVDFileType type) {
+	wxSvgXmlNode* node = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("cell"));
+	if (m_start.length())
+		node->AddProperty(wxT("start"), m_start);
+	if (m_end.length())
+		node->AddProperty(wxT("end"), m_end);
+	if (m_chapter)
+		node->AddProperty(wxT("chapter"), wxT("1"));
+	if (m_program)
+		node->AddProperty(wxT("program"), wxT("1"));
+	if (m_pause != 0)
+		node->AddProperty(wxT("pause"), m_pause == -1 ? wxT("inf") : wxString::Format(wxT("%d"), m_pause));
+	if (m_commands.length())
+		node->AddChild(new wxSvgXmlNode(wxSVGXML_TEXT_NODE, wxEmptyString, m_commands));
+	return node;
+}
+
+bool Cell::PutXML(wxSvgXmlNode* node) {
+	if (node == NULL || node->GetName() != wxT("cell"))
+		return false;
+	long lval;
+	wxString val;
+	if (node->GetPropVal(wxT("start"), &val))
+		SetStart(val);
+	if (node->GetPropVal(wxT("end"), &val))
+		SetEnd(val);
+	if (node->GetPropVal(wxT("chapter"), &val))
+		SetChapter(val == wxT("1") || val == wxT("on") || val == wxT("yes"));
+	if (node->GetPropVal(wxT("program"), &val))
+		SetProgram(val == wxT("1") || val == wxT("on") || val == wxT("yes"));
+	if (node->GetPropVal(wxT("pause"), &val) && val == wxT("inf"))
+		SetPause(-1);
+	else if (node->GetPropVal(wxT("pause"), &val) && val.ToLong(&lval))
+		SetPause(lval);
+	if (node->GetChildren() && (node->GetChildren()->GetType() == wxSVGXML_TEXT_NODE
+				|| node->GetChildren()->GetType() == wxSVGXML_CDATA_SECTION_NODE))
+		SetCommands(node->GetChildren()->GetContent().Strip(wxString::both));
+	return true;
+}
+
 //////////////////////////////// TextSub ////////////////////////////////////
 
 TextSub::TextSub(wxString filename) {
@@ -159,6 +206,7 @@ Stream::Stream(StreamType type, wxString codecName) {
 	m_sourceCodecName = codecName;
 	m_destinationFormat = 1; // vfCOPY/afCOPY/sfCOPY
 	m_sourceChannelNumber = -1;
+	m_sourceAspectRatio = -1;
 }
 
 VideoFormat Stream::GetSourceVideoFormat() {
@@ -193,6 +241,10 @@ wxString Stream::GetSourceFormat() {
 				result += wxT(" (PAL)");
 			else if (GetSourceVideoFormat() == vfNTSC)
 				result += wxT(" (NTSC)");
+			if (round(GetSourceAspectRatio()*100) == 133)
+				result += wxT(", 4:3");
+			else if (round(GetSourceAspectRatio()*100) == 155)
+				result += wxT(", 16:9");
 		}
 		break;
 	case stAUDIO:
@@ -217,8 +269,33 @@ wxString Stream::GetSourceFormat() {
 }
 
 ////////////////////////////////// Vob //////////////////////////////////////
+Vob::Vob() {
+	Init();
+}
 
-Vob::Vob(const Vob& vob) {
+Vob::Vob(wxString filename) {
+	Init();
+	SetFilename(filename);
+}
+
+Vob::Vob(Menu* menu) {
+	Init(menu);
+}
+
+Vob::Vob(Slideshow* slideshow) {
+	Init(NULL, slideshow);
+}
+
+void Vob::Init(Menu* menu, Slideshow* slideshow) {
+	m_pause = 0;
+	m_menu = menu;
+	m_slideshow = slideshow;
+	m_doNotTranscode = false;
+	m_interlaced = true;
+	m_firstField = ffAUTO;
+}
+
+Vob::Vob(const Vob& vob): m_pad(vob.m_pad), m_crop(vob.m_crop) {
 	m_filename = vob.m_filename;
 	m_audioFilenames = vob.m_audioFilenames;
 	for (unsigned int i = 0; i<vob.m_subtitles.size(); i++)
@@ -231,6 +308,9 @@ Vob::Vob(const Vob& vob) {
 	m_duration = vob.m_duration;
 	for (unsigned int i = 0; i<vob.m_streams.size(); i++)
 		m_streams.Add(new Stream(*vob.m_streams[i]));
+	VECTOR_COPY(vob.m_cells, m_cells, Cell);
+	m_interlaced = vob.m_interlaced;
+	m_firstField = vob.m_firstField;
 	m_doNotTranscode = vob.m_doNotTranscode;
 }
 
@@ -241,6 +321,7 @@ Vob::~Vob() {
 		delete m_slideshow;
 	WX_CLEAR_ARRAY(m_subtitles)
 	WX_CLEAR_ARRAY(m_streams)
+	VECTOR_CLEAR(m_cells, Cell)
 }
 
 bool Vob::SetFilename(wxString value) {
@@ -259,6 +340,7 @@ bool Vob::SetFilename(wxString value) {
 		} else if (stream->GetType() == stVIDEO) {
 			stream->SetSourceVideoSize(ffmpeg.GetVideoSize());
 			stream->SetSourceBitrate(ffmpeg.GetBitrate(stIdx));
+			stream->SetSourceAspectRatio(ffmpeg.GetFrameAspectRatio());
 		} else
 			continue;
 		m_streams.Insert(stream, stIdx);
@@ -286,7 +368,7 @@ unsigned int Vob::GetAudioStreamCount() {
 }
 
 unsigned int Vob::GetSubtitleStreamsCount() {
-	return GetSubtitles().GetCount();
+	return m_menu ? 1 : GetSubtitles().GetCount();
 }
 
 bool Vob::AddAudioFile(wxString filename) {
@@ -362,6 +444,7 @@ wxSvgXmlNode* Vob::GetXML(DVDFileType type, DVD* dvd) {
 	}
 	if (fname.length())
 		node->AddProperty(wxT("file"), fname);
+	
 	if (type == DVDSTYLER_XML) {
 		int stIdx = 0;
 		for (; stIdx < (int)(GetStreams().GetCount() - GetAudioFilenames().GetCount()); stIdx++) {
@@ -383,8 +466,11 @@ wxSvgXmlNode* Vob::GetXML(DVDFileType type, DVD* dvd) {
 			if (streamName.length() == 0)
 				continue;
 			wxSvgXmlNode* streamNode = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, streamName);
-			streamNode->AddProperty(wxT("format"),
-					wxString::Format(wxT("%d"), stream->GetDestinationFormat()));
+			streamNode->AddProperty(wxT("format"), wxString::Format(wxT("%d"), stream->GetDestinationFormat()));
+			if (stream->GetType() == stVIDEO && GetInterlaced()) {
+				streamNode->AddProperty(wxT("interlaced"), wxT("1"));
+				streamNode->AddProperty(wxT("firstField"), wxString::Format(wxT("%d"), GetFirstField()));
+			}
 			node->AddChild(streamNode);
 		}
 		for (int i = 0; i < (int)GetAudioFilenames().GetCount(); i++) {
@@ -401,21 +487,50 @@ wxSvgXmlNode* Vob::GetXML(DVDFileType type, DVD* dvd) {
 		}
 		if (GetDoNotTranscode())
 			node->AddProperty(wxT("doNotTranscode"), wxT("1"));
+		if (m_pad.size() == 4 && m_pad[0] + m_pad[1] + m_pad[2] + m_pad[3] > 0) {
+			wxSvgXmlNode* padNode = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("pad"));
+			if (m_pad[0] > 0)
+				padNode->AddProperty(wxT("left"), wxString::Format(wxT("%d"), m_pad[0]));
+			if (m_pad[1] > 0)
+				padNode->AddProperty(wxT("right"), wxString::Format(wxT("%d"), m_pad[1]));
+			if (m_pad[2] > 0)
+				padNode->AddProperty(wxT("top"), wxString::Format(wxT("%d"), m_pad[2]));
+			if (m_pad[3] > 0)
+				padNode->AddProperty(wxT("bottom"), wxString::Format(wxT("%d"), m_pad[3]));
+			node->AddChild(padNode);
+		}
+		if (m_crop.size() == 4 && m_crop[0] + m_crop[1] + m_crop[2] + m_crop[3] > 0) {
+			wxSvgXmlNode* cropNode = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("crop"));
+			if (m_crop[0] > 0)
+				cropNode->AddProperty(wxT("left"), wxString::Format(wxT("%d"), m_crop[0]));
+			if (m_crop[1] > 0)
+				cropNode->AddProperty(wxT("right"), wxString::Format(wxT("%d"), m_crop[1]));
+			if (m_crop[2] > 0)
+				cropNode->AddProperty(wxT("top"), wxString::Format(wxT("%d"), m_crop[2]));
+			if (m_crop[3] > 0)
+				cropNode->AddProperty(wxT("bottom"), wxString::Format(wxT("%d"), m_crop[3]));
+			node->AddChild(cropNode);
+		}
 	}
-
+	
 	if (GetPause() != 0) {
 		wxString pause= wxT("inf");
 		if (GetPause()>0)
 			pause = wxString::Format(wxT("%d"), GetPause());
 		node->AddProperty(wxT("pause"), pause);
 	}
-
+	
 	if (GetChapters().length() && !GetSlideshow())
 		node->AddProperty(wxT("chapters"), GetChapters());
-
+	
+	if (m_cells.size() > 0) {
+		for (vector<Cell*>::iterator it = m_cells.begin(); it != m_cells.end(); it++)
+			node->AddChild((*it)->GetXML(type));
+	}
+	
 	if (GetMenu() && type == DVDSTYLER_XML)
 		node->AddChild(GetMenu()->GetXML(type));
-
+	
 	if (GetSlideshow()) {
 		if (type == DVDSTYLER_XML)
 			node->AddChild(GetSlideshow()->GetXML(type));
@@ -434,11 +549,11 @@ wxSvgXmlNode* Vob::GetXML(DVDFileType type, DVD* dvd) {
 			node->AddProperty(wxT("chapters"), chapters);
 		}
 	}
-
+	
 	return node;
 }
 
-bool Vob::PutXML(wxSvgXmlNode* node, DVD* dvd, int tsi) {
+bool Vob::PutXML(wxSvgXmlNode* node, DVD* dvd, int tsi, int pgci) {
 	wxString val;
 	long lval;
 
@@ -476,6 +591,11 @@ bool Vob::PutXML(wxSvgXmlNode* node, DVD* dvd, int tsi) {
 					stIdx++;
 				}
 			}
+			if (child->GetPropVal(wxT("interlaced"), &val) && val == wxT("1")) {
+				m_interlaced = true;
+				if (child->GetPropVal(wxT("firstField"), &val) && val.length() > 0 && val.ToLong(&lval))
+					m_firstField = (FirstField) lval;
+			}
 		} else if (child->GetName() == wxT("audio")) {
 			if (child->GetChildren() != NULL && child->GetChildren()->GetContent().length() > 0) {
 				val = child->GetChildren()->GetContent();
@@ -499,12 +619,26 @@ bool Vob::PutXML(wxSvgXmlNode* node, DVD* dvd, int tsi) {
 					stIdx++;
 				}
 			}
+		} else if (child->GetName() == wxT("cell")) {
+			Cell* cell = new Cell;
+			cell->PutXML(child);
+			m_cells.push_back(cell);
+		} else if (child->GetName() == wxT("pad")) {
+			m_pad.push_back(child->GetPropVal(wxT("left"), &val) && val.ToLong(&lval) ? lval : 0);
+			m_pad.push_back(child->GetPropVal(wxT("right"), &val) && val.ToLong(&lval) ? lval : 0);
+			m_pad.push_back(child->GetPropVal(wxT("top"), &val) && val.ToLong(&lval) ? lval : 0);
+			m_pad.push_back(child->GetPropVal(wxT("bottom"), &val) && val.ToLong(&lval) ? lval : 0);
+		} else if (child->GetName() == wxT("crop")) {
+			m_crop.push_back(child->GetPropVal(wxT("left"), &val) && val.ToLong(&lval) ? lval : 0);
+			m_crop.push_back(child->GetPropVal(wxT("right"), &val) && val.ToLong(&lval) ? lval : 0);
+			m_crop.push_back(child->GetPropVal(wxT("top"), &val) && val.ToLong(&lval) ? lval : 0);
+			m_crop.push_back(child->GetPropVal(wxT("bottom"), &val) && val.ToLong(&lval) ? lval : 0);
 		} else if (child->GetName() == wxT("textsub")) {
 			TextSub* textsub = new TextSub;
 			textsub->PutXML(child);
 			GetSubtitles().Add(textsub);
 		} else if (child->GetName() == wxT("menu")) {
-			m_menu = new Menu(dvd, tsi);
+			m_menu = new Menu(dvd, tsi, pgci);
 			if (!m_menu->PutXML(child))
 				return false;
 		} else if (child->GetName() == wxT("slideshow"))
@@ -667,34 +801,39 @@ wxSvgXmlNode* Pgc::GetXML(DVDFileType type, DVD* dvd, wxString nextTitle) {
 	wxSvgXmlNode* node = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("pgc"));
 	if (m_entries.size() > 0)
 		node->AddProperty(wxT("entry"), GetEntriesAsString());
-
+	
 	if (m_palette.length())
 		node->AddProperty(wxT("palette"), m_palette);
-
+	
 	if (GetMenu() && type == DVDAUTHOR_XML)
 		GetMenu()->GetXML(type, node);
-
+	
 	if (GetSlideshow() && type == DVDAUTHOR_XML)
 		GetSlideshow()->GetXML(type, node);
-
+	
 	for (int i = 0; i < (int) GetVobs().GetCount(); i++)
 		node->AddChild(GetVobs()[i]->GetXML(type, dvd));
-
-	if (GetPreCommands().length())
-		XmlWriteValue(node, wxT("pre"), GetPreCommands());
-
+	
+	wxString preCommands = GetPreCommands();
+	if (type == DVDAUTHOR_XML && dvd->GetRememberLastButtonRegister() != -1) {
+		wxString g = wxT("g") + wxString::Format(wxT("%d"), dvd->GetRememberLastButtonRegister());
+		preCommands = wxT("if (") + g + wxT(">0) button = ") + g + wxT(";") + preCommands;
+	}
+	if (preCommands.length())
+		XmlWriteValue(node, wxT("pre"), preCommands);
+	
 	wxString postCommands = GetPostCommands();
-	if (dvd->GetPlayAllRegister() != -1 && nextTitle.length()) {
-		wxString g = wxT("g") + wxString::Format(wxT("%d"),dvd->GetPlayAllRegister());
+	if (type == DVDAUTHOR_XML && dvd->GetPlayAllRegister() != -1 && nextTitle.length()) {
+		wxString g = wxT("g") + wxString::Format(wxT("%d"), dvd->GetPlayAllRegister());
 		postCommands = wxT("if (") + g + wxT("==1) jump ") + nextTitle + wxT(";") + postCommands;
 	}
 	if (postCommands.length())
 		XmlWriteValue(node, wxT("post"), postCommands);
-
+	
 	return node;
 }
 
-bool Pgc::PutXML(wxSvgXmlNode* node, DVD* dvd, int tsi) {
+bool Pgc::PutXML(wxSvgXmlNode* node, DVD* dvd, int tsi, int pgci) {
 	WX_CLEAR_ARRAY(GetVobs());
 
 	wxString val;
@@ -713,7 +852,7 @@ bool Pgc::PutXML(wxSvgXmlNode* node, DVD* dvd, int tsi) {
 	while (child) {
 		if (child->GetName() == wxT("spumux")) {
 			// get spunode (old)
-			menu = new Menu(dvd, tsi);
+			menu = new Menu(dvd, tsi, pgci);
 			menu->PutXML(child);
 		} else if (child->GetName() == wxT("vob")) {
 			// get vob
@@ -723,7 +862,7 @@ bool Pgc::PutXML(wxSvgXmlNode* node, DVD* dvd, int tsi) {
 			else
 				vob = new Vob(wxT(""));
 			menu = NULL;
-			if (vob->PutXML(child, dvd, tsi))
+			if (vob->PutXML(child, dvd, tsi, pgci))
 				GetVobs().Add(vob);
 			else {
 				delete vob;
@@ -764,19 +903,21 @@ wxArrayString Video::GetWidescreenStrings(wxString def) {
 	return ret;
 }
 
-wxSvgXmlNode* Video::GetXML(DVDFileType type) {
+wxSvgXmlNode* Video::GetXML(DVDFileType type, bool menu) {
 	wxSvgXmlNode* node = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("video"));
-
-	if (m_format.length())
-		node->AddProperty(wxT("format"), m_format);
-	if (m_aspect.length())
-		node->AddProperty(wxT("aspect"), m_aspect);
-	if (m_resolution.length())
-		node->AddProperty(wxT("resolution"), m_resolution);
+	
+	if (!menu || type == DVDAUTHOR_XML) {
+		if (m_format.length())
+			node->AddProperty(wxT("format"), m_format);
+		if (m_aspect != arAUTO)
+			node->AddProperty(wxT("aspect"), GetAspectStrings()[m_aspect]);
+		if (m_resolution.length())
+			node->AddProperty(wxT("resolution"), m_resolution);
+	}
 	if (m_caption.length())
 		node->AddProperty(wxT("caption"), m_caption);
-	if (m_widescreen.length())
-		node->AddProperty(wxT("widescreen"), m_widescreen);
+	if (m_widescreen != wtAUTO && (m_aspect == ar16_9 || type == DVDSTYLER_XML))
+		node->AddProperty(wxT("widescreen"), GetWidescreenStrings()[m_widescreen]);
 
 	if (node->GetProperties() == NULL) {
 		delete node;
@@ -785,20 +926,98 @@ wxSvgXmlNode* Video::GetXML(DVDFileType type) {
 	return node;
 }
 
-bool Video::PutXML(wxSvgXmlNode* node) {
+bool Video::PutXML(wxSvgXmlNode* node, bool menu) {
 	if (node == NULL || node->GetName() != wxT("video"))
 		return false;
 
 	wxString val;
-	node->GetPropVal(wxT("format"), &m_format);
-	node->GetPropVal(wxT("aspect"), &m_aspect);
-	node->GetPropVal(wxT("resolution"), &m_resolution);
+	if (!menu) {
+		node->GetPropVal(wxT("format"), &m_format);
+		if (node->GetPropVal(wxT("aspect"), &val) && GetAspectStrings().Index(val) > 0)
+			m_aspect = (AspectRatio) GetAspectStrings().Index(val);
+		node->GetPropVal(wxT("resolution"), &m_resolution);
+	}
 	node->GetPropVal(wxT("caption"), &m_caption);
-	node->GetPropVal(wxT("widescreen"), &m_widescreen);
+	if (node->GetPropVal(wxT("widescreen"), &val) && GetWidescreenStrings().Index(val) > 0)
+		m_widescreen = (WidescreenType) GetWidescreenStrings().Index(val);
 
 	return true;
 }
 
+////////////////////////////////// SubStream //////////////////////////////////////
+
+wxString s_subModeArray[4] = { wxT("normal"), wxT("widescreen"), wxT("letterbox"), wxT("panscan") };
+wxArrayString s_subModes(4, s_subModeArray);
+
+wxString s_subConentArray[10] = { wxT("normal"), wxT("large"), wxT("children"), wxT("normal_cc"), wxT("large_cc"),
+		wxT("children_cc"), wxT("forced"), wxT("director"), wxT("large_director"), wxT("children_director") };
+wxArrayString s_subConents(10, s_subConentArray);
+
+wxSvgXmlNode* SubStream::GetXML(DVDFileType type) {
+	wxSvgXmlNode* node = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("stream"));
+	if (m_id.length())
+		node->AddProperty(wxT("id"), m_id);
+	if (m_mode != ssmNORMAL)
+		node->AddProperty(wxT("mode"), s_subModes[m_mode]);
+	if (m_content != sscNORMAL)
+		node->AddProperty(wxT("content"), s_subConents[m_content]);
+	return node;
+}
+
+bool SubStream::PutXML(wxSvgXmlNode* node) {
+	if (node == NULL || node->GetName() != wxT("stream"))
+		return false;
+
+	wxString val;
+	m_id = node->GetPropVal(wxT("id"), wxT(""));
+	int ival = node->GetPropVal(wxT("mode"), &val) && s_subModes.Index(val) > 0 ? s_subModes.Index(val) : 0;
+	m_mode = (SubStreamMode) ival;
+	ival = node->GetPropVal(wxT("content"), &val) && s_subConents.Index(val) > 0 ? s_subConents.Index(val) : 0;
+	m_content = (SubStreamContent) ival;
+
+	return true;
+}
+
+////////////////////////////////// SubPicture //////////////////////////////////////
+
+wxSvgXmlNode* SubPicture::GetXML(DVDFileType type, Video& video, bool menu) {
+	wxSvgXmlNode* node = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("subpicture"));
+	
+	node->AddProperty(wxT("lang"), m_langCode);
+	for (vector<SubStream*>::iterator it = m_streams.begin(); it != m_streams.end(); it++)
+		node->AddChild((*it)->GetXML(type));
+	if (type == DVDAUTHOR_XML && menu && m_streams.size() == 0 && video.GetAspect() == ar16_9) {
+		node->AddChild(SubStream(wxT("0"), ssmWIDESCREEN).GetXML(type));
+		if (video.GetWidescreen() == wtNOLETTERBOX)
+			node->AddChild(SubStream(wxT("1"), ssmPANSCAN).GetXML(type));
+		else
+			node->AddChild(SubStream(wxT("1"), ssmLETTERBOX).GetXML(type));
+		if (video.GetWidescreen() == wtAUTO)
+			node->AddChild(SubStream(wxT("2"), ssmPANSCAN).GetXML(type));
+	}
+	
+	return node;
+}
+
+bool SubPicture::PutXML(wxSvgXmlNode* node) {
+	if (node == NULL || node->GetName() != wxT("subpicture"))
+		return false;
+	
+	wxString val;
+	if (node->GetPropVal(wxT("lang"), &val))
+		m_langCode = val;
+	
+	for (wxSvgXmlNode* child = node->GetChildren(); child != NULL; child = child->GetNext()) {
+		SubStream* subStream = new SubStream();
+		if (subStream->PutXML(child))
+			m_streams.push_back(subStream);
+		else
+			delete subStream;
+	}
+	
+	return true;
+}
+
 ////////////////////////////////// PgcArray //////////////////////////////////////
 
 unsigned int PgcArray::GetAudioStreamCount() {
@@ -829,37 +1048,69 @@ void PgcArray::UpdateAudioLangCodes() {
 
 void PgcArray::UpdateSubtitleLangCodes() {
 	unsigned int subtitleCnt = GetSubtitleStreamsCount();
-	while (m_subtitleLangCodes.size() < subtitleCnt)
-		m_subtitleLangCodes.push_back(s_config.GetDefSubtitleLanguage());
-	while (m_subtitleLangCodes.size() > subtitleCnt)
-		m_subtitleLangCodes.pop_back();
+	while (m_subPictures.size() < subtitleCnt)
+		m_subPictures.push_back(new SubPicture(s_config.GetDefSubtitleLanguage()));
+	while (m_subPictures.size() > subtitleCnt)
+		m_subPictures.pop_back();
+}
+
+/** Sets video format */
+void PgcArray::SetVideoFormat(VideoFormat videoFormat) {
+	for (unsigned int pgci=0; pgci<GetCount(); pgci++) {
+		Menu* menu = (*this)[pgci]->GetMenu();
+		if (menu != NULL)
+			menu->SetVideoFormat(videoFormat);
+	}
+}
+
+/** Sets aspect ratio */
+void PgcArray::SetAspectRatio(AspectRatio aspectRatio) {
+	m_video.SetAspect(aspectRatio);
+	for (unsigned int pgci=0; pgci<GetCount(); pgci++) {
+		Menu* menu = (*this)[pgci]->GetMenu();
+		if (menu != NULL)
+			menu->SetAspectRatio(aspectRatio);
+	}
 }
 
 ////////////////////////////////// Menus //////////////////////////////////////
 StringSet Menus::GetEntries() {
 	StringSet entries;
-	if (Count() > 0)
-		entries.insert(wxT("root"));
 	for (unsigned int pgci = 0; pgci < Count(); pgci++) {
 		const StringSet& pgcEntries = Item(pgci)->GetEntries();
 		for (StringSet::const_iterator entry = pgcEntries.begin(); entry != pgcEntries.end(); entry++) {
-			if (*entry != wxT("root"))
-				entries.insert(*entry);
+			entries.insert(*entry);
 		}
 	}
 	return entries;
 }
 
+int Menus::GetPgciByEntry(wxString entry) {
+	for (unsigned int pgci = 0; pgci < Count(); pgci++) {
+		const StringSet& pgcEntries = Item(pgci)->GetEntries();
+		for (StringSet::const_iterator e = pgcEntries.begin(); e != pgcEntries.end(); e++) {
+			if (*e == entry)
+				return pgci;
+		}
+	}
+	return -1;
+}
+
 wxSvgXmlNode* Menus::GetXML(DVDFileType type, DVD* dvd) {
 	wxSvgXmlNode* node = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("menus"));
+	wxSvgXmlNode* videoNode = GetVideo().GetXML(type, true);
+	if (videoNode)
+		node->AddChild(videoNode);
 	UpdateAudioLangCodes();
 	for (wxArrayString::iterator it = GetAudioLangCodes().begin(); it != GetAudioLangCodes().end(); it++) {
 		wxSvgXmlNode* audio = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("audio"));
 		audio->AddProperty(wxT("lang"), *it);
 		node->AddChild(audio);
 	}
-	if (GetVideo().GetXML(type))
-		node->AddChild(GetVideo().GetXML(type));
+	UpdateSubtitleLangCodes();
+	for (vector<SubPicture*>::iterator it = GetSubPictures().begin(); it != GetSubPictures().end(); it++) {
+		node->AddChild((*it)->GetXML(type, m_video, true));
+	}
 	for (int i = 0; i < (int) GetCount(); i++)
 		node->AddChild((*this)[i]->GetXML(type, dvd, wxT("")));
 	return node;
@@ -874,10 +1125,10 @@ bool Menus::PutXML(wxSvgXmlNode* node, DVD* dvd, int tsi) {
 		if (child->GetName() == wxT("audio") && child->HasAttribute(wxT("lang")))
 			GetAudioLangCodes().Add(child->GetAttribute(wxT("lang")));
 		else if (child->GetName() == wxT("video"))
-			GetVideo().PutXML(child);
+			m_video.PutXML(child, true);
 		else if (child->GetName() == wxT("pgc")) {
 			Pgc* pgc = new Pgc;
-			if (pgc->PutXML(child, dvd, tsi))
+			if (pgc->PutXML(child, dvd, tsi, GetCount()))
 				Add(pgc);
 			else {
 				delete pgc;
@@ -887,6 +1138,15 @@ bool Menus::PutXML(wxSvgXmlNode* node, DVD* dvd, int tsi) {
 		}
 		child = child->GetNext();
 	}
+	if (m_video.GetAspect() == arAUTO && GetCount() > 0) {
+		for (int i = 0; i < (int) GetCount(); i++) {
+			Menu* menu = (*this)[i]->GetMenu();
+			if (menu != NULL) {
+				m_video.SetAspect(menu->GetAspectRatio());
+				break;
+			}
+		}
+	}
 	return true;
 }
 
@@ -894,6 +1154,9 @@ bool Menus::PutXML(wxSvgXmlNode* node, DVD* dvd, int tsi) {
 
 wxSvgXmlNode* Titles::GetXML(DVDFileType type, DVD* dvd, int nextTitleset) {
 	wxSvgXmlNode* node = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("titles"));
+	wxSvgXmlNode* videoNode = GetVideo().GetXML(type, false);
+	if (videoNode)
+		node->AddChild(videoNode);
 	UpdateAudioLangCodes();
 	for (wxArrayString::iterator it = GetAudioLangCodes().begin(); it != GetAudioLangCodes().end(); it++) {
 		wxSvgXmlNode* audio = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("audio"));
@@ -901,13 +1164,9 @@ wxSvgXmlNode* Titles::GetXML(DVDFileType type, DVD* dvd, int nextTitleset) {
 		node->AddChild(audio);
 	}
 	UpdateSubtitleLangCodes();
-	for (wxArrayString::iterator it = GetSubtitleLangCodes().begin(); it != GetSubtitleLangCodes().end(); it++) {
-		wxSvgXmlNode* audio = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("subpicture"));
-		audio->AddProperty(wxT("lang"), *it);
-		node->AddChild(audio);
+	for (vector<SubPicture*>::iterator it = GetSubPictures().begin(); it != GetSubPictures().end(); it++) {
+		node->AddChild((*it)->GetXML(type, m_video, false));
 	}
-	if (GetVideo().GetXML(type))
-		node->AddChild(GetVideo().GetXML(type));
 	for (int i = 0; i < (int) GetCount(); i++) {
 		wxString nextTitle;
 		if (i != (int) GetCount() - 1)
@@ -924,13 +1183,15 @@ bool Titles::PutXML(wxSvgXmlNode* node, DVD* dvd, int tsi) {
 	while (child) {
 		if (child->GetName() == wxT("audio") && child->HasAttribute(wxT("lang"))) {
 			GetAudioLangCodes().Add(child->GetAttribute(wxT("lang")));
-		} else if (child->GetName() == wxT("subpicture") && child->HasAttribute(wxT("lang"))) {
-			GetSubtitleLangCodes().Add(child->GetAttribute(wxT("lang")));
+		} else if (child->GetName() == wxT("subpicture")) {
+			SubPicture* subPicture = new SubPicture();
+			if (subPicture->PutXML(child))
+				GetSubPictures().push_back(subPicture);
 		} else if (child->GetName() == wxT("video")) {
-			GetVideo().PutXML(child);
+			GetVideo().PutXML(child, false);
 		} else if (child->GetName() == wxT("pgc")) {
 			Pgc* pgc = new Pgc;
-			if (pgc->PutXML(child, dvd, tsi))
+			if (pgc->PutXML(child, dvd, tsi, GetCount()))
 				Add(pgc);
 			else {
 				delete pgc;
@@ -1029,6 +1290,7 @@ DVD::DVD() {
 	m_jumppad = false;
 	m_emptyMenu = true;
 	m_playAllRegister = -1;
+	m_rememberLastButtonRegister = -1;
 	m_videoFormat = vfPAL;
 	m_audioFormat = afMP2;
 	m_aspectRatio = ar4_3;
@@ -1098,7 +1360,7 @@ wxArrayInt DVD::CalculateVideoBitrate() {
 	if (m_videoBitrateAuto) {
 		if (GetCapacity() != dcUNLIMITED) {
 			long s = GetCapacityValue() - size - fixSize;
-			bitrate = 500 + (s > 0 && duration > 0 ? s/duration*8 : (duration > 0 ? 0 : 7000));
+			bitrate = 500 + (s > 0 && duration > 0 ? s/duration*8 : (duration > 0 ? 0 : 7500));
 			if (bitrate > 8000)
 				bitrate = 8000;
 		} else
@@ -1135,7 +1397,7 @@ int DVD::AddTitleset() {
 }
 
 int DVD::AddMenu(int tsi) {
-	return AddMenu(new Menu(this, tsi, m_videoFormat, m_aspectRatio), tsi);
+	return AddMenu(new Menu(this, tsi, GetPgcArray(tsi, true).GetCount(), m_videoFormat, m_aspectRatio), tsi);
 }
 
 int DVD::AddMenu(Menu* menu, int tsi) {
@@ -1196,6 +1458,31 @@ Menu* DVD::GetMenu(int tsi, int pgci) {
 	return NULL;
 }
 
+/** Returns true if all menus are ok */
+bool DVD::CheckMenus() {
+	for (int tsi = -1; tsi < (int) GetTitlesets().Count(); tsi++) {
+		PgcArray& pgcs = GetPgcArray(tsi, true);
+		for (unsigned int pgci = 0; pgci < pgcs.GetCount(); pgci++) {
+			Menu* menu = pgcs[pgci]->GetMenu();
+			if (menu) {
+				if (menu->GetAspectRatio() == ar16_9) {
+					if (pgcs.GetVideo().GetWidescreen() == wtAUTO && menu->GetButtonsCount() > 12) {
+						wxLogError(_("Wide screen DVD menu can contain maximal 12 buttons"));
+						return false;
+					} else if (menu->GetButtonsCount() > 18) {
+						wxLogError(_("Wide screen DVD menu (nopanscan/noletterbox) can contain maximal 18 buttons"));
+						return false;
+					}
+				} else if (menu->GetButtonsCount() > 34) {
+					wxLogError(_("DVD menu can contain maximal 34 buttons"));
+					return false;
+				}
+			}
+		}
+	}
+	return true;
+}
+
 /** Returns true if all actions are valid */
 bool DVD::CheckActions(bool skipInvalidTarget) {
 	bool valid = true;
@@ -1251,15 +1538,17 @@ bool DVD::CheckActions(bool skipInvalidTarget) {
 }
 
 /** Updates image in buttons with given jump action */
-void DVD::UpdateButtonImageFor(int actionTsi, int actionPgci) {
+bool DVD::UpdateButtonImageFor(int actionTsi, int actionPgci) {
+	bool result = false;
 	for (int tsi = -1; tsi < (int) GetTitlesets().GetCount(); tsi++) {
 		PgcArray& pgcs = GetPgcArray(tsi, true);
 		for (unsigned int pgci = 0; pgci < pgcs.GetCount(); pgci++) {
 			Menu* menu = pgcs[pgci]->GetMenu();
 			if (menu)
-				menu->UpdateButtonImageFor(actionTsi != tsi ? actionTsi : -2, actionPgci, this);
+				result |= menu->UpdateButtonImageFor(actionTsi != tsi ? actionTsi : -2, actionPgci, this);
 		}
 	}
+	return result;
 }
 
 
@@ -1275,6 +1564,30 @@ void DVD::FixButtonCoordinates() {
 	}
 }
 
+
+/** Sets video format of DVD */
+void DVD::SetVideoFormat(VideoFormat videoFormat, bool updateMenus) {
+	m_videoFormat = videoFormat;
+	if (updateMenus) {
+		for (int tsi=-1; tsi< (int) GetTitlesets().GetCount(); tsi++)
+			GetPgcArray(tsi, true).SetVideoFormat(videoFormat);
+	}
+}
+
+/** Sets audio format of DVD */
+void DVD::SetAudioFormat(AudioFormat format) {
+	m_audioFormat = format;
+}
+
+/** Sets aspect ratio of DVD */
+void DVD::SetAspectRatio(AspectRatio aspectRatio, bool updateMenus) {
+	m_aspectRatio = aspectRatio;
+	if (updateMenus) {
+		for (int tsi=-1; tsi< (int) GetTitlesets().GetCount(); tsi++)
+			GetPgcArray(tsi, true).SetAspectRatio(aspectRatio);
+	}
+}
+
 /** Loads a project file */
 bool DVD::Open(wxString fname) {
 	wxSvgXmlDocument xml;
@@ -1362,6 +1675,7 @@ bool DVD::PutXML(const wxSvgXmlDocument& xml) {
 /** Returns XML Document */
 wxSvgXmlDocument* DVD::GetXml() {
 	m_playAllRegister = -1;
+	m_rememberLastButtonRegister = -1;
 	wxSvgXmlDocument* xml = new wxSvgXmlDocument;
 	wxSvgXmlNode* root = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("dvdstyler"));
 	root->AddProperty(wxT("format"), wxT("4"));
@@ -1416,7 +1730,9 @@ void DVD::Deserialize(const wxString& data) {
 
 bool DVD::SaveDVDAuthor(wxString fname) {
 	// check if we need "play all"
+	m_playAllRegister = -1;
 	m_playAllRegister = HasPlayAllButton() ? FindFreeRegister() : -1;
+	m_rememberLastButtonRegister = HasRememberLastButton() ? FindFreeRegister() : -1;
 	// save config for dvdauthor
 	wxSvgXmlDocument xml;
 	wxSvgXmlNode* root = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("dvdauthor"));
@@ -1433,7 +1749,8 @@ bool DVD::SaveDVDAuthor(wxString fname) {
 }
 
 wxString DVD::GetPath(bool withSep) {
-	return withSep ? wxPathOnly(m_filename) + wxFILE_SEP_PATH : wxPathOnly(m_filename);
+	wxString path = wxPathOnly(m_filename);
+	return withSep && path.length() > 0 ? path + wxFILE_SEP_PATH : path;
 }
 
 long DVD::GetSize(bool generated) {
@@ -1472,6 +1789,7 @@ long DVD::GetRequiredSize(Cache* cache) {
 	return size;
 }
 
+/** Returns true if some of buttons have flag "Play All" checked */
 bool DVD::HasPlayAllButton() {
 	for (int tsi = -1; tsi<(int)GetTitlesets().Count(); tsi++) {
 		PgcArray& pgcs = GetPgcArray(tsi, true);
@@ -1492,6 +1810,22 @@ bool DVD::HasPlayAllButton() {
 	return false;
 }
 
+/** Returns true if some of menus have flag "Remember last selected button" checked */
+bool DVD::HasRememberLastButton() {
+	for (int tsi = -1; tsi<(int)GetTitlesets().Count(); tsi++) {
+		PgcArray& pgcs = GetPgcArray(tsi, true);
+		for (int pgci = 0; pgci<(int)pgcs.Count(); pgci++) {
+			Pgc* pgc = pgcs[pgci];
+			for (int vobi = 0; vobi<(int)pgc->GetVobs().Count(); vobi++) {
+				Vob* vob = pgc->GetVobs()[vobi];
+				if (vob->GetMenu() && vob->GetMenu()->GetRememberLastButton())
+					return true;
+			}
+		}
+	}
+	return false;
+}
+
 int DVD::FindFreeRegister() {
 	for (int g=0; g<=12; g++) {
 		if (!IsRegisterUsed(g))
@@ -1501,6 +1835,8 @@ int DVD::FindFreeRegister() {
 }
 
 bool DVD::IsRegisterUsed(int g) {
+	if (m_playAllRegister == g)
+		return true;
 	wxString gstr = wxT("g") + wxString::Format(wxT("%d"),g);
 	for (int tsi = -1; tsi<(int)GetTitlesets().Count(); tsi++) {
 		for (int m=0; m<=1; m++) {
