@@ -3,7 +3,7 @@
 // Purpose:     The class to store a DVD Structure (Titles/Menus)
 // Author:      Alex Thuering
 // Created:	29.01.2003
-// RCS-ID:      $Id: DVD.h,v 1.52 2010/05/03 06:03:20 ntalex Exp $
+// RCS-ID:      $Id: DVD.h,v 1.64 2010/10/10 15:54:55 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -12,12 +12,14 @@
 #define DVDAUTHOR_H
 
 #include "Slideshow.h"
+#include "mediatrc.h"
 #include <wx/wx.h>
 #include <wx/image.h>
 #include <wx/dynarray.h>
 #include <wx/hashset.h>
 #include <wxSVG/mediadec_ffmpeg.h>
 #include <map>
+#include <vector>
 
 using namespace std;
 
@@ -29,6 +31,62 @@ class DVD;
 class Menu;
 class Cache;
 
+#define VECTOR_CLEAR(obj, elem_type) \
+	for (vector<elem_type*>::iterator vectorIt = obj.begin(); vectorIt != obj.end(); vectorIt++) \
+		delete *vectorIt; \
+	obj.clear();
+
+#define VECTOR_COPY(src, dst, elem_type) \
+	for (vector<elem_type*>::const_iterator vectorIt = src.begin(); vectorIt != src.end(); vectorIt++) \
+		dst.push_back(new elem_type(**vectorIt));
+
+/** The cell. It defines a marker point in a title. */
+class Cell {
+public:
+	/** Constructor */
+	Cell();
+	
+	/** Returns the start time */
+	wxString GetStart() const { return m_start; }
+	/** Sets the start time */
+	void SetStart(wxString start) { m_start = start; }
+	
+	/** Returns the end time */
+	wxString GetEnd() const { return m_end; }
+	/** Sets the end time */
+	void SetEnd(wxString end) { m_end = end; }
+	
+	/** Returns true if cell is a chapter point */
+	bool IsChapter() const { return m_chapter; }
+	/** Sets cell as chapter point */
+	void SetChapter(bool chapter) { m_chapter = chapter; }
+	
+	/** Returns true if cell is a program point */
+	bool GetProgram() const { return m_program; }
+	/** Sets cell as program point */
+	void SetProgram(bool program) { m_program = program; }
+	
+	/** Returns the pause */
+	int GetPause() const { return m_pause; }
+	/** Sets the pause */
+	void SetPause(int pause) { m_pause = pause; }
+	
+	/** Returns VM commands that will be executed when the cell plays. */
+	wxString GetCommands() const { return m_commands; }
+	/** Sets the VM commands that will be executed when the cell plays. */
+	void SetCommands(wxString commands) { m_commands = commands; }
+		
+	wxSvgXmlNode* GetXML(DVDFileType type);
+	bool PutXML(wxSvgXmlNode* node);
+private:
+	wxString m_start;
+	wxString m_end;
+	bool m_chapter;
+	bool m_program;
+	int m_pause;
+	wxString m_commands;
+};
+
 class TextSub {
 public:
 	TextSub(wxString filename = wxT(""));
@@ -115,6 +173,9 @@ public:
 	wxSize GetSourceVideoSize() { return m_sourceVideoSize; }
 	void SetSourceVideoSize(wxSize sourceVideoSize) { m_sourceVideoSize = sourceVideoSize; }
 	
+	float GetSourceAspectRatio() { return m_sourceAspectRatio; }
+	void SetSourceAspectRatio(float sourceAspectRatio) { m_sourceAspectRatio = sourceAspectRatio; }
+		
 	VideoFormat GetSourceVideoFormat();
 	AudioFormat GetSourceAudioFormat();
 	
@@ -126,7 +187,7 @@ public:
 	
 	int GetSourceSampleRate() { return m_sourceSampleRate; }
 	void SetSourceSampleRate(int sourceSampleRate) { m_sourceSampleRate = sourceSampleRate; }
-
+	
 	/** @return the stream format string */
 	wxString GetSourceFormat();
 	
@@ -149,6 +210,7 @@ private:
 	int m_sourceChannelNumber; // number of audio channels
 	int m_sourceSampleRate; // sample rate of audio
 	wxSize m_sourceVideoSize; // frame size of source video
+	float m_sourceAspectRatio; // aspect ratio of source video
 	int m_destinationFormat; // destination audio, video or subtitle format
 };
 
@@ -156,10 +218,10 @@ WX_DEFINE_ARRAY(Stream*, StreamArray);
 
 class Vob {
 public:
-	Vob(): m_pause(0), m_menu(NULL), m_slideshow(NULL), m_doNotTranscode(false) {}
-	Vob(wxString filename): m_pause(0), m_menu(NULL), m_slideshow(NULL), m_doNotTranscode(false) { SetFilename(filename); }
-	Vob(Menu* menu): m_pause(0), m_menu(menu), m_slideshow(NULL), m_doNotTranscode(false) {}
-	Vob(Slideshow* slideshow): m_pause(0), m_menu(NULL), m_slideshow(slideshow), m_doNotTranscode(false) {}
+	Vob();
+	Vob(wxString filename);
+	Vob(Menu* menu);
+	Vob(Slideshow* slideshow);
 	Vob(const Vob& vob);
 	~Vob();
 	
@@ -201,6 +263,19 @@ public:
 	void SetPause(int value) { m_pause = value; }
     /** returns pause */
 	int GetPause() { return m_pause; }
+	
+    /** Returns cells */
+	vector<Cell*>& GetCells() { return m_cells; }
+	/** Returns pad values */
+	vector<int>& GetPad() { return m_pad; }
+	/** Returns crop values */
+	vector<int>& GetCrop() { return m_crop; }
+	
+	bool GetInterlaced() { return m_interlaced; }
+	void SetInterlaced(bool interlaced) { m_interlaced = interlaced; }
+	
+	FirstField GetFirstField() { return m_firstField;; }
+	void SetFirstField(FirstField firstField) { m_firstField = firstField; }
 
 	void SetMenu(Menu* menu) { m_menu = menu; }
 	Menu* GetMenu() { return m_menu; }
@@ -221,7 +296,7 @@ public:
    	inline void SetDoNotTranscode(bool doNotTranscode) { m_doNotTranscode = doNotTranscode; }
 	
 	wxSvgXmlNode* GetXML(DVDFileType type, DVD* dvd);
-	bool PutXML(wxSvgXmlNode* node, DVD* dvd, int tsi);
+	bool PutXML(wxSvgXmlNode* node, DVD* dvd, int tsi, int pgci);
 	
 	/**
 	 * Returns size in KB
@@ -245,6 +320,11 @@ private:
 	wxString m_tmpFilename;
 	wxString m_chapters;
 	int m_pause;
+	vector<Cell*> m_cells;
+	vector<int> m_pad;
+	vector<int> m_crop;
+	bool m_interlaced;
+	FirstField m_firstField;
 	Menu* m_menu;
     Slideshow* m_slideshow;
 	static unsigned int GetFileSize(wxString filename);
@@ -252,6 +332,7 @@ private:
 	double m_duration;
 	StreamArray m_streams;
 	bool m_doNotTranscode;
+	void Init(Menu* menu = NULL, Slideshow* slideshow = NULL);
 };
 
 WX_DEFINE_ARRAY(Vob*, VobArray);
@@ -281,7 +362,7 @@ public:
     unsigned int GetChapterCount();
 	
 	wxSvgXmlNode* GetXML(DVDFileType type, DVD* dvd, wxString nextTitle);
-	bool PutXML(wxSvgXmlNode* node, DVD* dvd, int tsi);
+	bool PutXML(wxSvgXmlNode* node, DVD* dvd, int tsi, int pgci);
 	
 private:
 	VobArray m_vobs;
@@ -291,15 +372,25 @@ private:
 
 WX_DEFINE_ARRAY(Pgc*, PgcArrayBase);
 
+enum WidescreenType {
+	wtAUTO = 0,
+	wtNOPANSCAN,
+	wtNOLETTERBOX
+};
+
+/** Video parameters of titleset */
 class Video {
 public:
+	/** Constructor */
+	Video(): m_aspect(arAUTO), m_widescreen(wtNOPANSCAN) {}
+
 	void SetFormat(wxString value) { m_format = value; }
 	wxString GetFormat() { return m_format; }
-	wxArrayString GetFormatStrings(wxString def);
+	wxArrayString GetFormatStrings(wxString def = wxT(""));
 	
-	void SetAspect(wxString value) { m_aspect = value; }
-	wxString GetAspect() { return m_aspect; }
-	wxArrayString GetAspectStrings(wxString def);
+	void SetAspect(AspectRatio value) { m_aspect = value; }
+	AspectRatio GetAspect() { return m_aspect; }
+	wxArrayString GetAspectStrings(wxString def = wxT(""));
 	
 	void SetResolution(wxString value) { m_resolution = value; }
 	wxString GetResolution() { return m_resolution; }
@@ -307,25 +398,98 @@ public:
 	void SetCaption(wxString value) { m_caption = value; }
 	wxString GetCaption() { return m_caption; }
 	
-	void SetWidescreen(wxString value) { m_widescreen = value; }
-	wxString GetWidescreen() { return m_widescreen; }
-	wxArrayString GetWidescreenStrings(wxString def);
+	void SetWidescreen(WidescreenType value) { m_widescreen = value; }
+	WidescreenType GetWidescreen() { return m_widescreen; }
+	wxArrayString GetWidescreenStrings(wxString def = wxT(""));
 	
-	wxSvgXmlNode* GetXML(DVDFileType type);
-	bool PutXML(wxSvgXmlNode* node);
+	wxSvgXmlNode* GetXML(DVDFileType type, bool menu);
+	bool PutXML(wxSvgXmlNode* node, bool menu);
 	
 private:
 	wxString m_format;
-	wxString m_aspect;
+	AspectRatio m_aspect;
 	wxString m_resolution;
 	wxString m_caption;
-	wxString m_widescreen;
+	WidescreenType m_widescreen;
+};
+
+enum SubStreamContent {
+	sscNORMAL = 0, sscLARGE, sscCHILDREN, sscNORMAL_CC, sscLARGE_CC, sscCHILDREN_CC, sscFORCED, sscDIRECTOR,
+	sscLARGE_DIRECTOR, sscCHILDREN_DIRECTOR
+};
+
+/**
+ * Stores information about subpicture stream
+ */
+class SubStream {
+public:
+	/** Default constructor */
+	SubStream(): m_mode(ssmNORMAL), m_content(sscNORMAL) {}
+	/** Constructor */
+	SubStream(wxString id, SubStreamMode mode): m_id(id), m_mode(mode), m_content(sscNORMAL) {}
+	
+	/** Sets id */
+	void SetId(wxString id) { m_id = id; }
+	/** Returns id */
+	wxString GetId() { return m_id; }
+	
+	/** Sets mode */
+	void SetMode(SubStreamMode mode) { m_mode = mode; }
+	/** Returns mode */
+	SubStreamMode GetMode() { return m_mode; }
+	
+	/** Sets content type */
+	void SetContent(SubStreamContent content) { m_content = content; }
+	/** Returns content type */
+	SubStreamContent GetContent() { return m_content; }
+	
+	wxSvgXmlNode* GetXML(DVDFileType type);
+	bool PutXML(wxSvgXmlNode* node);
+		
+private:
+	wxString m_id;
+	SubStreamMode m_mode;
+	SubStreamContent m_content;
+};
+
+/**
+ * Stores information about subpictures
+ */
+class SubPicture {
+public:
+	/** Constructor */
+	SubPicture() { }
+	/** Constructor */
+	SubPicture(wxString langCode): m_langCode(langCode) { }
+	/** Destructor */
+	~SubPicture() { VECTOR_CLEAR(m_streams, SubStream) }
+	
+	/** Returns language code */
+	wxString GetLangCode() { return m_langCode; }
+	/** Sets language code */
+	void SetLangCode(wxString value) { m_langCode = value; }
+	
+	/** Returns list of subpicture streams */
+	vector<SubStream*>& GetStreams() { return m_streams; }
+	
+	wxSvgXmlNode* GetXML(DVDFileType type, Video& video, bool menu);
+	bool PutXML(wxSvgXmlNode* node);
+	
+private:
+	wxString m_langCode;
+	vector<SubStream*> m_streams;
 };
 
 class PgcArray: public PgcArrayBase {
 public:
+	/** Destructor */
+	~PgcArray() { VECTOR_CLEAR(m_subPictures, SubPicture) }
+	
+	/** Returns list of audio codes */
 	wxArrayString& GetAudioLangCodes() { return m_audioLangCodes; }
-	wxArrayString& GetSubtitleLangCodes() { return m_subtitleLangCodes; }
+	/** Retuens list of subpictures */
+	vector<SubPicture*>& GetSubPictures() { return m_subPictures; }
+	/** Returns video parameters */
 	Video& GetVideo() { return m_video; }
 	
 	/** Returns count of audio streams */
@@ -333,9 +497,14 @@ public:
 	/** Returns count of subtitle streams */
 	unsigned int GetSubtitleStreamsCount();
 	
+	/** Sets video format */
+	void SetVideoFormat(VideoFormat videoFormat);
+	/** Sets aspect ratio */
+	void SetAspectRatio(AspectRatio aspectRatio);
+	
 protected:
 	wxArrayString m_audioLangCodes;
-	wxArrayString m_subtitleLangCodes;
+	vector<SubPicture*> m_subPictures;
 	Video m_video;
 	void UpdateAudioLangCodes();
 	void UpdateSubtitleLangCodes();
@@ -344,6 +513,7 @@ protected:
 class Menus: public PgcArray {
 public:
 	StringSet GetEntries();
+	int GetPgciByEntry(wxString entry);
 	
 	wxSvgXmlNode* GetXML(DVDFileType type, DVD* dvd);
 	bool PutXML(wxSvgXmlNode* node, DVD* dvd, int tsi);
@@ -470,10 +640,12 @@ public:
 	Vob* GetMenuVob(int tsi, int pgci);
 	/** returns menu from titleset tsi und pgc pgci */
 	Menu* GetMenu(int tsi, int pgci);
+	/** Returns true if all menus are ok */
+	bool CheckMenus();
 	/** Returns true if all actions are valid */
 	bool CheckActions(bool skipInvalidTarget);
 	/** Updates image in buttons with given jump action */
-	void UpdateButtonImageFor(int actionTsi, int actionPgci);
+	bool UpdateButtonImageFor(int actionTsi, int actionPgci);
 	/** Fix coordinates of buttons if they are out of range */
 	void FixButtonCoordinates();
 	
@@ -495,19 +667,25 @@ public:
 	 *  or -1 if "play all" is not used
 	 */
 	inline int GetPlayAllRegister() { return m_playAllRegister; }
+	/**
+	 *  Returns number of g-register that will be used to implement "remember last selected button"
+	 *  or -1 if "remember last selected button" is not used
+	 */
+	inline int GetRememberLastButtonRegister() { return m_rememberLastButtonRegister; }
 	
+
 	/** Returns video format of DVD */
 	inline VideoFormat GetVideoFormat() { return m_videoFormat; }
 	/** Sets video format of DVD */
-	void SetVideoFormat(VideoFormat format) { m_videoFormat = format; }
+	void SetVideoFormat(VideoFormat format, bool updateMenus = false);
 	/** Returns audio format of DVD */
 	inline AudioFormat GetAudioFormat() { return m_audioFormat; }
 	/** Sets audio format of DVD */
-	void SetAudioFormat(AudioFormat format) { m_audioFormat = format; }
+	void SetAudioFormat(AudioFormat format);
 	/** Returns aspect ratio of DVD */
 	inline AspectRatio GetAspectRatio() { return m_aspectRatio; }
 	/** Sets aspect ratio of DVD */
-	void SetAspectRatio(AspectRatio format) { m_aspectRatio = format; }
+	void SetAspectRatio(AspectRatio format, bool updateMenus = false);
 	
     /** Loads a project file */
 	bool Open(wxString fname);
@@ -565,11 +743,14 @@ private:
 	TitlesetArray m_titlesets;
 	Vmgm m_vmgm;
 	int m_playAllRegister;
+	int m_rememberLastButtonRegister;
 	VideoFormat m_videoFormat;
 	AudioFormat m_audioFormat;
 	AspectRatio m_aspectRatio;
 	/** Returns true if some of buttons have flag "Play All" checked */
 	bool HasPlayAllButton();
+	/** Returns true if some of menus have flag "Remember last selected button" checked */
+	bool HasRememberLastButton();
 	/** Returns number of g-regsiter that is not used */
 	int FindFreeRegister();
 	/** Returns true if given g-register is used in pgc pre/post or in menu button action */
