@@ -3,7 +3,7 @@
 // Purpose:     Stores a DVD Action
 // Author:      Alex Thuering
 // Created:     03.09.2009
-// RCS-ID:      $Id: DVDAction.cpp,v 1.15 2010/05/22 14:13:18 ntalex Exp $
+// RCS-ID:      $Id: DVDAction.cpp,v 1.20 2010/09/05 10:54:03 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -35,8 +35,13 @@ wxString DVDAction::GetId(bool translate) {
 	return m_id;
 }
 
+/* entry menu types */
+const wxString s_entriesArr[6] = {wxT("title"), wxT("root"), wxT("subtitle"), wxT("audio"), wxT("angle"), wxT("ptt")};
+const wxArrayString s_entries(6, s_entriesArr);
+
 wxString DVDAction::AsString(DVD* dvd, bool translate) {
 	wxString action;
+	wxString actionPost;
 	if (m_audio >= 0)
 		action = translate ? wxString::Format(_("select audio track %d and "), m_audio + 1)
 			: wxString::Format(wxT("audio=%d; "), m_audio);
@@ -46,46 +51,65 @@ wxString DVDAction::AsString(DVD* dvd, bool translate) {
 	if (m_custom.length()) // custom action selected
 		return action + m_custom;
 	// jump to selected
-	action += translate ? _("jump to") : wxT("jump");
-	action += wxT(" ");
-	if (GetPgci() == -1) // jump to cell 1
-		return action + (translate ? _("start of current menu") : wxT("cell 1;"));
+	if (!translate)
+		action += wxT("jump ");
+	if (GetPgci() == -1 && GetEntry().length() == 0) // jump to cell 1
+		return action + (translate ? _("jump to start of current menu") : wxT("cell 1;"));
 	int tsi = GetTsi();
 	bool menu = IsMenu();
 	wxString entry = GetEntry();
 	int pgci = GetPgci();
 	if (tsi >= 0 && (!m_vmg || menu || translate)) { // don't use titleset for jumping to title from vmg
-		action += translate ? _("titleset") : wxT("titleset");
-		action += wxString::Format(wxT(" %d "), tsi + 1);
+		if (!translate)
+			action += wxString::Format(wxT("titleset %d "), tsi + 1);
+		else if (dvd->GetTitlesets().size() > 1)
+			actionPost = wxT(" ") + wxString::Format(_("from titleset %d"), tsi + 1);
 	}
-	if (menu) {
-		if (tsi == -1)
-			action += translate ? _("vmMenu") : wxT("vmgm menu");
-		else
-			action += translate ? _("menu") : wxT("menu");
-	} else
-		action += translate ? _("title") : wxT("title");
-	bool hasEntry = false;
-	if (entry.length() > 0 && menu && tsi >= 0) {
-		StringSet entries = dvd->GetTitlesets()[tsi]->GetMenus().GetEntries();
-		hasEntry = entries.find(entry) != entries.end();
-	}
-	if (hasEntry) {
-		action += wxT(" entry ") + entry;
-	} else if (!menu || pgci > 0 || tsi == -2 || translate) {
+	if (menu && entry.length() > 0) {
+		if (translate) {
+			if (s_entries.Index(entry) >= 0) {
+				const wxString jumpToEntry[6] = {
+						_("jump to title vmMenu"), _("jump to root menu"), _("jump to subtitle menu"),
+						_("jump to audio menu"), _("jump to angle menu"), _("jump to chapter menu")};
+				action += jumpToEntry[s_entries.Index(entry)];
+				Menus& menus = (Menus&) dvd->GetPgcArray(tsi, true);
+				int menuIdx = menus.GetPgciByEntry(entry);
+				if (menuIdx >= 0)
+					action += wxT(" (") + wxString(_("menu")) + wxString::Format(wxT(" %d)"), menuIdx + 1);
+			} else {
+				action += _("jump to menu") + wxString(wxT(" entry ")) + entry;
+			}
+		} else
+			action += wxT(" entry ") + entry;
+	} else {
 		int n = pgci + 1;
 		if (m_vmg && !menu && !translate) // use sequence number of title for jumping to title from vmg
 			for (int i = 0; i < tsi; i++)
 				n += dvd->GetTitlesets()[i]->GetTitles().Count();
-		action += wxString::Format(wxT(" %d"), n);
+		
+		if (menu) {
+			if (tsi == -1)
+				action += translate ? _("jump to vmMenu") : wxT("vmgm menu");
+			else
+				action += translate ? _("jump to menu") : wxT("menu");
+			if (tsi < 0 || n > 1 || translate)
+				action += wxString::Format(wxT(" %d"), n);
+		} else {
+			if (GetChapter() > 0) {
+				if (translate) {
+					action += wxString::Format(_("jump to title %d chapter %d"), n, GetChapter() + 1);
+				} else {
+					action += wxString::Format(wxT("title %d chapter %d"), n, GetChapter() + 1);
+				}
+			} else {
+				action += translate ? _("jump to title") : wxT("title");
+				action += wxString::Format(wxT(" %d"), n);
+			}
+		}
 	}
-	if (GetChapter() > 0 && (!m_vmg
-			|| dvd->IsJumppadsEnabled())) { // cannot jump to chapter from vmg if jumppad disabled 
-		action += wxT(" ");
-		action += translate ? _("chapter") : wxT("chapter");
-		action += wxString::Format(wxT(" %d"), GetChapter() + 1);
-	}
-	if (!translate)
+	if (translate)
+		action += actionPost;
+	else
 		action += wxT(";");
 	return action;
 }
@@ -96,14 +120,14 @@ void DVDAction::SetCustom(wxString action) {
 	wxRegEx reSpace(wxT("[[:space:]]+"));
 	reSpace.ReplaceAll(&action, wxT(" "));
 	// check jump command
-	wxString ptJump = wxT("^jump (titleset ([[:digit:]]+) )?(vmgm )?(menu ([[:digit:]]+))?");
-	ptJump += wxT("(menu entry ([[:alpha:]]+))?(title ([[:digit:]]+)( chapter ([[:digit:]]+))?)?;$");
+	wxString ptJump = wxT("^jump ((titleset ([1-9][0-9]?) )|(vmgm ))?((menu ([1-9][0-9]?))|");
+	ptJump += wxT("(menu entry (title|root|subtitle|audio|angle|ptt))|(title ([1-9][0-9]?)( chapter ([1-9][0-9]*))?));$");
 	wxRegEx reJump(ptJump);
 	if (reJump.Matches(action)) {
-		int i = 2;
 		long lval;
 		m_tsi = -2;
 		// titleset
+		int i = 3;
 		wxString val = reJump.GetMatch(action, i++);
 		if (val.length() > 0 && val.ToLong(&lval))
 			m_tsi = lval - 1;
@@ -111,12 +135,13 @@ void DVDAction::SetCustom(wxString action) {
 		val = reJump.GetMatch(action, i++);
 		if (val.length())
 			m_tsi = -1;
-		i++;
+		i += 2;
 		// menu x
 		val = reJump.GetMatch(action, i++);
 		if (val.length() && val.ToLong(&lval)) {
 			m_menu = true;
 			m_pgci = lval - 1;
+			m_entry = wxT("");
 		} else {
 			// menu entry x
 			i++;
@@ -141,7 +166,10 @@ void DVDAction::SetCustom(wxString action) {
 				}
 			}
 		}
-		m_custom = wxT("");
+		if (m_tsi == -1 && !m_menu) // not valid action => set it as custom action
+			m_custom = action;
+		else
+			m_custom = wxT("");
 	} else {
 		m_custom = action;
 	}
@@ -161,17 +189,8 @@ wxString DVDAction::GetSourceStr(DVD* dvd, int tsi, int pgci, bool menu, wxStrin
 	return source;
 }
 
-/* entry menu types */
-const wxString entries[6] = {wxT("title"), wxT("root"), wxT("subtitle"), wxT("audio"), wxT("angle"), wxT("ptt")};
-const wxString entriesTransl[6] = {_("title"), _("root"), _("subtitle"), _("audio"), _("angle"), _("chapter")};
-
 /** Validates jump target */
-bool validateJumpTarget(wxString msgPrefix, DVD* dvd, int tsi, int pgci, bool menu, int chapter) {
-	if (pgci < 0) {
-		wxString target = menu ? _("menu") : _("title");
-		wxLogError(msgPrefix + _("%d is not valid %s index"), tsi + 1, target.c_str());
-		return false;
-	}
+bool validateJumpTarget(wxString msgPrefix, DVD* dvd, int tsi, int pgci, bool menu, int chapter, wxString entry) {
 	int titlesetCount = dvd->GetTitlesets().GetCount();
 	if (tsi >= titlesetCount) {
 		wxLogError(msgPrefix + _("cannot jump to titleset %d, only %d exist"), tsi + 1, titlesetCount);
@@ -179,18 +198,28 @@ bool validateJumpTarget(wxString msgPrefix, DVD* dvd, int tsi, int pgci, bool me
 	}
 	if (menu) {
 		Menus& menus = tsi < 0 ? dvd->GetVmgm() : dvd->GetTitlesets()[tsi]->GetMenus();
-		int menuCount = menus.Count();
+		int menuCount = menus.Count() > 0 ? menus.Count() : (dvd->IsEmptyMenuEnabled() ? 1 : 0);
 		if (menuCount == 0) {
-			wxLogError(msgPrefix + _("cannot jump to menu; none exist"));
+			if (tsi == -1)
+				wxLogError(msgPrefix + _("cannot jump to vmMenu; none exist"));
+			else
+				wxLogError(msgPrefix + _("cannot jump to menu; none exist"));
 			return false;
-		} else if (pgci >= 119 && pgci < 127) { // menu entry
-			StringSet entrySet = dvd->GetTitlesets()[tsi]->GetMenus().GetEntries();
-			if (pgci >= 121 && entrySet.find(entries[pgci-121]) == entrySet.end()){
-				wxLogError(msgPrefix + _("cannot find menu with entry %s"), entriesTransl[pgci - 121].c_str());
+		} else if (entry.length() > 0) { // menu entry
+			StringSet entrySet = menus.GetEntries();
+			if (entry != wxT("default") && entrySet.find(entry) == entrySet.end()){
+				const wxString entries[6] = {_("title menu"), _("root menu"), _("subtitle menu"), _("audio menu"),
+						_("angle menu"), _("chapter menu")};
+				int entryIdx = s_entries.Index(entry);
+				wxString entryTran = entryIdx >= 0 ? entries[entryIdx] : wxT("menu entry ") + entry;
+				wxLogError(msgPrefix + _("cannot find %s"), entryTran.c_str());
 				return false;
 			}
 		} else if (pgci >= menuCount) {
-			wxLogError(msgPrefix + _("cannot jump to menu %d, only %d exist"), pgci + 1, menuCount);
+			if (tsi == -1)
+				wxLogError(msgPrefix + _("cannot jump to vmMenu %d, only %d exist"), pgci + 1, menuCount);
+			else
+				wxLogError(msgPrefix + _("cannot jump to menu %d, only %d exist"), pgci + 1, menuCount);
 			return false;
 		}
 	} else {
@@ -218,6 +247,19 @@ bool validateJumpTarget(wxString msgPrefix, DVD* dvd, int tsi, int pgci, bool me
 	return true;
 }
 
+wxString GetJumpFormError(bool toMenu, bool fromMenu) {
+	wxString msg;
+	if (toMenu) { // jump to menu from another titleset
+		msg = _("cannot jump to a specific menu in another titleset, only to root/chapter/audio/subtitle/angle menu");
+	} else { // jump to title from another titleset
+		if (fromMenu)
+			msg = _("cannot jump from regular menu to a title in another titleset, use vmMenu instead");
+		else
+			msg = _("cannot jump to a title in another titleset");
+	}
+	return msg;
+}
+
 /** Returns true if action is valid */
 bool DVDAction::IsValid(DVD* dvd, int tsi, int pgci, bool menu, wxString buttonId, bool showMessage, bool showSource,
 		bool skipInvalidTarget) {
@@ -230,17 +272,24 @@ bool DVDAction::IsValid(DVD* dvd, int tsi, int pgci, bool menu, wxString buttonI
 	if (IsCustom()) {
 		res = IsCustomValid(msgPrefix, source, dvd, tsi, pgci, menu, buttonId);
 	} else {
-		if (!menu && ((IsMenu() && GetEntry().length() == 0) || tsi == -1)) {
+		if (m_tsi == tsi)
+			m_tsi = -2; // same titleset
+		if (m_tsi == -2 && m_pgci == pgci && m_menu == menu)
+			m_pgci = -1; // same pgci
+		if (!menu && ((m_menu && m_entry.length() == 0) || tsi == -1)) {
 			wxLogError(msgPrefix + _("cannot jump to a menu from a title, use 'call' instead"));
 			res = false;
-		} else if (GetTsi() >= 0 && ((IsMenu() && GetPgci() > 0 && GetEntry().length() == 0) || (!IsMenu() && !m_vmg))
-				&& !dvd->IsJumppadsEnabled()) { // jump to titleset X and jumppad disabled
-			wxLogError(msgPrefix + _("that form of jumping is not allowed"));
+		} else if (m_tsi >= 0 && ((m_menu && m_pgci > 0 && m_entry.length() == 0) || (!m_menu && tsi != -1))
+				&& !dvd->IsJumppadsEnabled()) { // jump to menu/title from another titleset and jumppad disabled
+			wxLogError(msgPrefix + GetJumpFormError(m_menu, menu));
 			res = false;
+		} else if (tsi == -1 && m_chapter > 0 && !dvd->IsJumppadsEnabled()) {
+			wxString target = m_chapter < 65536 ? _("chapter") : (m_chapter < 131072 ? _("program") : _("cell"));
+			wxLogError(msgPrefix + _("jumping from vmMenu to %s is not allowed"), target.c_str());
+			return false;
 		} else {
 			if (!skipInvalidTarget)
-				res = validateJumpTarget(msgPrefix, dvd, GetTsi() != -2 ? GetTsi() : tsi,
-						GetPgci() != -1 ? GetPgci() : pgci, IsMenu(), GetChapter());
+				res = validateJumpTarget(msgPrefix, dvd, m_tsi != -2 ? m_tsi : tsi, m_pgci, m_menu, m_chapter, m_entry);
 		}
 	}
 	wxLog::EnableLogging(log);
@@ -250,14 +299,27 @@ bool DVDAction::IsValid(DVD* dvd, int tsi, int pgci, bool menu, wxString buttonI
 bool allowallreg = false;
 struct vm_statement* dvd_vm_parsed_cmd = NULL;
 
+wxString s_vmlErrorMsg;
+wxString s_parseErrorMsg;
+
 extern "C" {
 
+/* reports a vml error. */
+void dvdvmlerror(const char *errorMesage, const char *dvdvmtext) {
+	wxString errorMsg(errorMesage, wxConvISO8859_1);
+	wxString dvdvmText(dvdvmtext, wxConvISO8859_1);
+	s_vmlErrorMsg = wxString::Format(errorMsg, dvdvmText.c_str()) + wxT(".");
+}
+
 /* reports a parse error. */
-void dvdvmerror(const char *s) {
+void dvdvmerror(const char *errorMesage) {
     extern char *dvdvmtext;
-    wxString error(s, wxConvISO8859_1);
+    s_parseErrorMsg = wxString(errorMesage, wxConvISO8859_1);
     wxString token(dvdvmtext, wxConvISO8859_1);
-    wxLogError(_("Parse error '%s' on token '%s'"), error.c_str(), token.c_str());
+    if (token.length() > 0)
+    	s_parseErrorMsg += wxT(" (") + wxString::Format(_("on token '%s'"), token.c_str()) + wxT(")");
+    if (s_parseErrorMsg.length())
+    	s_parseErrorMsg = s_parseErrorMsg.Mid(0, 1).Upper() + s_parseErrorMsg.Mid(1) + wxT(".");
 }
 
 } // extern "C"
@@ -269,15 +331,40 @@ typedef enum {
 	VTYPE_VMGM = 2 /* menu in VMG */
 } vtypes;
 
+bool validateTargetNumbers(wxString msgPrefix, int i1, int i2, int i3) {
+	if (!i1 && !i2 && !i3) { //  same VMGM/titleset and same PGC and no chapter/cell/program
+		wxLogError(msgPrefix + _("0 is not valid menu/chapter number"));
+		return false;
+	}
+	if (i2 == 128) {
+		wxLogError(msgPrefix + _("0 is not valid title number"));
+		return false;
+	}
+	if (i3 == 65536) {
+		wxLogError(msgPrefix + _("0 is not valid program number"));
+		return false;
+	}
+	if (i3 == 131072) {
+		wxLogError(msgPrefix + _("0 is not valid cell number"));
+		return false;
+	}
+	return true;
+}
+
 bool validateJump(wxString msgPrefix, DVD* dvd, const struct vm_statement *cs, int tsi, int pgci, vtypes ismenu) {
 	int i1 = cs->i1; // if nonzero, 1 for VMGM, or titleset nr + 1
 	int i2 = cs->i2; // menu number or menu entry ID + 120 or title number + 128
 	int i3 = cs->i3; // chapter number if nonzero and less than 65536 or program number + 65536 or cell number + 131072
 	
+	if (!validateTargetNumbers(msgPrefix, i1, i2, i3)) {
+		return false;
+	}
+		
 	// check for various disallowed combinations
 	if (i1 == 1 && ismenu == VTYPE_VMGM) {
 		i1 = 0; // no need to explicitly specify VMGM
 	}
+	
 	if (((i2 > 0 && i2 < 128) // jump to non-entry menu
 			|| (i2 == 0 && i1 == 1)) // jump to VMGM
 			&& ismenu == VTYPE_VTS) {
@@ -294,23 +381,22 @@ bool validateJump(wxString msgPrefix, DVD* dvd, const struct vm_statement *cs, i
 		wxLogError(msgPrefix + _("cannot omit menu/title if specifying vmgm/titleset"));
 		return false;
 	}
-	if (!i1 && !i2 && !i3) { //  same VMGM/titleset and same PGC and no chapter/cell/program
-		wxLogError(msgPrefix + _("nop jump statement"));
-		return false;
-	}
 	if (i2 == 121 && (i1 >= 2 || (i1 == 0 && ismenu != VTYPE_VMGM))) { //  jump to FPC
 		wxLogError(msgPrefix + _("VMGM must be specified with FPC"));
 		return false;
 	}
 
-	if (i1 >= 2 && i2 >= 120 && i2 < 128) { // jump to titleset menu entry
-		if (!validateJumpTarget(msgPrefix, dvd, i1 - 2, i2 - 1, true, i3 - 1))
+	if (i2 == 121) {  // jump to FPC
+		// ok
+	} else if (i2 >= 120 && i2 < 128) { // jump to menu entry
+		wxString entry = i2 >= 122 && i2 < 128 ? s_entriesArr[i2-122] : wxT("default");
+		if (!validateJumpTarget(msgPrefix, dvd, i1 == 0 ? tsi : i1 - 2, -1, true, i3 - 1, entry))
 			return false;
 	} else if (i1 == 1 && i2 >= 128) { // jump to vmgm and title
 		wxLogError(msgPrefix + _("VMGM does not have titles"));
 		return false;
-	} else if (i1 >= 2 && !dvd->IsJumppadsEnabled()) { // jump to titleset and jumppad disabled 
-		wxLogError(msgPrefix + _("that form of jumping is not allowed"));
+	} else if (i1 >= 2 && !dvd->IsJumppadsEnabled()) { // jump to titleset and jumppad disabled
+		wxLogError(msgPrefix + GetJumpFormError(i2 < 128, ismenu != VTYPE_VTS));
 		return false;
 	} else if (ismenu == VTYPE_VMGM && i2 >= 128 && i3) { // jump from vmgm to title chapter/program/cell
 		if (!dvd->IsJumppadsEnabled()) {
@@ -318,8 +404,6 @@ bool validateJump(wxString msgPrefix, DVD* dvd, const struct vm_statement *cs, i
 			wxLogError(msgPrefix + _("jumping from vmMenu to %s is not allowed"), target.c_str());
 			return false;
 		}
-	} else if (i2 == 121) {  // jump to FPC
-		// ok
 	} else if (!i1 && !i2 && i3) {
 		if (pgci < 0) {
 			wxLogError(msgPrefix + _("cannot jump to a chapter from a FPC"));
@@ -338,18 +422,23 @@ bool validateJump(wxString msgPrefix, DVD* dvd, const struct vm_statement *cs, i
 			}
 		}
 	} else {
-		if (!validateJumpTarget(msgPrefix, dvd, i1 == 0 ? tsi : i1 - 2, (i2 & 127) - 1, i2 < 128, i3 - 1))
+		if (!validateJumpTarget(msgPrefix, dvd, i1 == 0 ? tsi : i1 - 2, (i2 & 127) - 1, i2 < 128, i3 - 1, wxT(""))) {
 			return false;
+		}
 	}
 	return true;
 }
 
 bool validateCall(wxString msgPrefix, DVD* dvd, const struct vm_statement *cs, int tsi, int pgci, vtypes ismenu)  {
-	// cs->i1 if nonzero is 1 for VMGM, or titleset nr + 1
+	int i1 = cs->i1; // if nonzero is 1 for VMGM, or titleset nr + 1
 	int i2 = cs->i2; // menu number or menu entry ID + 120 or title number + 128
-	// cs->i3 is chapter number if nonzero and less than 65536; or program number + 65536; or cell number + 131072
+	int i3 = cs->i3; // chapter number if nonzero and less than 65536 or program number + 65536 or cell number + 131072
 	int i4 = cs->i4; // resume cell if specified, else zero
-
+	
+	if (!validateTargetNumbers(msgPrefix, i1, i2, i3)) {
+		return false;
+	}
+	
 	// CALL's from <post> MUST have a resume cell
 	if (!i4)
 		i4 = 1;
@@ -357,39 +446,29 @@ bool validateCall(wxString msgPrefix, DVD* dvd, const struct vm_statement *cs, i
 		wxLogError(msgPrefix + _("cannot 'call' a menu from another menu, use 'jump' instead"));
 		return false;
 	}
-	if (i2 == 0 || i2 >= 128) // title nr or no menu/title
-	{
+	if (i2 == 0 || i2 >= 128) { // title nr or no menu/title
 		wxLogError(msgPrefix + _("cannot 'call' another title, use 'jump' instead"));
 		return false;
 	}
-	if (cs->i3 != 0) // chapter/cell/program
-	{
+	if (i3 != 0) { // chapter/cell/program
 		wxLogError(msgPrefix + _("cannot 'call' a chapter within a menu"));
 		return false;
 	}
 	if (i2 == 121 // FPC
-			&& (cs->i1 >= 2 /*titleset*/|| (cs->i1 == 0 /*no VMGM/titleset*/&& ismenu != VTYPE_VMGM))) {
+			&& (i1 >= 2 /*titleset*/|| (i1 == 0 /*no VMGM/titleset*/&& ismenu != VTYPE_VMGM))) {
 		wxLogError(msgPrefix + _("VMGM must be specified with FPC"));
 		return false;
 	}
 
-	if (cs->i1 >= 2) { // titleset
-		if (!dvd->IsJumppadsEnabled()) {
-			wxLogError(msgPrefix + _("cannot call to a menu in another titleset"));
-			return false;
-		}
-	} else if (cs->i1 == 0 && i2 < 120) { // non-entry menu in current VMGM/titleset
-		if (!dvd->IsJumppadsEnabled()) {
-			wxLogError(msgPrefix + _("cannot call to a specific menu PGC, only an entry"));
-			return false;
-		}
-	} else if (cs->i1 == 1) { // jump to VMGM
-		// ok
-	} else {
-		// check menu entry
-		StringSet entrySet = dvd->GetTitlesets()[tsi]->GetMenus().GetEntries();
-		if (i2 >= 122 && entrySet.find(entries[i2-122]) == entrySet.end()){
-			wxLogError(msgPrefix + _("cannot find menu with entry %s"), entriesTransl[i2 - 122].c_str());
+	if (i1 >= 2 && !dvd->IsJumppadsEnabled()) { // titleset
+		wxLogError(msgPrefix + _("cannot call to a menu in another titleset"));
+		return false;
+	} else if (i1 == 0 && i2 < 120 && !dvd->IsJumppadsEnabled()) { // non-entry menu in current VMGM/titleset
+		wxLogError(msgPrefix + _("cannot call to a specific menu PGC, only an entry"));
+		return false;
+	} else if (i2 != 121) {
+		wxString entry = i2 >= 122 && i2 < 128 ? s_entriesArr[i2-122] : (i2 == 120 ? wxString(wxT("default")) : wxT(""));
+		if (!validateJumpTarget(msgPrefix, dvd, i1 == 0 ? tsi : i1 - 2, i2 - 1, true, i3 - 1, entry)) {
 			return false;
 		}
 	}
@@ -482,8 +561,15 @@ bool DVDAction::IsCustomValid(wxString msgPrefix, wxString source, DVD* dvd, int
 	const char* const cmd = strdup((const char*)m_custom.ToAscii());
 	dvdvm_buffer_state buf = dvdvm_scan_string(cmd);
 	dvd_vm_parsed_cmd = 0;
+	s_parseErrorMsg = wxT("");
+	s_vmlErrorMsg = wxT("");
 	if (dvdvmparse()) {
-		wxLogError(source + _("invalid action '%s'"), m_custom.c_str());
+		wxString msg = source + _("invalid action '%s'");
+		if (s_vmlErrorMsg.length())
+			msg += wxT(": ") + s_vmlErrorMsg;
+		else if (s_parseErrorMsg)
+			msg += wxT(": ") + s_parseErrorMsg;
+		wxLogError(msg, m_custom.c_str());
 		return false;
 	}
 	dvdvm_delete_buffer(buf);
@@ -501,22 +587,22 @@ wxSvgXmlNode* DVDAction::GetXML(DVDFileType type, DVD* dvd) {
 		if (m_actionType == MENU_ACTION)
 			rootNode->AddProperty(wxT("id"), GetId());
 		if (!GetCustom().length()) {
-			if (GetTsi() > -2)
+			if (GetTsi() >= -1)
 				rootNode->AddProperty(wxT("tsi"), wxString::Format(wxT("%d"), GetTsi()));
-			if (GetPgci() > -1)
+			if (GetEntry().length() > 0)
+				rootNode->AddProperty(wxT("entry"), GetEntry());
+			else if (GetPgci() >= 0)
 				rootNode->AddProperty(wxT("pgci"), wxString::Format(wxT("%d"), (GetPgci()*2 + (IsMenu() ? 0 : 1))));
 			if (GetChapter() > 0)
 				rootNode->AddProperty(wxT("chapter"), wxString::Format(wxT("%d"), GetChapter()));
-			if (GetEntry().length() > 0)
-				rootNode->AddProperty(wxT("entry"), GetEntry());
-			if (IsPlayAll())
-				rootNode->AddProperty(wxT("playAll"), wxT("true"));
-			if (GetAudio() >= 0)
-				rootNode->AddProperty(wxT("audio"), wxString::Format(wxT("%d"), GetAudio()));
-			if (GetSubtitle() >= 0)
-				rootNode->AddProperty(wxT("subtitle"), wxString::Format(wxT("%d"), GetSubtitle()));
 		} else
 			rootNode->AddChild(new wxSvgXmlNode(wxSVGXML_TEXT_NODE, wxEmptyString, GetCustom()));
+		if (IsPlayAll())
+			rootNode->AddProperty(wxT("playAll"), wxT("true"));
+		if (GetAudio() >= 0)
+			rootNode->AddProperty(wxT("audio"), wxString::Format(wxT("%d"), GetAudio()));
+		if (GetSubtitle() >= 0)
+			rootNode->AddProperty(wxT("subtitle"), wxString::Format(wxT("%d"), GetSubtitle()));
 		break;
 	case SPUMUX_XML:
 		rootNode->AddProperty(wxT("name"), GetId());
@@ -550,7 +636,7 @@ bool DVDAction::PutXML(wxSvgXmlNode* node) {
 	if (node->GetPropVal(wxT("chapter"), &val) && val.ToLong(&lval))
 		SetChapter(lval);
 	if (node->GetPropVal(wxT("playAll"), &val))
-		SetPlayAll(val == wxT("true"));
+		SetPlayAll(val == wxT("true") || val == wxT("1"));
 	if (node->GetPropVal(wxT("audio"), &val) && val.ToLong(&lval))
 		SetAudio(lval);
 	if (node->GetPropVal(wxT("subtitle"), &val) && val.ToLong(&lval))
