@@ -3,7 +3,7 @@
 // Purpose:     Main window
 // Author:      Alex Thuering
 // Created:     10.10.2003
-// RCS-ID:      $Id: MainWin.cpp,v 1.137 2010/04/28 07:33:34 ntalex Exp $
+// RCS-ID:      $Id: MainWin.cpp,v 1.143 2010/09/05 10:57:29 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -25,13 +25,14 @@
 #include <wx/filename.h>
 #include <wx/textfile.h>
 #include <wx/artprov.h>
+#include <wx/msgdlg.h>
 #include "math.h"
 
 #ifdef __WXMSW__
 #include <wx/msw/registry.h>
 #endif
 
-#ifdef __WXMSW__
+#if defined(__WXMSW__) || defined(__WXMAC__)
 #include "rc/new.png.h"
 #include "rc/open.png.h"
 #include "rc/save.png.h"
@@ -72,8 +73,10 @@ enum {
 	DIR_TREE_BT_ID = 2250,
 	BACKGROUNS_BT_ID,
 	BUTTONS_BT_ID,
+	DIR_BOOK_ID,
 	THUMBMENU_BACKGROUND_ID,
-	REFRESH_ID
+	REFRESH_ID,
+	DELETE_ID
 };
 
 BEGIN_EVENT_TABLE(MainWin, wxFrame)
@@ -114,9 +117,12 @@ BEGIN_EVENT_TABLE(MainWin, wxFrame)
 	EVT_TOOL(DIR_TREE_BT_ID, MainWin::OnDirTreeBt)
 	EVT_TOOL(BACKGROUNS_BT_ID, MainWin::OnBackgroundsBt)
 	EVT_TOOL(BUTTONS_BT_ID, MainWin::OnButtonsBt)
+	EVT_NOTEBOOK_PAGE_CHANGED(DIR_BOOK_ID, MainWin::OnPageChanged)
 	
 	EVT_MENU(THUMBMENU_BACKGROUND_ID, MainWin::OnSetBackground)
 	EVT_UPDATE_UI(THUMBMENU_BACKGROUND_ID, MainWin::OnUpdateUISetBackground)
+	EVT_MENU(DELETE_ID, MainWin::OnDelete)
+	EVT_UPDATE_UI(DELETE_ID, MainWin::OnDeleteUpdateUI)
 	EVT_MENU(REFRESH_ID, MainWin::OnRefresh)
 	
 	EVT_SIZE(MainWin::OnResize)
@@ -124,12 +130,20 @@ BEGIN_EVENT_TABLE(MainWin, wxFrame)
 	EVT_CLOSE(MainWin::OnClose)
 END_EVENT_TABLE()
 
+/** Constructor */
 MainWin::MainWin(): wxFrame(NULL, -1, _T(""), wxDefaultPosition, wxDefaultSize,
 		wxDEFAULT_FRAME_STYLE), m_dvd(NULL), m_undoPos(0) {
     // begin wxGlade: MainWin::MainWin
     m_splitterTitle = new wxSplitterWindow(this, SPLIITTER_TITLE_ID, wxDefaultPosition, wxDefaultSize, wxCLIP_CHILDREN);
-    m_splitterDir = new wxSplitterWindow(m_splitterTitle, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxCLIP_CHILDREN);
-    m_splitterThumb = new wxSplitterWindow(m_splitterDir, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxCLIP_CHILDREN);
+#ifdef __WXMAC__
+	m_dirBook = new wxNotebook(this, DIR_BOOK_ID, wxDefaultPosition, wxDefaultSize, wxNB_LEFT);
+	m_dirBookPanel = new wxPanel(m_splitterTitle);
+	m_splitterDir = new wxSplitterWindow(m_dirBookPanel, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxCLIP_CHILDREN);
+#else
+	m_dirBar = new wxToolBar(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxNO_BORDER|wxTB_VERTICAL|wxTB_FLAT);
+	m_splitterDir = new wxSplitterWindow(m_splitterTitle, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxCLIP_CHILDREN);
+#endif
+	m_splitterThumb = new wxSplitterWindow(m_splitterDir, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxCLIP_CHILDREN);
     m_thumbPanel = new wxPanel(m_splitterThumb, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL|wxCLIP_CHILDREN);
     m_menubar = new wxMenuBar();
     wxMenu* wxglade_tmp_menu_1 = new wxMenu();
@@ -171,7 +185,6 @@ MainWin::MainWin(): wxFrame(NULL, -1, _T(""), wxDefaultPosition, wxDefaultSize,
     wxglade_tmp_menu_5->Append(wxID_ABOUT, _("&About..."), wxEmptyString, wxITEM_NORMAL);
     m_menubar->Append(wxglade_tmp_menu_5, _("&Help"));
     SetMenuBar(m_menubar);
-    m_dirBar = new wxToolBar(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxNO_BORDER|wxTB_VERTICAL|wxTB_FLAT);
     m_dirTree = new DirCtrl(m_splitterDir, DIR_TREE_ID);
     m_thumbLabel = new wxStaticText(m_thumbPanel, wxID_ANY, _("Directory"), wxDefaultPosition, wxDefaultSize, wxALIGN_CENTRE|wxST_NO_AUTORESIZE);
     m_thumbnails = new wxThumbnails(m_thumbPanel, THUMBNAILS_ID);
@@ -215,7 +228,7 @@ MainWin::MainWin(): wxFrame(NULL, -1, _T(""), wxDefaultPosition, wxDefaultSize,
 	m_lastDir = s_config.GetLastProjectDir();
 	
 	// show Backgrouns
-	m_dirBar->ToggleTool(BACKGROUNS_BT_ID, true);
+	SelectButton(BACKGROUNS_BT_ID);
 	wxCommandEvent evt;
 	OnBackgroundsBt(evt);
 	VideoFormat vf = (VideoFormat) s_config.GetDefVideoFormat();
@@ -237,8 +250,12 @@ MainWin::MainWin(): wxFrame(NULL, -1, _T(""), wxDefaultPosition, wxDefaultSize,
     wxMenu* thumbMenu = new wxMenu;
     thumbMenu->Append(THUMBMENU_BACKGROUND_ID, _("&Assign to background"));
     thumbMenu->AppendSeparator();
-    thumbMenu->Append(REFRESH_ID, _("&Refresh") + wxString(wxT("\tF5")));
-    m_thumbnails->SetPopupMenu(thumbMenu);
+    item = new wxMenuItem(thumbMenu, DELETE_ID, _("&Delete"));
+	item->SetBitmap(wxArtProvider::GetBitmap(wxART_DELETE, wxART_MENU));
+	thumbMenu->Append(item);
+    thumbMenu->AppendSeparator();
+	thumbMenu->Append(REFRESH_ID, _("&Refresh") + wxString(wxT("\tF5")));
+	m_thumbnails->SetPopupMenu(thumbMenu);
 	
     wxMenu* thumbGlobalMenu = new wxMenu;
     thumbGlobalMenu->Append(REFRESH_ID, _("&Refresh") + wxString(wxT("\tF5")));
@@ -265,8 +282,7 @@ void MainWin::set_properties() {
     m_titlesetManager->SetMinSize(wxSize(939, 671));
     // end wxGlade
 	
-    m_dirBar->SetSize(wxSize(40, -1));
-	m_thumbLabel->SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_HIGHLIGHT));
+    m_thumbLabel->SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_HIGHLIGHT));
 #ifndef __WXMSW__
     m_thumbLabel->SetForegroundColour(wxColour(0, 0, 0));
 #endif
@@ -292,6 +308,18 @@ void MainWin::set_properties() {
 	m_thumbnails->EnableDragging();
 	
     // dir bar
+#ifdef __WXMAC__
+	const wxSize imageSize(16, 16);
+	m_dirBookImages = new wxImageList(imageSize.x, imageSize.y);
+	m_dirBookImages->Add(wxArtProvider::GetBitmap(wxART_FOLDER, wxART_TOOLBAR, imageSize)
+						 .ConvertToImage().Rotate90().Rotate90());
+	m_dirBook->SetImageList(m_dirBookImages);
+	m_dirBook->AddPage(new wxPanel(this), _("File browser"), false, 0);
+	m_dirBook->AddPage(new wxPanel(this), _("Backgrounds"));
+	m_dirBook->AddPage(new wxPanel(this), _("Buttons"));
+	m_dirBook->SetMinSize(wxSize(m_dirBook->GetSize().x - 10, -1));
+#else
+	m_dirBar->SetSize(wxSize(40, 40));
     m_dirBar->SetMargins(4, 4);
     wxVerticalToolbar* vertToolBar = new wxVerticalToolbar(m_dirBar);
     vertToolBar->AddTool(DIR_TREE_BT_ID, _("File browser"),
@@ -300,6 +328,7 @@ void MainWin::set_properties() {
 	vertToolBar->AddTool(BUTTONS_BT_ID, _("Buttons"), wxNullBitmap, wxITEM_RADIO);
     vertToolBar->Update();
     m_dirBar->Realize();
+#endif
 }
 
 
@@ -307,13 +336,21 @@ void MainWin::do_layout() {
     // begin wxGlade: MainWin::do_layout
     wxBoxSizer* mainWinSizer = new wxBoxSizer(wxHORIZONTAL);
     wxBoxSizer* thumbSizer = new wxBoxSizer(wxVERTICAL);
-    mainWinSizer->Add(m_dirBar, 0, wxEXPAND, 0);
     thumbSizer->Add(m_thumbLabel, 0, wxEXPAND|wxALIGN_CENTER_HORIZONTAL, 0);
     thumbSizer->Add(m_thumbnails, 1, wxEXPAND, 0);
     m_thumbPanel->SetSizer(thumbSizer);
     m_splitterThumb->SplitVertically(m_thumbPanel, m_menuEditor, 10);
     m_splitterDir->SplitVertically(m_dirTree, m_splitterThumb, 10);
+#ifdef __WXMAC__
+	wxBoxSizer* dirBookSizer = new wxBoxSizer(wxHORIZONTAL);
+	dirBookSizer->Add(m_dirBook, 0, wxEXPAND, 0);
+	dirBookSizer->Add(m_splitterDir, 1, wxEXPAND, 0);
+	m_dirBookPanel->SetSizer(dirBookSizer);
+	m_splitterTitle->SplitHorizontally(m_dirBookPanel, m_titlesetManager, 10);
+#else	
     m_splitterTitle->SplitHorizontally(m_splitterDir, m_titlesetManager, 10);
+    mainWinSizer->Add(m_dirBar, 0, wxEXPAND, 0);
+#endif
     mainWinSizer->Add(m_splitterTitle, 1, wxEXPAND, 0);
     SetSizer(mainWinSizer);
     Layout();
@@ -333,7 +370,9 @@ void MainWin::do_layout() {
 }
 
 void MainWin::ShowDirTree(bool show) {
+#ifndef __WXMAC__
 	m_dirBar->ToggleTool(DIR_TREE_BT_ID, show);
+#endif
 	if (show && !m_splitterDir->IsSplit()) {
 		m_splitterDir->SplitVertically(m_dirTree, m_splitterThumb, m_splitterDirSash);
 		m_dirTree->Show(true);
@@ -407,6 +446,41 @@ void MainWin::OnButtonsBt(wxCommandEvent& event) {
 	m_thumbnails->SetCaption(_("Buttons"));
 }
 
+void MainWin::OnPageChanged(wxNotebookEvent& event) {
+	wxCommandEvent evt;
+	switch (event.GetSelection()) {
+		case 0:
+			OnDirTreeBt(evt);
+			break;
+		case 1:
+			OnBackgroundsBt(evt);
+			break;
+		case 2:
+			OnButtonsBt(evt);
+			break;
+		default:
+			break;
+	}
+}
+
+bool MainWin::IsButtonSelected(int btId) {
+#ifdef __WXMAC__
+	return (m_dirBook->GetSelection() == 0 && btId == DIR_TREE_BT_ID)
+			|| (m_dirBook->GetSelection() == 1 && btId == BACKGROUNS_BT_ID)
+			|| (m_dirBook->GetSelection() == 2 && btId == BUTTONS_BT_ID);
+#else
+	return m_dirBar->GetToolState(btId);
+#endif
+}
+
+void MainWin::SelectButton(int btId) {
+#ifdef __WXMAC__
+	m_dirBook->SetSelection(btId == DIR_TREE_BT_ID ? 0 : (btId == BACKGROUNS_BT_ID ? 1 : 2));
+#else
+	m_dirBar->ToggleTool(btId, true);
+#endif
+}
+
 void MainWin::OnDirSelected(wxTreeEvent& event) {
 	if (!m_dvd)
 		return;
@@ -418,20 +492,24 @@ void MainWin::OnDirSelected(wxTreeEvent& event) {
 void MainWin::OnThumbInfo(wxCommandEvent& event) {
 	wxString text;
 	if (m_thumbnails->GetPointed() != -1) {
-		if (m_dirBar->GetToolState(BACKGROUNS_BT_ID)) {
+		if (IsButtonSelected(BACKGROUNS_BT_ID)) {
 			text = _("Double click to set the background.");
-		} else if (m_dirBar->GetToolState(BUTTONS_BT_ID)) {
+		} else if (IsButtonSelected(BUTTONS_BT_ID)) {
 			text = m_thumbnails->GetPointedItem()->GetFilename().AfterLast(wxFILE_SEP_PATH);
 		} else {
-			double size = wxFile(m_thumbnails->GetPointedItem()->GetFilename()).Length() / 1024;
-			if (size < 1024)
-				text.Printf(_("%s (%.1f KB)"),m_thumbnails->GetPointedItem()->GetFilename().c_str(), size);
-			else if (size < 1024 * 1024)
-				text.Printf(_("%s (%.1f MB)"),m_thumbnails->GetPointedItem()->GetFilename().c_str(),
-						size / 1024);
-			else
-				text.Printf(_("%s (%.1f GB)"),m_thumbnails->GetPointedItem()->GetFilename().c_str(),
-						size / 1024 / 1024);
+			if (wxFile::Exists(m_thumbnails->GetPointedItem()->GetFilename())) {
+				double size = wxFile(m_thumbnails->GetPointedItem()->GetFilename()).Length() / 1024;
+				if (size < 1024)
+					text.Printf(_("%s (%.1f KB)"),m_thumbnails->GetPointedItem()->GetFilename().c_str(), size);
+				else if (size < 1024 * 1024)
+					text.Printf(_("%s (%.1f MB)"),m_thumbnails->GetPointedItem()->GetFilename().c_str(),
+							size / 1024);
+				else
+					text.Printf(_("%s (%.1f GB)"),m_thumbnails->GetPointedItem()->GetFilename().c_str(),
+							size / 1024 / 1024);
+			} else {
+				m_thumbnails->UpdateItems();
+			}
 		}
 	}
 	m_statusbar->SetStatusText(text, 0);
@@ -487,10 +565,6 @@ void MainWin::OnSetBackground(wxCommandEvent& event) {
 	m_menuEditor->SetBackground(filename);
 }
 
-void MainWin::OnRefresh(wxCommandEvent& event) {
-	m_thumbnails->UpdateItems();
-}
-
 void MainWin::OnUpdateUISetBackground(wxUpdateUIEvent& event) {
 	if (m_thumbnails->GetSelected() == -1)
 		event.Enable(false);
@@ -501,6 +575,27 @@ void MainWin::OnUpdateUISetBackground(wxUpdateUIEvent& event) {
 	}
 }
 
+void MainWin::OnRefresh(wxCommandEvent& event) {
+	m_thumbnails->UpdateItems();
+}
+
+void MainWin::OnDelete(wxCommandEvent& event) {
+	wxString filename = m_thumbnails->GetSelectedItem()->GetFilename();
+	if (!wxFile::Exists(filename))
+		return;
+	wxMessageDialog msgDlg(this,
+			wxString::Format(_("Are you sure you want to delete '%s' from the file system?"), filename.c_str()),
+			wxT("DVDStyler"), wxYES_NO|wxICON_QUESTION);
+	if (msgDlg.ShowModal() != wxID_YES)
+		return;
+	wxRemoveFile(filename);
+	m_thumbnails->UpdateItems();
+}
+
+void MainWin::OnDeleteUpdateUI(wxUpdateUIEvent& event) {
+	event.Enable(IsButtonSelected(DIR_TREE_BT_ID) && m_thumbnails->GetSelected() >= 0);
+}
+
 void MainWin::OnSplitterTitleSash(wxSplitterEvent &event) {
 	int width, height;
 	GetClientSize(&width, &height);
@@ -526,6 +621,9 @@ void MainWin::OnClose(wxCloseEvent &event) {
 
 void MainWin::NewDVD(wxString templateFile, wxString discTitle, wxString discLabel, DiscCapacity capacity,
 		int videoBitrate, VideoFormat videoFormat, AudioFormat audioFormat, AspectRatio aspectRatio) {
+	if (aspectRatio == ar16_9 && templateFile.Length()
+			&& wxFile::Exists(templateFile.substr(0, templateFile.length()-5) + wxT("WS.dvdt")))
+		templateFile = templateFile.substr(0, templateFile.length()-5) + wxT("WS.dvdt");
 	if (templateFile.Length() && Open(templateFile)) {
 		m_dvd->SetLabel(discLabel);
 		m_dvd->SetCapacity(capacity);
@@ -535,11 +633,13 @@ void MainWin::NewDVD(wxString templateFile, wxString discTitle, wxString discLab
 		m_dvd->SetAudioFormat(audioFormat);
 		m_dvd->SetAspectRatio(aspectRatio);
 		for (int tsi = -1; tsi < (int) m_dvd->GetTitlesets().GetCount(); tsi++) {
-			unsigned int mCount = m_dvd->GetPgcArray(tsi, true).GetCount();
-			for (unsigned int mi = 0; mi < mCount; mi++) {
-				Menu* menu = m_dvd->GetMenu(tsi, mi);
+			PgcArray& menus = m_dvd->GetPgcArray(tsi, true);
+			menus.GetVideo().SetAspect(aspectRatio);
+			for (unsigned int mi = 0; mi < menus.GetCount(); mi++) {
+				Menu* menu = menus[mi]->GetMenu();
 				if (menu) {
 					menu->SetVideoFormat(videoFormat);
+					menu->SetAspectRatio(aspectRatio);
 					for (unsigned int obji = 0; obji < menu->GetObjectsCount(); obji++) {
 						MenuObject* obj = menu->GetObject(obji);
 						MenuObjectParam* initParam = obj->GetInitParam();
@@ -633,10 +733,13 @@ void MainWin::SetChanged(bool changed, bool doUndo) {
 
 void MainWin::OnChangedMenu(wxCommandEvent& event) {
 	SetChanged(true);
-	if (m_titlesetManager->GetSelectedMenu() < 0)
-		return;
-	m_titlesetManager->GetItem(m_titlesetManager->GetSelectedMenu())->Update();
-	m_titlesetManager->Refresh();
+	bool updateAllItems = (bool) event.GetInt();
+	if (updateAllItems) {
+		m_titlesetManager->UpdateItems();
+	} else if (m_titlesetManager->GetSelectedMenu() >= 0) {
+		m_titlesetManager->GetItem(m_titlesetManager->GetSelectedMenu())->Update();
+		m_titlesetManager->Refresh();
+	}
 }
 
 void MainWin::OnSelectMenu(wxCommandEvent& event) {
@@ -778,6 +881,9 @@ please drag a video file into the bottom section of the window."), emptyTsi+1),
 		_("Burn"), wxOK|wxICON_ERROR, this);
 		return;
 	}
+	// check if all menus are ok
+	if (!m_dvd->CheckMenus())
+		return;
 	// check if all actions are valid
 	if (!m_dvd->CheckActions(m_dvd->HasTemplate()))
 		return;
