@@ -3,7 +3,7 @@
 // Purpose:     The class to store a DVD Menu
 // Author:      Alex Thuering
 // Created:	04.11.2003
-// RCS-ID:      $Id: Menu.cpp,v 1.83 2010/05/03 06:03:20 ntalex Exp $
+// RCS-ID:      $Id: Menu.cpp,v 1.94 2010/09/05 10:53:06 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -12,6 +12,7 @@
 #include "MenuPalettes.h"
 #include "Palette3D.h"
 #include "DVD.h"
+#include "Config.h"
 #include <wxVillaLib/SConv.h>
 #include <wxVillaLib/utils.h>
 #include <wxSVG/svg.h>
@@ -30,12 +31,14 @@
 #define wxBITMAP_TYPE_ANY -1
 #endif
 
-Menu::Menu(DVD* dvd, int tsi, VideoFormat videoFormat, AspectRatio aspectRatio) {
+Menu::Menu(DVD* dvd, int tsi, int pgci, VideoFormat videoFormat, AspectRatio aspectRatio) {
 	m_dvd = dvd;
 	m_tsi = tsi;
+	m_pgci = pgci;
 	m_videoFormat = videoFormat;
 	m_aspectRatio = aspectRatio;
 	m_startTime = _T("00:00:00.00");
+	m_rememberLastButton = false;
 	m_transpColour = wxColour(8, 8, 8);
 	m_svg = CreateSVG(GetResolution().GetWidth(), GetResolution().GetHeight());
 	SetBackgroundColour(wxColour(0, 0, 0));
@@ -64,9 +67,14 @@ wxSVGDocument* Menu::CreateSVG(int width, int height) {
 	return svg;
 }
 
-void Menu::SetVideoFormat(VideoFormat format) {
-	VideoFormat oldFormat = m_videoFormat;
-	m_videoFormat = format;
+void Menu::SetVideoFormat(VideoFormat videoFormat) {
+	m_videoFormat = videoFormat;
+}
+
+void Menu::SetAspectRatio(AspectRatio aspectRatio) {
+	if (m_aspectRatio == aspectRatio)
+		return;
+	m_aspectRatio = aspectRatio;
 	// svg root element
 	m_svg->GetRootElement()->SetWidth(GetResolution().GetWidth());
 	m_svg->GetRootElement()->SetHeight(GetResolution().GetHeight());
@@ -81,21 +89,33 @@ void Menu::SetVideoFormat(VideoFormat format) {
 			((wxSVGImageElement*) bgElement)->SetHeight(GetResolution().GetHeight());
 		}
 	}
-	// change object coordinates
-	if (oldFormat != m_videoFormat) {
-		double f = m_videoFormat == vfNTSC ? (double)480/576 : (double)576/480;
-		for (unsigned int obji = 0; obji < m_objects.GetCount(); obji++) {
-			MenuObject* obj = m_objects[obji];
-			obj->SetY((int)(f*obj->GetY()));
-			if (!obj->IsDefaultSize()) {
-				obj->SetHeight((int)(f*obj->GetHeight()));
-			}
+	// change object coordinates and size
+	double f = m_aspectRatio == ar16_9 ? (double) 3/4 : (double) 4/3;
+	for (unsigned int obji = 0; obji < m_objects.GetCount(); obji++) {
+		MenuObject* obj = m_objects[obji];
+		// update font size
+		wxFont font = obj->GetParamFont(wxT("text"));
+		font.SetPointSize(round(font.GetPointSize()*f));
+		obj->SetParamFont(wxT("text"), font);
+		// update object Y-coordinate and size
+		obj->SetY(round(obj->GetY()*f));
+		if (!obj->IsDefaultSize() || obj->GetObjectParam(wxT("text")) == NULL) {
+			obj->SetDefaultSize(false);
+			obj->SetWidth(round(obj->GetWidth()*f));
+			obj->SetHeight(round(obj->GetHeight()*f));
 		}
+		obj->UpdateSize();
 	}
 	FixButtonCoordinates();
 }
 
 wxSize Menu::GetResolution() {
+	if (m_aspectRatio == ar16_9) // 16:9
+		return wxSize(720, 405);
+	return wxSize(720, 540); // 4:3
+}
+
+wxSize Menu::GetFrameResolution() {
 	if (m_videoFormat == vfNTSC)
 		return wxSize(720, 480);
 	return wxSize(720, 576); // vfPAL
@@ -177,10 +197,10 @@ bool Menu::HasVideoBackground() {
 void Menu::SetTransparentColor() {
 	m_transpColour = wxColour(0, 0, 0);
 	wxImageHistogram h1, h2, h3;
-	wxImage* images = GetImages();
-	images[1].ComputeHistogram(h1);
-	images[2].ComputeHistogram(h2);
-	images[3].ComputeHistogram(h3);
+	wxImage* images = GetSubPictures(ssmNORMAL);
+	images[0].ComputeHistogram(h1);
+	images[1].ComputeHistogram(h2);
+	images[2].ComputeHistogram(h3);
 	for (int i = 1; i < 30; i++) {
 		m_transpColour = wxColour(i * 8, i * 8, i * 8);
 		unsigned long colour = (i * 8 << 16) + (i * 8 << 8) + i * 8;
@@ -297,16 +317,16 @@ wxSVGSVGElement* Menu::GetSVGCopy(MenuDrawType drawType) {
 	elem = svgNode->GetElementById(wxT("grid"));
 	if (elem)
 		elem->GetParent()->RemoveChild(elem);
-
+	
 	if (drawType == mdBACKGROUND) {
 		// remove changeable elements of objects
 		for (unsigned int i = 0; i < GetObjectsCount(); i++) {
 			MenuObject* obj = GetObject(i);
 			if (obj->IsButton()) {
-				if (obj->GetAction().IsValid(m_dvd, m_tsi, 0, true)) {
+				if (obj->GetAction().IsValid(m_dvd, m_tsi, m_pgci, true)) {
 					// remove dynamic part of button
 					wxSVGElement* elem = (wxSVGElement*) svgNode->GetElementById(wxT("s_") + obj->GetId());
-					if (elem)
+					if (elem && !s_config.GetDrawButtonsOnBackground())
 						RemoveChangeable(elem, obj);
 				} else {
 					// remove buttons with invalid actions
@@ -336,7 +356,8 @@ wxSVGSVGElement* Menu::GetSVGCopy(MenuDrawType drawType) {
 				m_transpColour.Green(), m_transpColour.Blue()));
 		for (unsigned int i = 0; i < GetObjectsCount(); i++) {
 			MenuObject* obj = GetObject(i);
-			if (obj->IsButton() && obj->GetAction().IsValid(m_dvd, m_tsi, 0, true)) {
+			if (obj->IsButton() && obj->GetAction().IsValid(m_dvd, m_tsi, m_pgci, true)
+					&& (!s_config.GetDrawButtonsOnBackground() || drawType != mdBUTTONS_NORMAL)) {
 				wxSVGSVGElement* symbol = (wxSVGSVGElement*)svgNode->GetElementById(wxT("s_") + obj->GetId());
 				if (symbol)
 					RemoveNotChangeable(symbol, obj);
@@ -367,20 +388,21 @@ wxSVGSVGElement* Menu::GetSVGCopy(MenuDrawType drawType) {
 
 ////////////////////////////// Render ////////////////////////////////////////
 
-wxImage Menu::RenderImage(MenuDrawType drawType, int width, int height) {
+wxImage Menu::RenderImage(MenuDrawType drawType, int width, int height, const wxSVGRect* rect,
+		bool preserveAspectRatio) {
 	wxSVGDocument svg;
 	svg.AppendChild(GetSVGCopy(drawType));
-	return svg.Render(width, height);
+	return svg.Render(width, height, rect, preserveAspectRatio);
 }
 
 wxImage Menu::GetImage(int width, int height) {
-	return RenderImage(mdALL, width, height);
+	return RenderImage(mdALL, width, height, NULL, true);
 }
 
-wxSVGDocument* Menu::GetBackgroundSVG() {
-	wxSVGDocument* svg = new wxSVGDocument;
-	svg->AppendChild(GetSVGCopy(mdBACKGROUND));
-	return svg;
+wxImage Menu::GetBackgroundImage() {
+	int width = GetFrameResolution().GetWidth();
+	int height = GetFrameResolution().GetHeight();
+	return RenderImage(mdBACKGROUND, width, height);
 }
 
 bool Menu::ReduceColours() {
@@ -393,7 +415,8 @@ bool Menu::ReduceColours() {
 		for (int j = 0; j < obj.GetObjectParamsCount(); j++) {
 			MenuObjectParam* param = obj.GetObjectParam(j);
 			if (param->changeable) {
-				palette.Add(param->normalColour, param->highlightedColour, param->selectedColour);
+				palette.Add(s_config.GetDrawButtonsOnBackground() ? wxColour() : param->normalColour,
+						param->highlightedColour, param->selectedColour);
 			}
 		}
 	}
@@ -410,8 +433,11 @@ bool Menu::ReduceColours() {
 		for (int i = 0; i < obj.GetObjectParamsCount(); i++) {
 			MenuObjectParam* param = obj.GetObjectParam(i);
 			if (param->changeable) {
-				if (palette.Apply(param->normalColour, param->highlightedColour, param->selectedColour)) {
-					obj.SetParamColour(param->name, param->normalColour, mbsNORMAL);
+				wxColour tmp;
+				if (palette.Apply(s_config.GetDrawButtonsOnBackground() ? tmp : param->normalColour,
+						param->highlightedColour, param->selectedColour)) {
+					if (!s_config.GetDrawButtonsOnBackground()) 
+						obj.SetParamColour(param->name, param->normalColour, mbsNORMAL);
 					obj.SetParamColour(param->name, param->highlightedColour, mbsHIGHLIGHTED);
 					obj.SetParamColour(param->name, param->selectedColour, mbsSELECTED);
 				}
@@ -421,15 +447,27 @@ bool Menu::ReduceColours() {
 	return true;
 }
 
-wxImage* Menu::GetImages() {
+/** Returns subPictures */
+wxImage* Menu::GetSubPictures(SubStreamMode mode) {
 	ReduceColours();
 
 	// render images
-	wxImage* images = new wxImage[4];
-	images[0] = RenderImage(mdBACKGROUND, -1, -1);
-	images[1] = RenderImage(mdBUTTONS_NORMAL, -1, -1);
-	images[2] = RenderImage(mdBUTTONS_HIGHLIGHTED, -1, -1);
-	images[3] = RenderImage(mdBUTTONS_SELECTED, -1, -1);
+	int width = GetFrameResolution().GetWidth();
+	int height = GetFrameResolution().GetHeight();
+	int padY = 0;
+	wxSVGRect* cropRect = NULL;
+	if (mode == ssmLETTERBOX) {
+		padY = height*0.125;
+		height *= 0.75;
+	} else if (mode == ssmPANSCAN) {
+		wxSize res = GetResolution();
+		cropRect = new wxSVGRect(res.GetWidth()*0.125, 0, res.GetWidth()*0.75, res.GetHeight());
+		width *= 4.0/3; // increase width to get same width as without crop rect
+	}
+	wxImage* images = new wxImage[3];
+	images[0] = RenderImage(mdBUTTONS_NORMAL, width, height, cropRect);
+	images[1] = RenderImage(mdBUTTONS_HIGHLIGHTED, width, height, cropRect);
+	images[2] = RenderImage(mdBUTTONS_SELECTED, width, height, cropRect);
 
 	// make aliasing for buttons
 	for (unsigned int i = 0; i < GetObjectsCount(); i++) {
@@ -437,14 +475,15 @@ wxImage* Menu::GetImages() {
 		if (!obj.IsButton())
 			continue;
 		// make palette
-		MenuPalettes objPalette(obj, m_transpColour);
+		MenuPalettes objPalette(obj, m_transpColour, s_config.GetDrawButtonsOnBackground());
 
 		// apply palette
-		unsigned char* img1 = images[1].GetData() + obj.GetY() * images[1].GetWidth() * 3 + obj.GetX() * 3;
-		unsigned char* img2 = images[2].GetData() + obj.GetY() * images[2].GetWidth() * 3 + obj.GetX() * 3;
-		unsigned char* img3 = images[3].GetData() + obj.GetY() * images[3].GetWidth() * 3 + obj.GetX() * 3;
-		for (int y = 0; y < obj.GetHeight(); y++) {
-			for (int x = 0; x < obj.GetWidth(); x++) {
+		wxRect rect = obj.GetFrameBBox(mode, true);
+		unsigned char* img1 = images[0].GetData() + rect.GetY() * images[0].GetWidth() * 3 + rect.GetX() * 3;
+		unsigned char* img2 = images[1].GetData() + rect.GetY() * images[1].GetWidth() * 3 + rect.GetX() * 3;
+		unsigned char* img3 = images[2].GetData() + rect.GetY() * images[2].GetWidth() * 3 + rect.GetX() * 3;
+		for (int y = 0; y < rect.GetHeight(); y++) {
+			for (int x = 0; x < rect.GetWidth(); x++) {
 				objPalette.Apply(img1, img1);
 				objPalette.Apply(img1, img2, img2);
 				objPalette.Apply(img1, img2, img3, img3);
@@ -452,25 +491,27 @@ wxImage* Menu::GetImages() {
 				img2 += 3;
 				img3 += 3;
 			}
-			img1 += (images[1].GetWidth() - obj.GetWidth()) * 3;
-			img2 += (images[2].GetWidth() - obj.GetWidth()) * 3;
-			img3 += (images[3].GetWidth() - obj.GetWidth()) * 3;
+			img1 += (images[0].GetWidth() - rect.GetWidth()) * 3;
+			img2 += (images[1].GetWidth() - rect.GetWidth()) * 3;
+			img3 += (images[2].GetWidth() - rect.GetWidth()) * 3;
 		}
 	}
 
 	// all pixels that don't belong to objects must be transparent
-	unsigned char* img1 = images[1].GetData();
-	unsigned char* img2 = images[2].GetData();
-	unsigned char* img3 = images[3].GetData();
-	for (int y = 0; y < images[1].GetHeight(); y++) {
-		for (int x = 0; x < images[1].GetWidth(); x++) {
+	unsigned char* img1 = images[0].GetData();
+	unsigned char* img2 = images[1].GetData();
+	unsigned char* img3 = images[2].GetData();
+	for (int y = 0; y < images[0].GetHeight(); y++) {
+		for (int x = 0; x < images[0].GetWidth(); x++) {
 			bool found = false;
 			for (unsigned int i = 0; i < GetObjectsCount(); i++) {
 				MenuObject& obj = *GetObject(i);
 				if (!obj.IsButton())
 					continue;
-				if (x >= obj.GetX() && x < obj.GetX() + obj.GetWidth()
-						&& y >= obj.GetY() && y < obj.GetY() + obj.GetHeight()) {
+				
+				wxRect rect = obj.GetFrameBBox(mode, true);
+				if (x >= rect.GetX() && x < rect.GetX() + rect.GetWidth()
+						&& y >= rect.GetY() && y < rect.GetY() + rect.GetHeight()) {
 					found = true;
 					break;
 				}
@@ -491,10 +532,26 @@ wxImage* Menu::GetImages() {
 			img3 += 3;
 		}
 	}
+	
+	if (mode == ssmLETTERBOX) {
+		for (int i = 0; i < 3; i++) {
+			wxSize size(images[i].GetWidth(), images[i].GetHeight() + 2*padY);
+			wxPoint pos(0, padY);
+			images[i].Resize(size, pos, m_transpColour.Red(), m_transpColour.Green(), m_transpColour.Blue());
+		}
+	}
 
 	return images;
 }
 
+
+wxSVGDocument* Menu::GetBackgroundSVG() {
+	wxSVGDocument* svg = new wxSVGDocument;
+	svg->AppendChild(GetSVGCopy(mdBACKGROUND));
+	return svg;
+}
+
+/** Returns the count of buttons */
 unsigned int Menu::GetButtonsCount() {
 	int count = 0;
 	for (int i=0; i<(int)m_objects.Count(); i++)
@@ -503,13 +560,21 @@ unsigned int Menu::GetButtonsCount() {
 	return count;
 }
 
+/** Returns menu object with given id */
 MenuObject* Menu::GetObject(wxString id) {
-	for (int i=0; i<(int)m_objects.Count(); i++)
-		if (m_objects[i]->GetId() == id)
-			return m_objects[i];
-	return NULL;
+	int idx = GetIndexOfObject(id);
+	return idx >= 0 ? m_objects[idx] : NULL;
 }
 
+/** Returns index of menu object with given id */
+int Menu::GetIndexOfObject(wxString id) {
+	for (int i = 0; i < (int) m_objects.Count(); i++)
+		if (m_objects[i]->GetId() == id)
+			return i;
+	return -1;
+}
+
+/** Removes menu object with given id */
 void Menu::RemoveObject(wxString id) {
 	MenuObject* obj = GetObject(id);
 	if (obj) {
@@ -548,12 +613,26 @@ wxString Menu::AddObject(wxString fileName, wxString param, int x, int y) {
 	return obj->GetId();
 }
 
-/** Updates image in buttons with given jump action */
-void Menu::UpdateButtonImageFor(int actionTsi, int actionPgci, DVD* dvd) {
+/** Returns button with given jump action */
+MenuObject* Menu::GetButtonByJumpAction(int actionTsi, int actionPgci) {
 	for (unsigned int obji = 0; obji < m_objects.GetCount(); obji++) {
 		MenuObject* obj = m_objects[obji];
-		if (obj->IsButton() && !obj->GetAction().IsCustom()
-				&& obj->GetAction().GetTsi() == actionTsi && obj->GetAction().GetPgci() == actionPgci) {
+		if (obj->IsButton() && !obj->GetAction().IsCustom() && obj->GetAction().GetTsi() == actionTsi
+				&& obj->GetAction().GetPgci() == actionPgci && !obj->GetAction().IsMenu()) {
+			return obj;
+		}
+	}
+	return NULL;
+}
+
+/** Updates image in buttons with given jump action */
+bool Menu::UpdateButtonImageFor(int actionTsi, int actionPgci, DVD* dvd) {
+	bool result = false;
+	for (unsigned int obji = 0; obji < m_objects.GetCount(); obji++) {
+		MenuObject* obj = m_objects[obji];
+		if (obj->IsButton() && !obj->GetAction().IsCustom() && obj->IsDisplayVideoFrame()
+				&& obj->GetAction().GetTsi() == actionTsi && obj->GetAction().GetPgci() == actionPgci
+				&& !obj->GetAction().IsMenu()) {
 			for (int i = 0; i < obj->GetObjectParamsCount(); i++) {
 				MenuObjectParam* param = obj->GetObjectParam(i);
 				if (param->type == _T("image")) {
@@ -561,11 +640,17 @@ void Menu::UpdateButtonImageFor(int actionTsi, int actionPgci, DVD* dvd) {
 					wxString uri;
 					if (actionPgci < (int)pgcs.size() && pgcs[actionPgci]->GetVobs().size() > 0)
 						uri = pgcs[actionPgci]->GetVobs()[0]->GetVideoFrameUri(obj->GetAction().GetChapter());
-					obj->SetParam(param->name, uri);
+					wxString uriFname = uri.Find(wxT('#')) >= 0 ? uri.BeforeLast(wxT('#')) : uri;
+					if (!obj->IsCustomVideoFrame() || uri.length() == 0
+							|| uriFname != obj->GetParam(param->name).BeforeLast(wxT('#'))) {
+						obj->SetParam(param->name, uri);
+						result = true;
+					}
 				}
 			}
 		}
 	}
+	return result;
 }
 
 /** Fix coordinates of buttons if they are out of range */
@@ -585,7 +670,7 @@ void Menu::FixButtonCoordinates() {
 	}
 }
 
-bool Menu::SaveSpumux(wxString fileName, wxString btFile, wxString hlFile, wxString selFile) {
+bool Menu::SaveSpumux(wxString fileName, SubStreamMode mode, wxString btFile, wxString hlFile, wxString selFile) {
 	wxSvgXmlDocument xml;
 	wxSvgXmlNode* root = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, _T("subpictures"));
 	wxSvgXmlNode* streamNode = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, _T("stream"));
@@ -603,8 +688,8 @@ bool Menu::SaveSpumux(wxString fileName, wxString btFile, wxString hlFile, wxStr
 	for (int i=0; i<(int)GetObjectsCount(); i++)
 	{
 		MenuObject* obj = GetObject(i);
-		if (obj->IsButton() && obj->GetAction().IsValid(m_dvd, m_tsi, 0, true, obj->GetId(), false))
-			spuNode->AddChild(obj->GetXML(SPUMUX_XML, false));
+		if (obj->IsButton() && obj->GetAction().IsValid(m_dvd, m_tsi, m_pgci, true, obj->GetId(), false))
+			spuNode->AddChild(obj->GetXML(SPUMUX_XML, m_dvd, mode));
 	}
 	// actions
 	for (unsigned int i=0; i<m_actions.Count(); i++)
@@ -673,8 +758,8 @@ wxSvgXmlNode* Menu::GetXML(DVDFileType type, wxSvgXmlNode* node) {
 	// add buttons info (action, etc.)
 	for (unsigned int i=0; i<GetObjectsCount(); i++) {
 		MenuObject* obj = GetObject(i);
-		if (type == DVDSTYLER_XML || (obj->IsButton() && obj->GetAction().IsValid(m_dvd, m_tsi, 0, true)))
-			node->AddChild(obj->GetXML(type, m_dvd, false, m_dvd->GetPlayAllRegister()));
+		if (type == DVDSTYLER_XML || (obj->IsButton() && obj->GetAction().IsValid(m_dvd, m_tsi, m_pgci, true)))
+			node->AddChild(obj->GetXML(type, m_dvd, ssmNORMAL, false));
 	}
 	
 	// actions
@@ -759,6 +844,7 @@ bool Menu::PutXML(wxSvgXmlNode* node) {
 		for (wxSvgXmlNode* child = node->GetChildren(); child != NULL; child = child->GetNext())
 			if (child->GetType() == wxSVGXML_ELEMENT_NODE)
 				AddObject(child);
+		
 		// fix for old version (<=1.5b5)
 		if (m_svg->GetElementById(wxT("objects")) == NULL) {
 		  	wxSVGGElement* objectsElem = new wxSVGGElement;
@@ -778,6 +864,35 @@ bool Menu::PutXML(wxSvgXmlNode* node) {
 		  			objectsElem->AppendChild(useElem);
 	  		}
 		}
+		
+		// fix resolution (DVDStyler <= v1.8.1)
+		if (m_svg->GetRootElement()->GetHeight().GetBaseVal() != GetResolution().GetHeight()) {
+			double f = (double) GetResolution().GetHeight() / m_svg->GetRootElement()->GetHeight().GetBaseVal();
+			// svg root element
+			m_svg->GetRootElement()->SetWidth(GetResolution().GetWidth());
+			m_svg->GetRootElement()->SetHeight(GetResolution().GetHeight());
+			// background
+			wxSVGElement* bgElement = m_svg->GetElementById(wxT("background"));
+			if (bgElement) {
+				if (bgElement->GetDtd() == wxSVG_RECT_ELEMENT) {
+					((wxSVGRectElement*) bgElement)->SetWidth(GetResolution().GetWidth());
+					((wxSVGRectElement*) bgElement)->SetHeight(GetResolution().GetHeight());
+				} else if (bgElement->GetDtd() == wxSVG_IMAGE_ELEMENT) {
+					((wxSVGImageElement*) bgElement)->SetWidth(GetResolution().GetWidth());
+					((wxSVGImageElement*) bgElement)->SetHeight(GetResolution().GetHeight());
+				}
+			}
+			// change object coordinates
+			for (unsigned int obji = 0; obji < m_objects.GetCount(); obji++) {
+				MenuObject* obj = m_objects[obji];
+				obj->SetY(round(obj->GetY()*f));
+				if (!obj->IsDefaultSize()) {
+					obj->SetWidth(round(obj->GetWidth()*f));
+					obj->SetHeight(round(obj->GetHeight()*f));
+				}
+				obj->UpdateSize();
+			}
+		}
 	}
 	return true;
 }
@@ -832,8 +947,7 @@ void Menu::FixPosition(MenuObject* obj) {
 		obj->SetX(GetResolution().x / 4);
 	if (obj->GetY() == -1)
 		obj->SetY(GetResolution().y / 4);
-	while (obj->GetX() < GetResolution().x - 48 && obj->GetY()
-			< GetResolution().y - 48) {
+	while (obj->GetX() < GetResolution().x - 48 && obj->GetY() < GetResolution().y - 48) {
 		bool found = false;
 		for (int i = 0; i < (int) GetObjectsCount(); i++) {
 			MenuObject* obj1 = GetObject(i);
