@@ -3,7 +3,7 @@
 // Purpose:     The widget to edit a DVD Menu
 // Author:      Alex Thuering
 // Created:     11.10.2003
-// RCS-ID:      $Id: MenuEditor.cpp,v 1.39 2010/02/22 22:47:18 ntalex Exp $
+// RCS-ID:      $Id: MenuEditor.cpp,v 1.49 2010/08/15 17:54:16 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -26,6 +26,7 @@
 #include <wx/clipbrd.h>
 #include <wx/artprov.h>
 #include <wx/filename.h>
+#include <wx/colordlg.h>
 
 #define OBJECTS_DIR wxFindDataDirectory(_T("objects"))
 #define BUTTONS_DIR wxFindDataDirectory(_T("buttons"))
@@ -74,7 +75,7 @@ public:
 			return false;
 		bool res = false;
 		for (int i = 0; i<(int)filenames.Count(); i++) {
-			res = AddFile(x, y, filenames[i]) || res;
+			res = AddFile(x, y, filenames[i], i == 0) || res;
 			if (x < m_menuEditor->GetMenu()->GetResolution().x-48
 					&& y < m_menuEditor->GetMenu()->GetResolution().y-48) {
 				x += 16;
@@ -84,18 +85,29 @@ public:
 		return res;
 	}
 
-	bool AddFile(wxCoord x, wxCoord y, wxString filename) {
+	bool AddFile(wxCoord x, wxCoord y, wxString filename, bool first) {
 		wxString ext = filename.AfterLast('.').Lower();
 		if (ext == _T("xml"))
-			m_menuEditor->AddButton(filename, (int) (x/m_menuEditor->GetScale()), (int) (y/m_menuEditor->GetScale()));
+			m_menuEditor->AddButton(filename, (int) (x/m_menuEditor->GetScaleX()), (int) (y/m_menuEditor->GetScaleY()));
 		else if (wxImage::FindHandler(ext, wxBITMAP_TYPE_ANY))
-			m_menuEditor->AddImage(filename, (int) (x/m_menuEditor->GetScale()), (int) (y/m_menuEditor->GetScale()));
+			m_menuEditor->AddImage(filename, (int) (x/m_menuEditor->GetScaleX()), (int) (y/m_menuEditor->GetScaleY()));
 		else if (wxThumbnails::IsVideo(filename)) {
 			wxFfmpegMediaDecoder decoder;
 			if (!decoder.Load(filename)) {
 				wxLogError(filename + _(": unknown format"));
 				return false;
 			}
+			// check video
+			bool hasVideo = false;
+			for (unsigned int i = 0; i < decoder.GetStreamCount(); i++) {
+				if (decoder.GetStreamType(i) == stVIDEO) {
+					hasVideo = true;
+					break;
+				}
+			}
+			if (!hasVideo) {
+				return false;
+			}
 			// try to find this file
 			bool found = false;
 			for (unsigned int tsi = 0; tsi < m_menuEditor->GetDVD()->GetTitlesets().GetCount(); tsi++) {
@@ -106,7 +118,7 @@ public:
 						Vob& vob = *pgc.GetVobs()[vobi];
 						if (vob.GetFilename() == filename) {
 							found = true;
-							AddButton(x, y, tsi, pgci);
+							AddButton(x, y, tsi, pgci, false, !first);
 							break;
 						}
 					}
@@ -121,7 +133,7 @@ public:
 					tsi = 0;
 				if (m_menuEditor->GetTitlesetManager()->AddVideo(filename, true, tsi)) {
 					int pgci = m_menuEditor->GetDVD()->GetPgcArray(tsi, false).GetCount()-1;
-					AddButton(x, y, tsi, pgci);
+					AddButton(x, y, tsi, pgci, false, !first);
 				}
 			}
 		}
@@ -129,11 +141,9 @@ public:
 		return true;
 	}
 
-	void AddButton(wxCoord x, wxCoord y, int tsi, int pgci, bool isMenu = false) {
+	void AddButton(wxCoord x, wxCoord y, int tsi, int pgci, bool isMenu, bool force = false) {
 		wxString objId = m_menuEditor->HitTest(x, y);
-		x = (int) (x/m_menuEditor->GetScale());
-		y = (int) (y/m_menuEditor->GetScale());
-		if (objId.length() && m_menuEditor->GetMenu()->GetObject(objId)->IsButton()) {
+		if (!force && objId.length() && m_menuEditor->GetMenu()->GetObject(objId)->IsButton()) {
 			MenuObject* obj = m_menuEditor->GetMenu()->GetObject(objId);
 			MenuObjectParam* initParam = obj->GetInitParam();
 			if (initParam && initParam->type == wxT("image")) {
@@ -143,10 +153,12 @@ public:
 					m_menuEditor->Select(objId);
 				}
 			}
-			obj->GetAction().SetTsi(tsi);
+			obj->GetAction().SetTsi(tsi == m_menuEditor->GetTsi() ? -2 : tsi);
 			obj->GetAction().SetPgci(pgci);
 			obj->GetAction().SetMenu(isMenu);
 		} else {
+			x = (int) (x/m_menuEditor->GetScaleX());
+			y = (int) (y/m_menuEditor->GetScaleY());
 			if (isMenu)
 				m_menuEditor->AddButton(BUTTONS_DIR + s_config.GetDefTextButton(), x, y, pgci, tsi, isMenu);
 			else
@@ -164,24 +176,22 @@ private:
 /////////////////////////// Help functions ///////////////////////////////////
 //////////////////////////////////////////////////////////////////////////////
 
-wxRect ScaleRect(wxRect rect, double scale)
-{
-  wxRect res;
-  res.x = (int) (rect.x*scale);  
-  res.y = (int) (rect.y*scale);
-  res.width = (int) (rect.width*scale);
-  res.height = (int) (rect.height*scale);
-  return res;
+wxRect ScaleRect(wxRect rect, double scaleX, double scaleY) {
+	wxRect res;
+	res.x = (int) (rect.x * scaleX);
+	res.y = (int) (rect.y * scaleY);
+	res.width = (int) (rect.width * scaleX);
+	res.height = (int) (rect.height * scaleY);
+	return res;
 }
 
-wxRect ScaleRect(wxSVGRect rect, double scale)
-{
-  wxRect res;
-  res.x = (int) (rect.GetX()*scale);  
-  res.y = (int) (rect.GetY()*scale);
-  res.width = (int) (rect.GetWidth()*scale);
-  res.height = (int) (rect.GetHeight()*scale);
-  return res;
+wxRect ScaleRect(wxSVGRect rect, double scaleX, double scaleY) {
+	wxRect res;
+	res.x = (int) (rect.GetX() * scaleX);
+	res.y = (int) (rect.GetY() * scaleY);
+	res.width = (int) (rect.GetWidth() * scaleX);
+	res.height = (int) (rect.GetHeight() * scaleY);
+	return res;
 }
 
 //////////////////////////////////////////////////////////////////////////////
@@ -210,6 +220,7 @@ enum {
 	MENUEDITOR_GRIDMENU_ITEM_ID, //8601..8605
 	MENUEDITOR_GRIDMENU_SHOW_ID = 8610,
 	MENUEDITOR_GRIDMENU_OVEROBJECTS_ID,
+	MENUEDITOR_GRIDMENU_COLOUR_ID,
 	MENUEDITOR_ADDMENU_ID = 9000,
 	MENUEDITOR_ADDMENU_OBJECT_ID //9001..9999
 };
@@ -249,13 +260,14 @@ BEGIN_EVENT_TABLE(MenuEditor, wxSVGCtrl)
   EVT_MENU(MENUEDITOR_GRIDMENU_ITEM_ID+5, MenuEditor::OnGridMenu)
   EVT_MENU(MENUEDITOR_GRIDMENU_SHOW_ID, MenuEditor::OnShowGrid)
   EVT_MENU(MENUEDITOR_GRIDMENU_OVEROBJECTS_ID, MenuEditor::OnShowGridOverObjects)
+  EVT_MENU(MENUEDITOR_GRIDMENU_COLOUR_ID, MenuEditor::OnGridColour)
   EVT_MENU_RANGE(MENUEDITOR_ADDMENU_OBJECT_ID, MENUEDITOR_ADDMENU_OBJECT_ID+999, MenuEditor::OnAddObject)
 END_EVENT_TABLE()
 
 DEFINE_EVENT_TYPE(EVT_COMMAND_MENUEDITOR_MENU_CHANGED)
 DEFINE_EVENT_TYPE(EVT_COMMAND_MENUEDITOR_OBJECT_POINTED)
 
-MenuEditor::MenuEditor(wxWindow *parent, wxWindowID id): wxSVGCtrl(parent, id) {
+MenuEditor::MenuEditor(wxWindow *parent, wxWindowID id): wxSVGCtrl(parent, id), m_gridColour(255, 255, 255) {
 	m_dvd = NULL;
 	m_menu = NULL;
 	objectIsMoving = false;
@@ -306,6 +318,7 @@ MenuEditor::MenuEditor(wxWindow *parent, wxWindowID id): wxSVGCtrl(parent, id) {
 	m_gridMenu->Check(MENUEDITOR_GRIDMENU_ITEM_ID + m_grid - 1, true);
 	m_gridMenu->Append(MENUEDITOR_GRIDMENU_SHOW_ID, _("&Show"), wxEmptyString, wxITEM_CHECK);
 	m_gridMenu->Append(MENUEDITOR_GRIDMENU_OVEROBJECTS_ID, _("&Over objects"), wxEmptyString, wxITEM_CHECK);
+	m_gridMenu->Append(MENUEDITOR_GRIDMENU_COLOUR_ID, _("&Colour..."));
 	
 	// object context menu
 	m_objMenu = new wxMenu;
@@ -420,7 +433,10 @@ bool MenuEditor::AddButton(wxString fname, int x, int y, int pgci, int tsi, bool
 	if (!m_menu || !wxFileExists(fname))
 		return false;
 	
-	if (m_menu->GetButtonsCount() > 33) {
+	if (m_menu->GetAspectRatio() == ar16_9 && m_menu->GetButtonsCount() >= 12) {
+		wxLogError(_("Wide screen DVD menu can contain maximal 12 buttons"));
+		return false;
+	} else if (m_menu->GetButtonsCount() >= 34) {
 		wxLogError(_("DVD menu can contain maximal 34 buttons"));
 		return false;
 	}
@@ -495,8 +511,8 @@ bool MenuEditor::AddObject(wxString fname, int x, int y) {
 }
 
 wxString MenuEditor::HitTest(int x, int y) {
-	x = (int) (x / GetScale());
-	y = (int) (y / GetScale());
+	x = (int) (x / GetScaleX());
+	y = (int) (y / GetScaleY());
 	wxString id;
 	// at first try to find a button
 	bool button = true;
@@ -532,16 +548,16 @@ void MenuEditor::DoSelect(wxString id, bool refresh, bool sendEvent) {
 	wxSVGRectElement* selRect = (wxSVGRectElement*) m_doc->GetElementById(wxT("selection"));
 	if (selRect) {
 		if (refresh)
-			RefreshRect(ScaleRect(selRect->GetBBox(), GetScale()).Inflate(4));
+			RefreshRect(ScaleRect(selRect->GetBBox(), GetScaleX(), GetScaleY()).Inflate(4));
 		selRect->GetParent()->RemoveChild(selRect);
 	}
 
 	// create selection rectangle
 	if (id.length()) {
-		wxSVGUseElement* elem = (wxSVGUseElement*) m_doc->GetElementById(id);
-		if (!elem || elem->GetDtd() != wxSVG_USE_ELEMENT)
+		MenuObject* obj = m_menu->GetObject(id);
+		if (!obj)
 			return;
-		wxSVGRect bbox = elem->GetBBox();
+		wxRect bbox = obj->GetBBox();
 
 		selRect = new wxSVGRectElement;
 		selRect->SetId(wxT("selection"));
@@ -554,7 +570,7 @@ void MenuEditor::DoSelect(wxString id, bool refresh, bool sendEvent) {
 		m_doc->GetRootElement()->AppendChild(selRect);
 
 		if (refresh)
-			RefreshRect(ScaleRect(selRect->GetBBox(), GetScale()).Inflate(4));
+			RefreshRect(ScaleRect(selRect->GetBBox(), GetScaleX(), GetScaleY()).Inflate(4));
 	}
 
 	if (sendEvent)
@@ -611,7 +627,7 @@ void MenuEditor::OnMouseMove(wxMouseEvent &event) {
 		int x1 = x - m_oldx;
 		int y1 = y - m_oldy;
 		if (MoveObject(x1, y1)) {
-			DoSelect(GetSelected(), true); // refresh
+			DoSelect(m_selected, true); // refresh
 			objectIsMoving = true;
 		}
 		m_oldx += x1;
@@ -619,7 +635,7 @@ void MenuEditor::OnMouseMove(wxMouseEvent &event) {
 	} else if (m_tt != ttOut) { // m_tt=TL,TR,BL,BR
 		// resize object
 		if (ResizeObject(x, y, m_tt)) {
-			DoSelect(GetSelected(), true); // refresh
+			DoSelect(m_selected, true); // refresh
 			objectIsMoving = true;
 		}
 	}
@@ -630,25 +646,25 @@ void MenuEditor::OnMouseRightButton(wxMouseEvent &event) {
 	if (event.RightDown() || !m_menu)
 		return;
 	// select Object
-	wxString sel_old = GetSelected();
+	wxString sel_old = m_selected;
 	DoSelect(HitTest(event.GetX(), event.GetY()), true);
 	m_menuPos = event.GetPosition();
 	if (event.RightUp())
-		PopupMenu(GetSelected().length() ? m_objMenu : m_pmenu, event.GetPosition());
+		PopupMenu(m_selected.length() ? m_objMenu : m_pmenu, event.GetPosition());
 }
 
 TransformType MenuEditor::GetTransformType(int x, int y) {
-	if (!m_menu || !GetSelected().length())
+	if (!m_menu || !m_selected.length())
 		return ttOut;
 
 	// test all covered object
-	if (HitTest(x, y) != GetSelected())
+	if (HitTest(x, y) != m_selected)
 		return ttOut; // outside of the selected object
 
-	MenuObject* obj = m_menu->GetObject(GetSelected());
+	MenuObject* obj = m_menu->GetObject(m_selected);
 	if (!obj)
 		return ttOut;
-	wxRect rect = ScaleRect(obj->GetBBox(), GetScale());
+	wxRect rect = ScaleRect(obj->GetBBox(), GetScaleX(), GetScaleY());
 	wxRect rect2 = rect;
 	rect2.Deflate(3, 3);
 	if (!rect2.Contains(x, y)) {
@@ -690,15 +706,15 @@ bool MenuEditor::MoveObject(int& x, int& y) {
 	if (!m_menu)
 		return false;
 
-	MenuObject* obj = m_menu->GetObject(GetSelected());
+	MenuObject* obj = m_menu->GetObject(m_selected);
 	if (!obj)
 		return false;
 
-	x = (int) round(x / GetScale());
-	y = (int) round(y / GetScale());
+	x = (int) round(x / GetScaleX());
+	y = (int) round(y / GetScaleY());
 	bool res = MoveObjectInt(obj, x, y);
-	x = (int) round(x * GetScale());
-	y = (int) round(y * GetScale());
+	x = (int) round(x * GetScaleX());
+	y = (int) round(y * GetScaleY());
 	return res;
 }
 
@@ -706,7 +722,7 @@ bool MenuEditor::MoveObjectInt(MenuObject* obj, int& x, int& y) {
 	wxRect rect = obj->GetBBox();
 
 	if (m_copy) {
-		wxString id = m_menu->AddObject(obj->GetXML(DVDSTYLER_XML, m_dvd, true), true);
+		wxString id = m_menu->AddObject(obj->GetXML(DVDSTYLER_XML, m_dvd, ssmNORMAL, true), true);
 		if (id.length() == 0)
 			return false;
 		obj = m_menu->GetObject(id);
@@ -763,8 +779,8 @@ bool MenuEditor::MoveObjectInt(MenuObject* obj, int& x, int& y) {
 
 bool MenuEditor::ResizeObject(int x, int y, TransformType transformType)
 {
-  x = (int) round(x/GetScale());
-  y = (int) round(y/GetScale());
+  x = (int) round(x/GetScaleX());
+  y = (int) round(y/GetScaleY());
   return ResizeObjectInt(x, y, transformType);
 }
 
@@ -772,7 +788,7 @@ bool MenuEditor::ResizeObjectInt(int x, int y, TransformType transformType) {
 	if (!m_menu)
 		return false;
 
-	MenuObject* obj = m_menu->GetObject(GetSelected());
+	MenuObject* obj = m_menu->GetObject(m_selected);
 	if (!obj)
 		return false;
 	wxRect rect = obj->GetBBox();
@@ -837,28 +853,31 @@ void MenuEditor::ShowSafeTV() {
 		gElem->GetParent()->RemoveChild(gElem);
     
 	if (m_safeTV) {
+		int safeWidth = m_menu->GetAspectRatio() == ar4_3
+				|| m_dvd->GetPgcArray(m_tsi, true).GetVideo().GetWidescreen() == wtNOPANSCAN ? 48 : 90; // 6.7%/12.5%
+		int safeHeight = m_menu->GetAspectRatio() == ar4_3 ? 27 : 20; // 5%
 		gElem = new wxSVGGElement;
 		gElem->SetId(wxT("safeTV"));
 		wxSVGRectElement* rectElem = new wxSVGRectElement;
 	    rectElem->SetX(0);
 	    rectElem->SetY(0);
 	    rectElem->SetWidth(m_menu->GetResolution().GetWidth());
-	    rectElem->SetHeight(32);
+	    rectElem->SetHeight(safeHeight);
 	    rectElem->SetStroke(wxSVGPaint());
 	    rectElem->SetFill(wxSVGPaint(255, 255, 255));
 	    rectElem->SetFillOpacity(0.1);
 	    gElem->AppendChild(rectElem);
 	    rectElem = (wxSVGRectElement*) rectElem->CloneNode();
-	    rectElem->SetY(m_menu->GetResolution().GetHeight()-33);
+	    rectElem->SetY(m_menu->GetResolution().GetHeight()  - safeHeight - 1);
 	    gElem->AppendChild(rectElem);
 	    rectElem = (wxSVGRectElement*) rectElem->CloneNode();
 	    rectElem->SetX(0);
-	    rectElem->SetY(32);
-	    rectElem->SetWidth(40);
-	    rectElem->SetHeight(m_menu->GetResolution().GetHeight()-65);
+	    rectElem->SetY(safeHeight);
+	    rectElem->SetWidth(safeWidth);
+	    rectElem->SetHeight(m_menu->GetResolution().GetHeight() - safeHeight*2 - 1);
 	    gElem->AppendChild(rectElem);
 	    rectElem = (wxSVGRectElement*) rectElem->CloneNode();
-	    rectElem->SetX(m_menu->GetResolution().GetWidth()-41);
+	    rectElem->SetX(m_menu->GetResolution().GetWidth() - safeWidth - 1);
 	    gElem->AppendChild(rectElem);
 		m_doc->GetRootElement()->AppendChild(gElem);
 	}
@@ -877,7 +896,7 @@ void MenuEditor::ShowGrid() {
 		wxSVGLineElement* lineElem = new wxSVGLineElement;
 	    lineElem->SetY1(0);
 	    lineElem->SetY2(m_menu->GetResolution().GetHeight()-1);
-	    lineElem->SetStroke(wxSVGPaint(255, 255, 255));
+	    lineElem->SetStroke(wxSVGPaint(m_gridColour));
 	    lineElem->SetFill(wxSVGPaint());
 	    lineElem->SetStrokeOpacity(0.1);
 	    int step = 1<<m_grid;
@@ -906,42 +925,60 @@ void MenuEditor::ShowGrid() {
 
 //////////////////////////////////////////////////////////////////////////////
 
-void MenuEditor::OnMouseDClick(wxMouseEvent& WXUNUSED(event)) {
+void MenuEditor::OnMouseDClick(wxMouseEvent& event) {
 	wxCommandEvent evt;
 	OnProperties(evt);
 }
 
 void MenuEditor::OnKeyDown(wxKeyEvent& event) {
-	if (event.ControlDown() && event.GetKeyCode() == 'V') {
-		m_menuPos = wxPoint(-1, -1);
-		wxCommandEvent evt;
-		OnObjectPaste(evt);
+	if (!m_menu)
 		return;
-	}
-	if (GetSelected().length()) {
-		if (event.GetKeyCode() == WXK_DELETE) {
-			wxCommandEvent evt;
-			OnObjectDelete(evt);
-			return;
-		} else if (event.ControlDown() && event.GetKeyCode() == 'C') {
-			wxCommandEvent evt;
-			OnObjectCopy(evt);
-			return;
-		} else if (event.ControlDown() && event.GetKeyCode() == 'X') {
-			wxCommandEvent evt;
-			OnObjectCopy(evt);
-			OnObjectDelete(evt);
-			return;
+	switch (event.GetKeyCode()) {
+		case WXK_TAB: {
+			int idx = m_selected.length() > 0 ? m_menu->GetIndexOfObject(m_selected) + 1 : 0;
+			if (idx >= (int) m_menu->GetObjectsCount())
+				idx = 0;
+			if (m_menu->GetObject(idx))
+				DoSelect(m_menu->GetObject(idx)->GetId(), true, true);
+			break;
 		}
+		case WXK_DELETE:
+			if (m_selected.length()) {
+				wxCommandEvent evt;
+				OnObjectDelete(evt);
+			}
+			break;
+		case 'C':
+			if (event.ControlDown() && m_selected.length()) {
+				wxCommandEvent evt;
+				OnObjectCopy(evt);
+			}
+			break;
+		case 'X':
+			if (event.ControlDown() && m_selected.length()) {
+				wxCommandEvent evt;
+				OnObjectCopy(evt);
+				OnObjectDelete(evt);
+			}
+			break;
+		case 'V':
+			if (event.ControlDown()) {
+				m_menuPos = wxPoint(-1, -1);
+				wxCommandEvent evt;
+				OnObjectPaste(evt);
+			}
+			break;
+		default:
+			event.Skip();
+			break;
 	}
-	event.Skip();
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
 void MenuEditor::OnObjectCopy(wxCommandEvent& WXUNUSED(event)) {
-	CopyXmlToClipboard(m_menu->GetObject(GetSelected())->GetXML(DVDSTYLER_XML, m_dvd, true),
-			DATAFORMAT_MENU_OBJECT);
+	CopyXmlToClipboard(m_menu->GetObject(m_selected)->GetXML(
+			DVDSTYLER_XML, m_dvd, ssmNORMAL, true), DATAFORMAT_MENU_OBJECT);
 }
 
 void MenuEditor::OnObjectPaste(wxCommandEvent& WXUNUSED(event)) {
@@ -953,8 +990,8 @@ void MenuEditor::OnObjectPaste(wxCommandEvent& WXUNUSED(event)) {
 		if (id.length() > 0) {
 			MenuObject* obj = m_menu->GetObject(id);
 			if (obj && m_menuPos.x != -1 && m_menuPos.y != -1) {
-				obj->SetX((int)(m_menuPos.x/GetScale()));
-				obj->SetY((int)(m_menuPos.y/GetScale()));
+				obj->SetX((int)(m_menuPos.x/GetScaleX()));
+				obj->SetY((int)(m_menuPos.y/GetScaleY()));
 			}
 		}
 	} else if (wxTheClipboard->IsSupported(wxDF_TEXT)) {
@@ -964,8 +1001,8 @@ void MenuEditor::OnObjectPaste(wxCommandEvent& WXUNUSED(event)) {
 		wxString id = m_menu->AddText(text);
 		MenuObject* obj = m_menu->GetObject(id);
 		if (obj && m_menuPos.x != -1 && m_menuPos.y != -1) {
-			obj->SetX((int)(m_menuPos.x/GetScale()));
-			obj->SetY((int)(m_menuPos.y/GetScale()));
+			obj->SetX((int)(m_menuPos.x/GetScaleX()));
+			obj->SetY((int)(m_menuPos.y/GetScaleY()));
 		}
 	}
 	Update();
@@ -983,75 +1020,79 @@ void MenuEditor::OnObjectCut(wxCommandEvent& event) {
 }
 
 void MenuEditor::OnObjectDelete(wxCommandEvent& event) {
-	m_menu->RemoveObject(GetSelected());
+	m_menu->RemoveObject(m_selected);
 	DoSelect(wxT(""), true, true);
 }
 
 void MenuEditor::OnFirstButton(wxCommandEvent& event) {
-	m_menu->SetFirstButton(GetSelected());
+	m_menu->SetFirstButton(m_selected);
 }
 
 void MenuEditor::OnUpdateUIFirstButton(wxUpdateUIEvent& event) {
-	event.Enable(m_menu->GetObject(GetSelected())->IsButton());
-	event.Check(m_menu->IsFirstButton(GetSelected()));
+	event.Enable(m_menu->GetObject(m_selected)->IsButton());
+	event.Check(m_menu->IsFirstButton(m_selected));
 }
 
 void MenuEditor::OnObjectToFront(wxCommandEvent& event) {
-	m_menu->ObjectToFront(GetSelected());
-	DoSelect(GetSelected(), true, true);
+	m_menu->ObjectToFront(m_selected);
+	DoSelect(m_selected, true, true);
 }
 
 void MenuEditor::OnObjectForward(wxCommandEvent& event) {
-	m_menu->ObjectForward(GetSelected());
-	DoSelect(GetSelected(), true, true);
+	m_menu->ObjectForward(m_selected);
+	DoSelect(m_selected, true, true);
 }
 
 void MenuEditor::OnObjectBackward(wxCommandEvent& event) {
-	m_menu->ObjectBackward(GetSelected());
-	DoSelect(GetSelected(), true, true);
+	m_menu->ObjectBackward(m_selected);
+	DoSelect(m_selected, true, true);
 }
 
 void MenuEditor::OnObjectToBack(wxCommandEvent& event) {
-	m_menu->ObjectToBack(GetSelected());
-	DoSelect(GetSelected(), true, true);
+	m_menu->ObjectToBack(m_selected);
+	DoSelect(m_selected, true, true);
 }
 
 void MenuEditor::OnUpdateUIObjectForward(wxUpdateUIEvent& event) {
-	event.Enable(!m_menu->IsLastObject(GetSelected()));
+	event.Enable(!m_menu->IsLastObject(m_selected));
 }
 
 void MenuEditor::OnUpdateUIObjectBackward(wxUpdateUIEvent& event) {
-	event.Enable(!m_menu->IsFirstObject(GetSelected()));
+	event.Enable(!m_menu->IsFirstObject(m_selected));
 }
 
 void MenuEditor::OnAlignToCenterH(wxCommandEvent& event) {
-	MenuObject* obj = m_menu->GetObject(GetSelected());
+	MenuObject* obj = m_menu->GetObject(m_selected);
 	obj->SetX((m_menu->GetResolution().GetWidth() - obj->GetWidth())/2);
-	DoSelect(GetSelected(), true, true);
+	DoSelect(m_selected, true, true);
 }
 
 void MenuEditor::OnAlignToCenterV(wxCommandEvent& event) {
-	MenuObject* obj = m_menu->GetObject(GetSelected());
+	MenuObject* obj = m_menu->GetObject(m_selected);
 	obj->SetY((m_menu->GetResolution().GetHeight() - obj->GetHeight())/2);
-	DoSelect(GetSelected(), true, true);
+	DoSelect(m_selected, true, true);
 }
 
 void MenuEditor::OnProperties(wxCommandEvent& event) {
 	if (!m_menu)
 		return;
-	if (!GetSelected().length()) {
+	if (!m_selected.length()) {
 		// show menu properties
 		MenuPropDlg propDlg(this, m_dvd, m_tsi, m_pgci);
-		if (propDlg.ShowModal() == wxID_OK)
-			Update();
+		if (propDlg.ShowModal() == wxID_OK) {
+			ShowSafeTV();
+			ShowGrid(); 
+			SendChangedEvent(true);
+			Refresh(true, NULL);
+		}
 	} else {
 		// show object properties
-		MenuObject* obj = m_menu->GetObject(GetSelected());
+		MenuObject* obj = m_menu->GetObject(m_selected);
 		if (!obj)
 			return;
-		MenuObjectPropDlg propDlg(this, GetSelected(), m_menu, m_dvd, m_tsi, m_pgci);
+		MenuObjectPropDlg propDlg(this, m_selected, m_menu, m_dvd, m_tsi, m_pgci);
 		if (propDlg.ShowModal() == wxID_OK)
-			DoSelect(GetSelected(), true, true);
+			DoSelect(m_selected, true, true);
 	}
 }
 
@@ -1070,6 +1111,18 @@ void MenuEditor::OnShowGridOverObjects(wxCommandEvent& event) {
 	ShowGrid();
 }
 
+void MenuEditor::OnGridColour(wxCommandEvent& event) {
+	wxColourData data;
+	data.SetColour(m_gridColour);
+	wxColourDialog dlg(this, &data);
+	if (dlg.ShowModal() == wxID_OK) {
+		m_gridColour = dlg.GetColourData().GetColour();
+		m_showGrid = true;
+		m_gridMenu->FindItem(MENUEDITOR_GRIDMENU_SHOW_ID)->Check();
+		ShowGrid();
+	}
+}
+
 void MenuEditor::OnGridMenu(wxCommandEvent& event) {
 	long id = event.GetId() - MENUEDITOR_GRIDMENU_ITEM_ID;
 	m_grid = id;
@@ -1081,11 +1134,11 @@ void MenuEditor::OnGridMenu(wxCommandEvent& event) {
 
 void MenuEditor::OnAddObject(wxCommandEvent& event) {
 	long n = event.GetId() - MENUEDITOR_ADDMENU_OBJECT_ID;
-	AddObject(m_addMenuObjects[n], (int) (m_menuPos.x / GetScale()),
-			(int) (m_menuPos.y / GetScale()));
+	AddObject(m_addMenuObjects[n], (int) (m_menuPos.x / GetScaleX()), (int) (m_menuPos.y / GetScaleY()));
 }
 
-void MenuEditor::SendChangedEvent() {
+void MenuEditor::SendChangedEvent(bool updateAllItems) {
 	wxCommandEvent evt(EVT_COMMAND_MENUEDITOR_MENU_CHANGED, this->GetId());
+	evt.SetInt(updateAllItems);
 	GetEventHandler()->ProcessEvent(evt);
 }
