@@ -3,7 +3,7 @@
 // Purpose:     The class to store a DVD Menu Object
 // Author:      Alex Thuering
 // Created:	04.11.2006
-// RCS-ID:      $Id: MenuObject.h,v 1.12 2009/12/30 16:56:48 ntalex Exp $
+// RCS-ID:      $Id: MenuObject.h,v 1.17 2010/09/05 10:54:46 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -20,11 +20,19 @@ class wxSVGDocument;
 class wxSVGElement;
 class wxSVGUseElement;
 class wxSVGSVGElement;
+class wxSVGRect;
 class Menu;
 class DVD;
 
 enum MenuButtonState { mbsNORMAL, mbsHIGHLIGHTED, mbsSELECTED };
 
+enum SubStreamMode {
+	ssmNORMAL = 0,
+	ssmWIDESCREEN,
+	ssmLETTERBOX,
+	ssmPANSCAN
+};
+
 struct MenuObjectParam {
   wxString name;
   wxString title;
@@ -47,13 +55,17 @@ struct MenuObjectSize {
 };
 
 class MenuObject {
-  public:
-	MenuObject(Menu* menu, bool vmg, wxString fileName = wxT(""), int x = 0, int y = 0,
-			wxString param = wxT(""));
+public:
+	/** Constructor */
+	MenuObject(Menu* menu, bool vmg, wxString fileName = wxT(""), int x = 0, int y = 0, wxString param = wxT(""));
+	/** Destructor */
 	virtual ~MenuObject();
 	
+	/** Returns id of menu object */
 	virtual wxString GetId(bool translate = false);
+	/** Returns true id this menu object is a button */
 	inline bool IsButton() { return m_button; }
+	/** Returns button action */
 	inline DVDAction& GetAction() { return m_action; }
     
     inline wxString GetFileName() { return m_fileName; }
@@ -67,8 +79,8 @@ class MenuObject {
     void SetWidth(int value);
     int  GetHeight();
     void SetHeight(int value);
-    
-    inline wxRect GetBBox() { return wxRect(GetX(), GetY(), GetWidth(), GetHeight()); }
+    wxRect GetBBox();
+    wxRect GetFrameBBox(SubStreamMode mode, bool ignorePadding = false);
 	
 	inline bool IsDefaultSize() { return m_defaultSize; }
 	inline void SetDefaultSize(bool value = true) { m_defaultSize = value; }
@@ -90,7 +102,16 @@ class MenuObject {
 	void SetParamFont(wxString name, wxFont value);
 	wxColour GetParamColour(wxString name, MenuButtonState state = mbsNORMAL);
 	void SetParamColour(wxString name, wxColour value, MenuButtonState state = mbsNORMAL);
-    
+	
+	/** Returns true, if video frame must be displayed (if button has image parameter and "jump to title" action) */
+	inline bool IsDisplayVideoFrame() { return m_displayVideoFrame; }
+	/** Sets, if video frame must be displayed */
+	inline void SetDisplayVideoFrame(bool displayVideoFrame) { m_displayVideoFrame = displayVideoFrame; }
+	/** Returns true, if custom video frame is selected */
+	inline bool IsCustomVideoFrame() { return m_customVideoFrame; }
+	/** Sets, if custom video frame is selected */
+	inline void SetCustomVideoFrame(bool customVideoFrame) { m_customVideoFrame = customVideoFrame; }
+	
     void ToFront();
     void Forward();
     void Backward();
@@ -103,10 +124,11 @@ class MenuObject {
 	
 	wxImage GetImage(int maxWidth, int maxHeight);
 	
-	virtual wxSvgXmlNode* GetXML(DVDFileType type, DVD* dvd, bool withSVG = false, int playAllRegister = -1);
+	virtual wxSvgXmlNode* GetXML(DVDFileType type, DVD* dvd, SubStreamMode mode = ssmNORMAL, bool withSVG = false);
 	virtual bool PutXML(wxSvgXmlNode* node);
 	
   private:
+	Menu* m_menu; // can be null
 	wxString m_id;
 	bool m_button;
 	DVDAction m_action;
@@ -118,7 +140,9 @@ class MenuObject {
     
     MenuObjectParams m_params;
     wxString m_initParameter;
-	
+    bool m_displayVideoFrame; // sets if video frame must be displayed
+    bool m_customVideoFrame; // shows if custom video frame is selected
+    	
     bool m_defaultSize;
     MenuObjectSize m_defaultWidth;
     MenuObjectSize m_defaultHeight;
@@ -133,6 +157,8 @@ class MenuObject {
     wxSVGSVGElement* AddSymol(wxString id, wxSVGElement* content);
     void AddUse(wxString id, int x, int y, int width, int height);
     
+    void SetScale(double scaleX, double scaleY);
+    
     bool Init(wxString filename, int x = 0, int y = 0, wxString param = wxT(""));
     bool LoadSVG(wxSVGDocument& svg, wxSvgXmlNode* node);
     wxString GenerateId(wxString prefix);
