@@ -3,12 +3,13 @@
 // Purpose:     DVD menu button properties dialog
 // Author:      Alex Thuering
 // Created:	20.11.2003
-// RCS-ID:      $Id: MenuObjectPropDlg.cpp,v 1.36 2010/04/28 07:34:26 ntalex Exp $
+// RCS-ID:      $Id: MenuObjectPropDlg.cpp,v 1.41 2010/09/13 20:22:05 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
 
 #include "MenuObjectPropDlg.h"
+#include "Config.h"
 #include <wx/statline.h>
 #include <stdint.h>
 
@@ -44,6 +45,7 @@ MenuObjectPropDlg::MenuObjectPropDlg(wxWindow* parent, wxString id, Menu* menu,
 	m_videoFrameRadio = NULL;
 	m_videoFrameChoice = NULL;
 	m_object = menu->GetObject(id);
+	m_displayVideoFrame = m_object->IsDisplayVideoFrame();
 	Create();
 }
 
@@ -57,8 +59,21 @@ void MenuObjectPropDlg::CreatePropPanel(wxSizer* sizer) {
 		AddStaticLine(mainSizer, _("Action"));
 		grid = new wxFlexGridSizer(2, 2, 4, 16);
 		grid->AddGrowableCol(1);
+		// check menu entry if it is set
+		wxString customAction = action.GetCustom();
+		if (action.GetEntry().length() > 0 && customAction.length() == 0) {
+			// find menu with given entry
+			Menus& menus = (Menus&) m_dvd->GetPgcArray(action.GetTsi(), true);
+			int pgci = menus.GetPgciByEntry(action.GetEntry());
+			if (pgci >= 0) {
+				action.SetPgci(pgci);
+			} else {
+				// entry is not valid => set custom action
+				customAction = action.AsString(m_dvd);
+			}
+		}
 		// jump to
-		AddRadioProp(grid, _("Jump to") + wxString(wxT(":")), !action.IsCustom(), wxRB_GROUP, false,
+		AddRadioProp(grid, _("Jump to") + wxString(wxT(":")), customAction.length() == 0, wxRB_GROUP, false,
 				JUMP_ACTION_RADIO_ID);
 		wxSizer* jumpSizer = new wxFlexGridSizer(4, 4, 4);
 		AddChoiceProp(jumpSizer, wxT(""), wxT(""), wxArrayString(), -1, false, TARGET_CHOICE_ID);
@@ -70,7 +85,6 @@ void MenuObjectPropDlg::CreatePropPanel(wxSizer* sizer) {
 		AddCheckProp(jumpSizer, _("Play all titles"), action.IsPlayAll());
 		grid->Add(jumpSizer);
 		// customAction
-		wxString customAction = action.GetCustom();
 		if (!customAction.length()) {
 			// check if selection was ok, otherwise set this action as custom action
 			int tsi = action.GetTsi() > -2 ? action.GetTsi() : m_tsi;
@@ -80,7 +94,7 @@ void MenuObjectPropDlg::CreatePropPanel(wxSizer* sizer) {
 					|| GetSelectedChapter() != action.GetChapter())
 				customAction = action.AsString(m_dvd);
 		}
-		AddRadioProp(grid, _("Custom") + wxString(wxT(":")), customAction.length()>0, 0, false,
+		AddRadioProp(grid, _("Custom") + wxString(wxT(":")), customAction.length() > 0, 0, false,
 				CUSTOM_ACTION_RADIO_ID);
 		AddTextProp(grid, wxT(""), customAction);
 		m_customActionEdit = (wxTextCtrl*) m_controls[m_controls.Count()-1];
@@ -184,7 +198,7 @@ void MenuObjectPropDlg::CreatePropPanel(wxSizer* sizer) {
 				if (m_object->IsButton()) {
 					bool hasVideo = GetVideoFilename().length();
 					wxString fname = imageFile.Find(wxT('#')) != -1 ? imageFile.BeforeLast(wxT('#')) : imageFile;
-					bool image = !hasVideo || fname != GetVideoFilename();
+					bool image = !m_displayVideoFrame || !hasVideo || fname != GetVideoFilename();
 					AddRadioProp(grid, wxString(_("Image")) + wxT(":"), image, wxRB_GROUP, false, IMAGE_RADIO_ID);
 					m_imageRadio = (wxRadioButton*) GetLastControl();
 					AddFileProp(grid, wxT(""), image ? imageFile : wxT(""), wxFD_OPEN, wxT("..."),
@@ -301,6 +315,7 @@ void MenuObjectPropDlg::FillTargets(wxChoice* ctrl) {
 	// add a jump target for every title and menu in every titleset if possible
 	for (int tsi = -1; tsi < (int)m_dvd->GetTitlesets().Count(); tsi++) {
 		if (tsi == -1) { // Video-Manager-Domain
+			// jumpt to vmMenu
 			for (int pgci = 0; pgci < (int) m_dvd->GetVmgm().Count(); pgci++) {
 				if (tsi == m_tsi && pgci == m_pgci)
 					continue; // current menu
@@ -310,6 +325,7 @@ void MenuObjectPropDlg::FillTargets(wxChoice* ctrl) {
 			}
 		} else { // Titleset
 			Titleset* ts = m_dvd->GetTitlesets()[tsi];
+			// jumpt to menu
 			for (int pgci = 0; pgci < (int) ts->GetMenus().Count(); pgci++) {
 				if (tsi == m_tsi && pgci == m_pgci)
 					continue; // current menu
@@ -323,8 +339,17 @@ void MenuObjectPropDlg::FillTargets(wxChoice* ctrl) {
 				if (tsi == actionTsi && pgci == actionPgci && actionMenu)
 					sel = ctrl->GetCount() - 1;
 			}
+			// jump to not existing menu
+			if (tsi == actionTsi && actionPgci >= (int) ts->GetMenus().Count() && actionMenu) {
+				wxString label = _("menu") + wxString::Format(wxT(" %d*"), actionPgci + 1);
+				if (m_dvd->GetTitlesets().Count() > 1)
+					label = _("titleset") + wxString::Format(wxT(" %d "), tsi + 1) + label;
+				ctrl->Append(label, TargetToPtr(tsi, actionPgci, true));
+				sel = ctrl->GetCount() - 1;
+			}
 			// jumping from titleset menu is allowed only to titles from the same titleset
 			if (m_dvd->IsJumppadsEnabled() || m_tsi == -1 || m_tsi == tsi) {
+				// jump to title
 				for (int pgci = 0; pgci < (int) ts->GetTitles().Count(); pgci++) {
 					wxString label = _("title") + wxString::Format(wxT(" %d"), pgci + 1);
 					if (m_dvd->GetTitlesets().Count() > 1)
@@ -333,6 +358,14 @@ void MenuObjectPropDlg::FillTargets(wxChoice* ctrl) {
 					if (tsi == actionTsi && pgci == actionPgci && !actionMenu)
 						sel = ctrl->GetCount() - 1;
 				}
+				// jump to not existing title
+				if (tsi == actionTsi && actionPgci >= (int) ts->GetTitles().Count() && !actionMenu) {
+					wxString label = _("title") + wxString::Format(wxT(" %d*"), actionPgci + 1);
+					if (m_dvd->GetTitlesets().Count() > 1)
+						label = _("titleset") + wxString::Format(wxT(" %d "), tsi + 1) + label;
+					ctrl->Append(label, TargetToPtr(tsi, actionPgci, false));
+					sel = ctrl->GetCount() - 1;
+				}
 			}
 		}
 	}
@@ -359,11 +392,15 @@ void MenuObjectPropDlg::OnChangeTarget(wxCommandEvent& evt) {
 	UpdateChapters();
 	if (m_videoFrameRadio) {
 		m_videoFrameRadio->Enable(GetVideoFilename().length());
-		if (!m_videoFrameRadio->IsEnabled()) {
+		if (!m_videoFrameRadio->IsEnabled() && !m_imageRadio->GetValue()) {
 			m_imageRadio->SetValue(true);
 			wxCommandEvent event(wxEVT_COMMAND_RADIOBUTTON_SELECTED, m_imageRadio->GetId());
 			GetEventHandler()->ProcessEvent(event);
-		}
+		} else if (m_displayVideoFrame && !m_videoFrameRadio->GetValue()) {
+			m_videoFrameRadio->SetValue(true);
+			wxCommandEvent event(wxEVT_COMMAND_RADIOBUTTON_SELECTED, m_imageRadio->GetId());
+			GetEventHandler()->ProcessEvent(event);
+		} 
 	}
 }
 
@@ -380,6 +417,8 @@ void MenuObjectPropDlg::OnJumpActionSelected(wxCommandEvent& evt) {
 		}
 		else if (GetVideoFilename().length() && m_imageEdit->GetValue().length() == 0)
 			m_videoFrameRadio->SetValue(true);
+		wxCommandEvent event(wxEVT_COMMAND_RADIOBUTTON_SELECTED, m_videoFrameRadio->GetId());
+		GetEventHandler()->ProcessEvent(event);
 	}
 }
 
@@ -402,7 +441,7 @@ void MenuObjectPropDlg::OnCustomActionSelected(wxCommandEvent& evt) {
 }
 
 wxString MenuObjectPropDlg::GetVideoFilename(bool withTimestamp, long position) {
-	if (!GetBool(0))
+	if (!GetBool(0)) // custom action
 		return wxT("");
 	if (IsSelectedMenu()) // menu
 		return wxT("");
@@ -416,6 +455,8 @@ wxString MenuObjectPropDlg::GetVideoFilename(bool withTimestamp, long position)
 
 
 void MenuObjectPropDlg::OnImageRadio(wxCommandEvent& evt){
+	if (m_videoFrameRadio->IsEnabled())
+		m_displayVideoFrame = m_videoFrameRadio->GetValue();
 	m_videoFrameChoice->Enable(m_videoFrameRadio->GetValue());
 	wxCommandEvent event(wxEVT_COMMAND_CHOICE_SELECTED, m_videoFrameChoice->GetId());
 	event.SetEventObject(m_videoFrameChoice);
@@ -470,11 +511,6 @@ void MenuObjectPropDlg::AddDirectionProp(wxSizer* sizer, ButtonDirCommand dir) {
 bool MenuObjectPropDlg::SetValues() {
 	// check Action
 	if (m_object->IsButton()) {
-		if (!m_dvd->IsJumppadsEnabled() && m_tsi < 0 && GetString(0).Find(_T("chapter")) >= 0) {
-			wxMessageBox(_("You cannot jump from vmgMenu to a chapter."),
-			GetTitle(), wxOK|wxICON_ERROR, this);
-			return false;
-		}
 		if (GetBool(4) && GetString(5).length() == 0) // empty custom action
 		{
 			wxMessageBox(_("Please enter the action command."),
@@ -484,7 +520,8 @@ bool MenuObjectPropDlg::SetValues() {
 		if (GetBool(4)) {
 			DVDAction action(m_object->GetAction());
 			action.SetCustom(GetString(5));
-			if (!action.IsValid(m_dvd, m_tsi, m_pgci, true, m_object->GetId(), true, false))
+			if (!action.IsValid(m_dvd, m_tsi, m_pgci, true, m_object->GetId(), true, false)
+					&& !s_config.GetAcceptInvalidActions())
 				return false;
 		}
 	}
@@ -555,6 +592,7 @@ bool MenuObjectPropDlg::SetValues() {
 				m_object->SetParam(param->name, opacity, wxT("-opacity"));
 			} else if (param->type == _T("image")) {
 				if (m_object->IsButton()) {
+					m_object->SetDisplayVideoFrame(m_displayVideoFrame);
 					bool image = GetBool(n++);
 					wxString imgFile = GetString(n++);
 					n++;
@@ -564,6 +602,7 @@ bool MenuObjectPropDlg::SetValues() {
 							videoFrame = GetInt(n++);
 						else
 							n++;
+						m_object->SetCustomVideoFrame(videoFrame != -1);
 						imgFile = GetVideoFilename(true, videoFrame);
 					} else
 						n += 2;
