@@ -3,7 +3,7 @@
 // Purpose:     DVD menu properties dialog
 // Author:      Alex Thuering
 // Created:     25.04.2004
-// RCS-ID:      $Id: MenuPropDlg.cpp,v 1.27 2010/05/10 20:43:10 ntalex Exp $
+// RCS-ID:      $Id: MenuPropDlg.cpp,v 1.34 2010/10/10 18:39:13 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -12,6 +12,8 @@
 #include "TitlesetManager.h"
 #include "AVPropDlg.h"
 #include "Config.h"
+#include <wxVillaLib/utils.h>
+#include "rc/preferences.png.h"
 
 enum {
 	AV_BT_ID = 7850,
@@ -46,43 +48,49 @@ MenuPropDlg::MenuPropDlg(wxWindow *parent, DVD* dvd, int tsi, int pgci):
 }
 
 void MenuPropDlg::CreatePropPanel(wxSizer* sizer) {
-	wxFlexGridSizer* grid = new wxFlexGridSizer(3, 4, 16);
-	grid->AddGrowableCol(1);
-	grid->AddGrowableCol(2);
+	wxSizer* formatSizer = new wxBoxSizer(wxHORIZONTAL);
+	AddText(formatSizer, _("Format:"));
 	wxArrayString formats = DVD::GetVideoFormatLabels(false, false, true);
 	int vf = m_menu->GetVideoFormat() >= 2 ? m_menu->GetVideoFormat() - 2 : 0;
-	AddChoiceProp(grid, _("Format:"), formats[vf], formats);
+	formatSizer->Add(AddChoiceProp(formats[vf], formats), 0, wxEXPAND | wxLEFT, 4);
+	formatSizer->AddSpacer(4);
 	formats = DVD::GetAspectRatioLabels();
 	int ar = m_menu->GetAspectRatio() >= 1 ? m_menu->GetAspectRatio() - 1 : 0;
-	AddChoiceProp(grid, wxT(""), formats[ar], formats);
-	sizer->Add(grid, 0, wxEXPAND | wxALL, 6);
+	formatSizer->Add(AddChoiceProp(formats[ar], formats), 0, wxEXPAND);
+	wxButton* avBt = new wxBitmapButton(this, AV_BT_ID, wxBITMAP_FROM_MEMORY(preferences));
+	avBt->SetHelpText(_("Video properties..."));
+	formatSizer->Add(avBt, 0, wxLEFT, 4);
+	formatSizer->AddStretchSpacer(1);
+	sizer->Add(formatSizer, 0, wxEXPAND | wxALL, 6);
 
 	wxSizer* grpSizer = BeginGroup(sizer, _("Background"));
-	grid = new wxFlexGridSizer(2, 2, 4, 16);
+	wxFlexGridSizer* grid = new wxFlexGridSizer(2, 2, 4, 16);
 	grid->AddGrowableCol(1);
 	AddFileProp(grid, _("Image:"), m_menu->GetBackground(), wxFD_OPEN, wxT("..."),
 			_("Image Files ") + wxImage::GetImageExtWildcard()
-		+ wxT("|") + wxString(_("All Files")) + wxT(" (*.*)|*.*"));
+			+ wxT("|") + wxString(_("All Files")) + wxT(" (*.*)|*.*"));
 	AddColourProp(grid, _("Colour:"), m_menu->GetBackgroundColour());
 	grpSizer->Add(grid, 0, wxEXPAND|wxTOP|wxLEFT|wxRIGHT, 6);
-
+	
 	sizer->Add(6,6);
-
-	grpSizer = BeginGroup(sizer, _("Subtitles (buttons)"));
+	
+	grpSizer = BeginGroup(sizer, _("Buttons"));
 	grid = new wxFlexGridSizer(2, 2, 4, 16);
 	grid->AddGrowableCol(1);
-	AddTextProp(grid, _("Start time:"), m_menu->GetStartTime());
-	AddTextProp(grid, _("End time:"), m_menu->GetEndTime());
-	grpSizer->Add(grid, 0, wxEXPAND|wxALL, 6);
-
+	wxSizer* timeSizer = AddTextProp(grid, _("Start/End:"), m_menu->GetStartTime(), false, 100, false);
+	AddTextProp(timeSizer, wxT(" - "), m_menu->GetEndTime(), false, 100);
+	grpSizer->Add(grid, 0, wxEXPAND|wxALL, 2);
+	AddCheckProp(grpSizer, _("Remember last selected button"), m_menu->GetRememberLastButton());
+	
 	// VOB properties
 	grpSizer = BeginGroup(sizer, _("Video object"));
 	grid = new wxFlexGridSizer(2, 4, 16);
 	grid->AddGrowableCol(1);
 	// audio file
 	wxString audioFile = m_vob->GetAudioFilenames().size() ? m_vob->GetAudioFilenames()[0] : wxT("");
+	wxString audio = wxThumbnails::GetAudioExtWildcard();
 	AddFileProp(grid, _("Audio:"), audioFile, wxFD_OPEN, wxT("..."),
-			_("Audio Files") + wxString(wxT(" (*.mp2;*.ac3)|*.mp2;*.ac3|"))
+			_("Audio Files") + wxString::Format(wxT(" (%s)|%s|"), audio.c_str(), audio.c_str())
 			+ _("All Files") + wxString(wxT(" (*.*)|*.*")));
 	// audio language
 	wxArrayString labels = DVD::GetAudioLanguageCodes();
@@ -98,13 +106,13 @@ void MenuPropDlg::CreatePropPanel(wxSizer* sizer) {
 	pSizer->AddSpacer(4);
 	AddCheckProp(pSizer, _T("Loop"), loop, false, LOOP_CB_ID);
 	grpSizer->Add(grid, 0, wxEXPAND|wxALL, 6);
-
+	
 	sizer->Add(6,6);
-
+	
 	grpSizer = BeginGroup(sizer, _("Menu"));
 	grid = new wxFlexGridSizer(2, 2, 4, 16);
 	grid->AddGrowableCol(1);
-
+	
 	const StringSet& entries = m_pgc->GetEntries();
 	if (m_tsi>=0) {
 		AddCheckProp(grid, _("Root"), entries.find(wxT("root")) != entries.end());
@@ -115,7 +123,7 @@ void MenuPropDlg::CreatePropPanel(wxSizer* sizer) {
 	} else
 		AddCheckProp(grid, _("Title"), entries.find(wxT("title")) != entries.end());
 	AddText(grid, wxEmptyString);
-
+	
 	wxArrayString commands = GetCommands();
 	AddComboProp(grid, _("Pre commands:"), m_pgc->GetPreCommands(), commands);
 	AddComboProp(grid, _("Post commands:"), m_pgc->GetPostCommands(), commands);
@@ -140,26 +148,41 @@ wxArrayString MenuPropDlg::GetCommands() {
 }
 
 bool MenuPropDlg::SetValues() {
-	// check commands
+	// validate commands
+	wxString preCommands = GetString(m_tsi >= 0 ? 15 : 11);
+	wxString postCommands = GetString(m_tsi >= 0 ? 16 : 12);
 	DVDAction action(false, BUTTON_ACTION);
-	action.SetCustom(GetString(m_tsi >= 0 ? 15 : 11));
-	if (!action.IsValid(m_dvd, m_tsi, m_pgci, true, wxT(""), true, false))
+	action.SetCustom(preCommands);
+	if (preCommands.length() && !action.IsValid(m_dvd, m_tsi, m_pgci, true, wxT(""), true, false)) {
 		return false;
-	action.SetCustom(GetString(m_tsi >= 0 ? 16 : 12));
-	if (!action.IsValid(m_dvd, m_tsi, m_pgci, true, wxT(""), true, false))
+	}
+	action.SetCustom(postCommands);
+	if (postCommands.length() && !action.IsValid(m_dvd, m_tsi, m_pgci, true, wxT(""), true, false)) {
 		return false;
+	}
 	
 	int n = 0;
-	m_menu->SetVideoFormat((VideoFormat) (GetInt(n++) + 2));
-	m_menu->SetAspectRatio((AspectRatio) (GetInt(n++) + 1));
+	
+	// set video format
+	VideoFormat videoFormat = (VideoFormat) (GetInt(n++) + 2);
+	if (videoFormat != m_menu->GetVideoFormat()) {
+		m_dvd->GetPgcArray(m_tsi, true).SetVideoFormat(videoFormat);
+	}
+	// set aspect ratio
+	AspectRatio aspectRatio = (AspectRatio) (GetInt(n++) + 1);
+	if (aspectRatio != m_menu->GetAspectRatio()) {
+		m_dvd->GetPgcArray(m_tsi, true).SetAspectRatio(aspectRatio);
+	}
+	
 	m_menu->SetBackground(GetString(n++));
 	if (!GetString(n - 1).length())
 		m_menu->SetBackgroundColour(GetColour(n));
 	n++;
-
+	
 	m_menu->SetStartTime(GetString(n++));
 	m_menu->SetEndTime(GetString(n++));
-
+	m_menu->SetRememberLastButton(GetBool(n++));
+	
 	// VOB properties
 	wxString audioFile = GetString(n++).Strip(wxString::both);
 	while (m_vob->GetAudioFilenames().GetCount() > 0)
@@ -170,7 +193,7 @@ bool MenuPropDlg::SetValues() {
 	m_langCodes->Insert(GetString(n++), 0);
 	m_vob->SetPause(GetInt(n++));
 	n++;
-
+	
 	// set entries
 	StringSet entries;
 	if (m_tsi >= 0) {
@@ -194,7 +217,7 @@ bool MenuPropDlg::SetValues() {
 			pgcs[pgci]->RemoveEntries(entries);
 		}
 	}
-
+	
 	m_pgc->SetPreCommands(GetString(n++));
 	m_pgc->SetPostCommands(GetString(n++));
 	// fix coordinates of buttons if they are out of range
@@ -202,20 +225,6 @@ bool MenuPropDlg::SetValues() {
 	return true;
 }
 
-void MenuPropDlg::CreateButtonPane(wxSizer* sizer, bool resetButton, bool dontShowCheckbox) {
-	wxBoxSizer* buttonPane = new wxBoxSizer(wxHORIZONTAL);
-	wxButton* avBt = new wxButton(this, AV_BT_ID, _("Video..."));
-	buttonPane->Add(avBt, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
-	buttonPane->Add(10, 10, 1, wxEXPAND);
-	wxButton* okBt = new wxButton(this, wxID_OK, _("OK"));
-	okBt->SetDefault();
-	buttonPane->Add(okBt, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
-	buttonPane->Add(8, 0);
-	wxButton* cancelBt = new wxButton(this, wxID_CANCEL, _("Cancel"));
-	buttonPane->Add(cancelBt, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
-	sizer->Add(buttonPane, 0, wxEXPAND | wxLEFT | wxRIGHT | wxBOTTOM, 10);
-}
-
 void MenuPropDlg::OnAV(wxCommandEvent& event) {
 	AVPropDlg propDlg(this, m_video, true);
 	if (propDlg.ShowModal() == wxID_OK) {
