@@ -3,7 +3,7 @@
 // Purpose:     Progress of generation/burning dialog
 // Author:      Alex Thuering
 // Created:     14.08.2004
-// RCS-ID:      $Id: ProgressDlg.cpp,v 1.94 2010/05/22 14:14:09 ntalex Exp $
+// RCS-ID:      $Id: ProgressDlg.cpp,v 1.104 2010/10/10 16:01:32 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -25,6 +25,17 @@
 #include <wxSVG/SVGDocument.h>
 #include <wxSVG/mediadec_ffmpeg.h>
 
+#ifdef __WXMAC__
+extern "C" {
+	wxString GetDeviceName(const wxString& ioRegistryEntryPath);
+	wxString GetDeviceNode(const wxString& ioRegistryEntryPath);
+	bool IsMediaPresent(const wxString& ioRegistryEntryPath);
+	bool IsMediaErasable(const wxString& ioRegistryEntryPath);
+	bool IsMediaBlank(const wxString& ioRegistryEntryPath);
+	long GetMediaSpaceFree(const wxString& ioRegistryEntryPath);
+}
+#endif
+
 #define DATA_FILE(fname) wxFindDataFile(_T("data") + wxString(wxFILE_SEP_PATH) + fname)
 #define TMP_ISO wxT("dvd-out.iso")
 
@@ -116,6 +127,26 @@ protected:
 	int     m_percent;
 };
 
+class BurnExecute: public ProgressExecute {
+public:
+	BurnExecute(ProgressDlg* processDlg, wxString filter): ProgressExecute(processDlg, filter) {}
+
+	virtual void ProcessOutput(wxString line) {
+		if (line.Find(wxT(": writing lead-out")) >= 0)
+			m_burnOk = true;
+		ProgressExecute::ProcessOutput(line);
+	}
+
+	bool Execute(wxString command, wxString inputFile = wxEmptyString, wxString outputFile = wxEmptyString) {
+		m_burnOk = false;
+		bool res = ProgressExecute::Execute(command, wxEmptyString, wxEmptyString);
+		return res || m_burnOk;
+	}
+
+private:
+	bool m_burnOk;
+};
+
 class BlocksExecute: public ProcessExecute {
 public:
 	BlocksExecute(ProgressDlg* processDlg): ProcessExecute(processDlg), m_percent(0) {
@@ -209,6 +240,7 @@ BEGIN_EVENT_TABLE(ProgressDlg, wxDialog)
   EVT_BUTTON(ICONIZE_BT_ID, ProgressDlg::OnMinimize)
 END_EVENT_TABLE()
 
+/** Constructor */
 ProgressDlg::ProgressDlg(wxWindow* parent, Cache* cache, bool autoStart): wxDialog(parent, -1, wxEmptyString,
 		wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER) {
     // begin wxGlade: ProgressDlg::ProgressDlg
@@ -371,7 +403,7 @@ bool ProgressDlg::Start(BurnDlg* burnDlg, DVD* dvd) {
 	return isOk;
 }
 
-int ProgressDlg::GetMenuSubSteps(wxArrayPtrVoid& menuVobs, DVD* dvd, wxString dvdTmpDir) {
+int ProgressDlg::GetMenuSubSteps(wxArrayPtrVoid& menuVobs, wxArrayInt& menuWSTypes, DVD* dvd, wxString dvdTmpDir) {
 	int menuSubSteps = 0;
 	for (int tsi = -1; tsi < (int) dvd->GetTitlesets().Count(); tsi++) {
 		PgcArray& pgcs = dvd->GetPgcArray(tsi, true);
@@ -391,6 +423,7 @@ int ProgressDlg::GetMenuSubSteps(wxArrayPtrVoid& menuVobs, DVD* dvd, wxString dv
 				menuSubSteps += 25; // generate mpeg
 			menuSubSteps += 50; // spumux
 			menuVobs.Add(vob);
+			menuWSTypes.Add((int)pgcs.GetVideo().GetWidescreen());
 		}
 	}
 	return menuSubSteps;
@@ -488,7 +521,8 @@ void ProgressDlg::Run(BurnDlg* burnDlg, DVD* dvd) {
 	m_cache->BeginClean();
 	// menus
 	wxArrayPtrVoid menuVobs;
-	int menuSubSteps = GetMenuSubSteps(menuVobs, dvd, dvdTmpDir);
+	wxArrayInt menuWSTypes;
+	int menuSubSteps = GetMenuSubSteps(menuVobs, menuWSTypes, dvd, dvdTmpDir);
 	// titles
 	wxArrayPtrVoid titleVobs;
 	int titleSubSteps = GetTitleSubSteps(titleVobs, dvd, m_cache);
@@ -524,7 +558,7 @@ void ProgressDlg::Run(BurnDlg* burnDlg, DVD* dvd) {
 	SetSteps(stepCount);
 	
 	// Start generation
-	if (!GenerateMenus(menuVobs, menuSubSteps, dvd)
+	if (!GenerateMenus(menuVobs, menuWSTypes, menuSubSteps, dvd)
 			|| !Transcode(titleVobs, titleSubSteps, dvd)
 			|| !GenerateSlideshow(slideshowVobs, slideshowSubSteps, dvd)
 			|| !MultiplexSubtitles(subtitleVobs, subtitleSubSteps, dvd)
@@ -573,7 +607,7 @@ bool ProgressDlg::CleanTemp(const wxString& tmpDir, const wxString& dvdTmpDir, c
 	return true;
 }
 
-bool ProgressDlg::GenerateMenus(wxArrayPtrVoid& menuVobs, int menuSubSteps, DVD* dvd) {
+bool ProgressDlg::GenerateMenus(wxArrayPtrVoid& menuVobs, wxArrayInt& menuWSTypes, int menuSubSteps, DVD* dvd) {
 	if (menuVobs.Count() == 0)
 		return true;
 	if (IsCanceled())
@@ -583,7 +617,8 @@ bool ProgressDlg::GenerateMenus(wxArrayPtrVoid& menuVobs, int menuSubSteps, DVD*
 	for (int i = 0; i < (int) menuVobs.Count(); i++) {
 		AddDetailMsg(wxString::Format(_("Generating menu %d of %d"), i + 1, menuVobs.Count()));
 		Vob* vob = (Vob*) menuVobs[i];
-		if (!GenerateMenu(vob->GetMenu(), vob->GetTmpFilename(), dvd->GetAudioFormat(),
+		WidescreenType wsType = (WidescreenType) menuWSTypes[i];
+		if (!GenerateMenu(vob->GetMenu(), wsType, vob->GetTmpFilename(), dvd->GetAudioFormat(),
 				vob->GetAudioFilenames().size() > 0 ? vob->GetAudioFilenames()[0] : wxT(""),
 				dvd->GetVideoBitrate()))
 			return false;
@@ -604,8 +639,7 @@ bool ProgressDlg::Transcode(wxArrayPtrVoid& titleVobs, int titleSubSteps, DVD* d
 		wxString cacheFile = m_cache->Add(vob, dvd);
 		AddDetailMsg(wxT("Add file to cache:") + cacheFile);
 		vob->SetTmpFilename(cacheFile);
-		if (!Transcode(vob->GetStreams(), vob->GetFilename(), vob->GetAudioFilenames(),
-				vob->GetTmpFilename(), dvd->GetVideoBitrate())) {
+		if (!Transcode(vob, dvd->GetVideoBitrate(), s_config.GetUseMplex())) {
 			wxRemoveFile(vob->GetTmpFilename());
 			return false;
 		}
@@ -775,7 +809,7 @@ or open the following directory with your DVD player: ");
 				if (cmd.Find(wxT("wmplayer")) >= 0)
 					cmd += wxT(" \"$DIR\\VIDEO_TS\\VIDEO_TS.VOB\"");
 				else
-					cmd += wxT(" \"dvd:/$DIR\"");
+					cmd += wxT(" \"dvd://$DIR\"");
 			}
 			cmd.Replace(wxT("$DIR"), dvdOutDir.Mid(0, dvdOutDir.length() - 1));
 			if (!Exec(cmd))
@@ -805,9 +839,13 @@ long GetFreeSpaceOn(wxString dir) {
 	return (long) (pFree.ToDouble() / 1024);
 }
 
-bool ProgressDlg::CreateIsoImage(BurnDlg* burnDlg, DVD* dvd, const wxString& dvdOutDir,
-		const wxString& tmpDir) {
-	if (burnDlg->DoCreateIso() || (burnDlg->DoBurn() && burnDlg->DoAddECC())) {
+bool ProgressDlg::CreateIsoImage(BurnDlg* burnDlg, DVD* dvd, const wxString& dvdOutDir, const wxString& tmpDir) {
+#ifdef __WXMAC__
+	bool createISO = burnDlg->DoCreateIso() || burnDlg->DoBurn();
+#else
+	bool createISO = burnDlg->DoCreateIso() || (burnDlg->DoBurn() && burnDlg->DoAddECC());
+#endif
+	if (createISO)  {
 		if (IsCanceled())
 			return false;
 		wxString isoFile = burnDlg->DoCreateIso() ? burnDlg->GetIsoFile() : tmpDir + TMP_ISO;
@@ -815,7 +853,7 @@ bool ProgressDlg::CreateIsoImage(BurnDlg* burnDlg, DVD* dvd, const wxString& dvd
 		long size = 0;
 		wxString cmd = s_config.GetIsoSizeCmd();
 		cmd.Replace(_T("$DIR"), dvdOutDir.Mid(0, dvdOutDir.length() - 1));
-#ifdef __WXMSW__
+#if defined(__WXMSW__) || defined(__WXMAC__)
 		cmd = wxGetAppPath() + wxString(wxFILE_SEP_PATH) + cmd;
 #endif
 		wxArrayString output;
@@ -872,7 +910,21 @@ bool ProgressDlg::FormatDvd(BurnDlg* burnDlg) {
 	while (1) {
 		SetSubSteps(100);
 		wxString cmd = s_config.GetFormatCmd();
-		cmd.Replace(_T("$DEV"), burnDlg->GetDevice());
+		wxString device = burnDlg->GetDevice();
+#ifdef __WXMAC__
+		if (device.Mid(0,5) != wxT("/dev/")) {
+			while (!IsMediaPresent(device) || !IsMediaErasable(device)) {
+				if (IsMediaBlank(device))
+					return true;
+				if (wxMessageBox(wxString::Format(_("Please insert erasable DVD into the device %s."),
+						GetDeviceName(device).c_str()), _("Burn"), wxOK|wxCANCEL, this) == wxCANCEL) {
+					AddSummaryMsg(_("Aborted"), wxEmptyString, *wxRED);
+					return false;
+				}
+			}
+		}
+#endif		
+		cmd.Replace(wxT("$DEV"), device);
 		if (!Exec(cmd)) {
 			int repeat = wxMessageBox(_("Formatting DVD-RW failed. Try again?"),
 			_("Burn"), wxYES_NO|wxCANCEL | wxICON_QUESTION, this);
@@ -892,8 +944,7 @@ bool ProgressDlg::FormatDvd(BurnDlg* burnDlg) {
 	return true;
 }
 
-bool ProgressDlg::BurnDvd(BurnDlg* burnDlg, DVD* dvd, const wxString& dvdOutDir,
-		const wxString& tmpDir) {
+bool ProgressDlg::BurnDvd(BurnDlg* burnDlg, DVD* dvd, const wxString& dvdOutDir, const wxString& tmpDir) {
 	if (!burnDlg->DoBurn())
 		return true;
 	if (IsCanceled())
@@ -901,8 +952,21 @@ bool ProgressDlg::BurnDvd(BurnDlg* burnDlg, DVD* dvd, const wxString& dvdOutDir,
 	AddSummaryMsg(_("Burning"));
 	SetSubSteps(100);
 	// check disc
-	wxString cmd = s_config.GetBurnScanCmd();
-	cmd.Replace(_T("$DEVICE"), burnDlg->GetDevice());
+	wxString device = burnDlg->GetDevice();
+	wxString cmd;
+#ifdef __WXMAC__
+	while (!IsMediaPresent(device)) {
+		if (wxMessageBox(wxString::Format(_("Please insert empty DVD into the device %s."),
+				GetDeviceName(device).c_str()), _("Burn"), wxOK|wxCANCEL, this) == wxCANCEL) {
+		   AddSummaryMsg(_("Aborted"), wxEmptyString, *wxRED);
+		   return false;
+		}
+	}
+	long discSize = GetMediaSpaceFree(device); // size in 2048 blocks
+	AddDetailMsg(wxString::Format(wxT("Disc size: %d MB"), discSize / 512));
+#else
+	cmd = s_config.GetBurnScanCmd();
+	cmd.Replace(wxT("$DEVICE"), device);
 #ifdef __WXMSW__
 	cmd = wxGetAppPath() + wxString(wxFILE_SEP_PATH) + cmd;
 #endif
@@ -910,14 +974,14 @@ bool ProgressDlg::BurnDvd(BurnDlg* burnDlg, DVD* dvd, const wxString& dvdOutDir,
 	while (true) {
 		if (wxExecute(cmd, output, wxEXEC_SYNC | wxEXEC_NODISABLE) == 0)
 			break;
-		if (wxMessageBox(wxString::Format(_("Please insert empty DVD into the device %s."), burnDlg->GetDevice().c_str()),
+		if (wxMessageBox(wxString::Format(_("Please insert empty DVD into the device %s."), device.c_str()),
 				_("Burn"), wxOK|wxCANCEL, this) == wxCANCEL) {
 			AddSummaryMsg(_("Aborted"), wxEmptyString, *wxRED);
 			return false;
 		}
 	}
 	// get disc size
-	long discSize = 0;
+	long discSize = 0; // size in 2048 blocks
 	for (unsigned int i = 0; i < output.Count(); i++) {
 		if (output[i].length() > 12 && output[i].SubString(1, 12) == wxT("Free Blocks:")) {
 			wxString discSizeStr = output[i].AfterFirst(wxT(':')).Trim(false).BeforeFirst(wxT('*'));
@@ -928,14 +992,20 @@ bool ProgressDlg::BurnDvd(BurnDlg* burnDlg, DVD* dvd, const wxString& dvdOutDir,
 	}
 	if (discSize < 2290000)
 		discSize = 2295104;
+#endif
 	// check size
 	long size = 0;
-	if (burnDlg->DoAddECC()) {
+#ifdef __WXMAC__
+	bool burnIso = true;
+#else
+	bool burnIso = burnDlg->DoAddECC();
+#endif
+	if (burnIso) {
 		size = wxFile(tmpDir + TMP_ISO).Length() / 2048; // size in 2048 blocks
 	} else {
 		cmd = s_config.GetIsoSizeCmd();
 		cmd.Replace(_T("$DIR"), dvdOutDir.Mid(0, dvdOutDir.length() - 1));
-#ifdef __WXMSW__
+#if defined(__WXMSW__) || defined(__WXMAC__)
 		cmd = wxGetAppPath() + wxString(wxFILE_SEP_PATH) + cmd;
 #endif
 		wxArrayString output;
@@ -952,7 +1022,7 @@ bool ProgressDlg::BurnDvd(BurnDlg* burnDlg, DVD* dvd, const wxString& dvdOutDir,
 		return false;
 	}
 	// burn
-	if (burnDlg->DoAddECC()) {
+	if (burnIso) {
 		cmd = s_config.GetBurnISOCmd();
 		cmd.Replace(_T("$FILE"), tmpDir + TMP_ISO);
 		// get iso size in sectors
@@ -963,14 +1033,14 @@ bool ProgressDlg::BurnDvd(BurnDlg* burnDlg, DVD* dvd, const wxString& dvdOutDir,
 		cmd.Replace(_T("$DIR"), dvdOutDir.Mid(0, dvdOutDir.length() - 1));
 	}
 	cmd.Replace(_T("$VOL_ID"), dvd->GetLabel());
-	cmd.Replace(_T("$DEV"), burnDlg->GetDevice());
+	cmd.Replace(_T("$DEV"), device);
 	wxString speedStr;
 	if (burnDlg->GetSpeed() > 0) {
 		speedStr = s_config.GetBurnSpeedOpt();
 		speedStr.Replace(_T("$SPEED"), wxString::Format(_T("%d"), burnDlg->GetSpeed()));
 	}
 	cmd.Replace(_T("$SPEEDSTR"), speedStr);
-	ProgressExecute exec(this, wxT(".*"));
+	BurnExecute exec(this, wxT(".*"));
 	if (!exec.Execute(cmd)) {
 		Failed();
 		return false;
@@ -979,25 +1049,21 @@ bool ProgressDlg::BurnDvd(BurnDlg* burnDlg, DVD* dvd, const wxString& dvdOutDir,
 	return true;
 }
 
-bool ProgressDlg::Transcode(const StreamArray& streams, const wxString& mpegFile,
-		const wxArrayString& audioFiles, const wxString& vobFile, int videoBitrate) {
+bool ProgressDlg::Transcode(Vob* vob, int videoBitrate, bool useMplex) {
 	if (IsCanceled())
 		return false;
 	int subStep = GetSubStep();
-	AddDetailMsg(_("Transcode video file: ") + mpegFile);
-	bool disableAudio = streams.GetCount() > 1 + audioFiles.GetCount()
-		&& streams[1]->GetDestinationFormat() == afNONE;
-	if (disableAudio) {
-		wxLogMessage(wxT("disableAudio"));
-	}
+	AddDetailMsg(_("Transcode video file: ") + vob->GetFilename());
+	bool disableAudio = vob->GetStreams().GetCount() > 1 + vob->GetAudioFilenames().GetCount()
+		&& vob->GetStreams()[1]->GetType() == stAUDIO && vob->GetStreams()[1]->GetDestinationFormat() == afNONE;
 	wxFfmpegMediaTranscoder transcoder(s_config.GetThreadCount());
-	if (!transcoder.AddInputFile(mpegFile, false, disableAudio)) {
-		Failed(wxT("Error by transcoding of ") + mpegFile);
+	if (!transcoder.AddInputFile(vob->GetFilename(), false, disableAudio)) {
+		Failed(wxT("Error by transcoding of ") + vob->GetFilename());
 		return false;
 	}
-	for (int i = 0; i < (int)audioFiles.GetCount(); i++) {
-		if (!transcoder.AddInputFile(audioFiles[i])) {
-			Failed(wxT("Error by transcoding of ") + audioFiles[i]);
+	for (int i = 0; i < (int)vob->GetAudioFilenames().GetCount(); i++) {
+		if (!transcoder.AddInputFile(vob->GetAudioFilenames()[i])) {
+			Failed(wxT("Error by transcoding of ") + vob->GetAudioFilenames()[i]);
 			return false;
 		}
 	}
@@ -1005,8 +1071,8 @@ bool ProgressDlg::Transcode(const StreamArray& streams, const wxString& mpegFile
 	VideoFormat videoFormat = vfCOPY;
 	wxArrayInt audioFormats;
 	wxArrayInt subtitleFormats;
-	for (unsigned int stIdx = 0; stIdx < streams.GetCount(); stIdx++) {
-		Stream* stream = streams[stIdx];
+	for (unsigned int stIdx = 0; stIdx < vob->GetStreams().GetCount(); stIdx++) {
+		Stream* stream = vob->GetStreams()[stIdx];
 		switch (stream->GetType()) {
 		case stVIDEO:
 			videoFormat = stream->GetVideoFormat();
@@ -1021,28 +1087,42 @@ bool ProgressDlg::Transcode(const StreamArray& streams, const wxString& mpegFile
 			break;
 		}
 	}
-	if (s_config.GetUseMplex()) {
-		wxString videoFile = vobFile + wxT(".m2v");
+	transcoder.SetInterlaced(vob->GetInterlaced());
+	transcoder.SetFirstField(vob->GetFirstField());
+	if (vob->GetPad().size() == 4
+			&& vob->GetPad()[0] + vob->GetPad()[1] + vob->GetPad()[2] + vob->GetPad()[3] > 0) {
+		wxLogMessage(wxT("pad: %d %d %d %d"),
+				vob->GetPad()[0], vob->GetPad()[1], vob->GetPad()[2], vob->GetPad()[3]);
+		transcoder.SetPad(vob->GetPad());
+	}
+	if (vob->GetCrop().size() == 4
+				&& vob->GetCrop()[0] + vob->GetCrop()[1] + vob->GetCrop()[2] + vob->GetCrop()[3] > 0) {
+		wxLogMessage(wxT("crop: %d %d %d %d"),
+				vob->GetCrop()[0], vob->GetCrop()[1], vob->GetCrop()[2], vob->GetCrop()[3]);
+		transcoder.SetCrop(vob->GetCrop());
+	}
+	if (useMplex) {
+		wxString videoFile = vob->GetTmpFilename() + wxT(".m2v");
 		if (wxFileExists(videoFile) && !wxRemoveFile(videoFile)) {
 			wxLogError(wxString::Format(_("Can't remove file '%s'"), videoFile.c_str()));
 		}
 		if (!transcoder.SetOutputFile(videoFile, videoFormat, afNONE, sfNONE,
 				videoBitrate, s_config.GetVbr(), s_config.GetAudioBitrate())) {
-			Failed(_("Error transcoding of ") + mpegFile);
+			Failed(_("Error transcoding of ") + vob->GetFilename());
 			return false;
 		}
 		wxArrayString audioFiles;
 		for (unsigned int audioIdx = 0; audioIdx < audioFormats.GetCount(); audioIdx++) {
 			if (audioFormats[audioIdx] == afNONE)
 				continue;
-			wxString audioFile = vobFile + wxString::Format(wxT(".audio%d"), audioIdx);
+			wxString audioFile = vob->GetTmpFilename() + wxString::Format(wxT(".audio%d"), audioIdx);
 			audioFiles.Add(audioFile);
 			if (wxFileExists(audioFile) && !wxRemoveFile(audioFile)) {
 				wxLogError(wxString::Format(_("Can't remove file '%s'"), audioFile.c_str()));
 			}
 			if (!transcoder.SetOutputFile(audioFile, vfNONE, (AudioFormat) audioFormats[audioIdx], sfNONE,
-						videoBitrate, s_config.GetVbr(), s_config.GetAudioBitrate())) {
-				Failed(_("Error transcoding of ") + mpegFile);
+					videoBitrate, s_config.GetVbr(), s_config.GetAudioBitrate())) {
+				Failed(_("Error transcoding of ") + vob->GetFilename());
 				return false;
 			}
 		}
@@ -1050,11 +1130,11 @@ bool ProgressDlg::Transcode(const StreamArray& streams, const wxString& mpegFile
 			wxRemoveFile(videoFile);
 			for (unsigned int audioIdx = 0; audioIdx < audioFiles.GetCount(); audioIdx++)
 				wxRemoveFile(audioFiles[audioIdx]);
-			Failed(_("Error transcoding of ") + mpegFile);
+			Failed(_("Error transcoding of ") + vob->GetFilename());
 			return false;
 		}
 		SetSubStep(subStep+150);
-		if (!Multiplex(videoFile, audioFiles, vobFile)) {
+		if (!Multiplex(videoFile, audioFiles, vob->GetTmpFilename())) {
 			wxRemoveFile(videoFile);
 			for (unsigned int audioIdx = 0; audioIdx < audioFiles.GetCount(); audioIdx++)
 				wxRemoveFile(audioFiles[audioIdx]);
@@ -1070,15 +1150,14 @@ bool ProgressDlg::Transcode(const StreamArray& streams, const wxString& mpegFile
 			}
 		}
 	} else {
-		if (!transcoder.SetOutputFile(vobFile, videoFormat, audioFormats, subtitleFormats,
+		if (!transcoder.SetOutputFile(vob->GetTmpFilename(), videoFormat, audioFormats, subtitleFormats,
 				videoBitrate, s_config.GetVbr(), s_config.GetAudioBitrate())
 				|| !transcoder.Run(m_cancel)) {
-			wxRemoveFile(vobFile);
-			Failed(_("Error transcoding of ") + mpegFile);
+			wxRemoveFile(vob->GetTmpFilename());
+			Failed(_("Error transcoding of ") + vob->GetFilename());
 			return false;
 		}
 	}
-	transcoder.End();
 	SetSubStep(subStep+200);
 	return true;
 }
@@ -1103,7 +1182,7 @@ bool ProgressDlg::Multiplex(const wxString& videoFile, const wxArrayString& audi
 	return true;
 }
 
-bool ProgressDlg::GenerateMenu(Menu* menu, const wxString& menuFile,
+bool ProgressDlg::GenerateMenu(Menu* menu, WidescreenType widescreenType, const wxString& menuFile,
 		AudioFormat audioFormat, wxString audioFile, int videoBitrate) {
 	if (IsCanceled())
 		return false;
@@ -1117,17 +1196,10 @@ bool ProgressDlg::GenerateMenu(Menu* menu, const wxString& menuFile,
 	bool videoMenu = menu->HasVideoBackground();
 	wxYield();
 
-	// save background and buttons
+	// generate menu background and buttons
 	AddDetailMsg(_("Prepare"));
 	menu->SetTransparentColor();
-	wxImage* images = menu->GetImages();
-	wxImage bgImage = images[0];
-	images[1].SaveFile(btFile);
-	images[2].SaveFile(hlFile);
-	images[3].SaveFile(selFile);
-	delete[] images;
-	// save spumux
-	menu->SaveSpumux(spuFile, btFile, hlFile, selFile);
+	wxImage bgImage = menu->GetBackgroundImage();
 	IncSubStep(10);
 
 	// convert jpeg to mpeg
@@ -1163,8 +1235,8 @@ bool ProgressDlg::GenerateMenu(Menu* menu, const wxString& menuFile,
 					}
 				}
 			}
-			if (!Transcode(menuVob.GetStreams(), menuVob.GetFilename(), menuVob.GetAudioFilenames(),
-					mpegFile, videoBitrate))
+			menuVob.SetTmpFilename(mpegFile);
+			if (!Transcode(&menuVob, videoBitrate, s_config.GetUseMplexForMenus()))
 				return false;
 		} else
 			mpegFile = menu->GetBackground();
@@ -1185,8 +1257,7 @@ bool ProgressDlg::GenerateMenu(Menu* menu, const wxString& menuFile,
 		}
 		wxFfmpegMediaEncoder ffmpeg(s_config.GetThreadCount());
 		if (!ffmpeg.BeginEncode(audioFile.length() ? m2vFile : mpegFile, menu->GetVideoFormat(),
-				audioFile.length() ? afNONE : audioFormat,
-				menu->GetAspectRatio(), s_config.GetMenuVideoBitrate())
+				audioFile.length() ? afNONE : audioFormat, menu->GetAspectRatio(), s_config.GetMenuVideoBitrate())
 				|| !ffmpeg.EncodeImage(bgImage, frameCount, &m_cancel)) {
 			Failed(_("Error creation of menu"));
 			return false;
@@ -1216,8 +1287,8 @@ bool ProgressDlg::GenerateMenu(Menu* menu, const wxString& menuFile,
 					}
 				}
 			}
-			if (!Transcode(menuVob.GetStreams(), menuVob.GetFilename(), menuVob.GetAudioFilenames(),
-					mpegFile, videoBitrate))
+			menuVob.SetTmpFilename(mpegFile);
+			if (!Transcode(&menuVob, videoBitrate, s_config.GetUseMplexForMenus()))
 				return false;
 			if (s_config.GetRemoveTempFiles())
 				if (!DeleteFile(m2vFile))
@@ -1228,20 +1299,42 @@ bool ProgressDlg::GenerateMenu(Menu* menu, const wxString& menuFile,
 	//spumux
 	if (IsCanceled())
 		return false;
-	AddDetailMsg(_("Multiplexing subtitles (buttons) into mpeg"));
-	wxString cmd = s_config.GetSpumuxCmd();
-	cmd.Replace(wxT("$FILE_CONF"), spuFile);
-	cmd.Replace(wxT("$STREAM"), wxT("0"));
-	if (!Exec(cmd, mpegFile, menuFile)) {
-		Failed();
-		return false;
-	}
-	if (s_config.GetRemoveTempFiles()) {
-		if ((!videoMenu || mpegFile != menu->GetBackground()) && !DeleteFile(mpegFile))
-			return false;
-		if (!DeleteFile(btFile) || !DeleteFile(hlFile) || !DeleteFile(selFile)
-				|| !DeleteFile(spuFile))
+	AddDetailMsg(_("Multiplexing subpictures into mpeg"));
+	int stCount = menu->GetAspectRatio() == ar4_3 ? 1 : widescreenType != wtAUTO ? 2 : 3;
+	for (int stIdx = 0; stIdx < stCount; stIdx++) {
+		if (stIdx > 0) {
+			if (mpegFile == menu->GetBackground())
+				mpegFile = menuFile + _T("_bg.mpg");
+			if (!wxRenameFile(menuFile, mpegFile, false)) {
+				Failed(wxString::Format(_("Can't rename file '%s' in '%s'"), menuFile.c_str(), mpegFile.c_str()));
+				return false;
+			}
+		}
+		SubStreamMode mode = menu->GetAspectRatio() == ar4_3 ? ssmNORMAL : ssmWIDESCREEN;
+		if (stIdx == 1)
+			mode = widescreenType == wtNOLETTERBOX ? ssmPANSCAN : ssmLETTERBOX;
+		else if (stIdx == 2)
+			mode = ssmPANSCAN;
+		wxImage* images = menu->GetSubPictures(mode);
+		images[0].SaveFile(btFile);
+		images[1].SaveFile(hlFile);
+		images[2].SaveFile(selFile);
+		delete[] images;
+		// save spumux
+		menu->SaveSpumux(spuFile, mode, btFile, hlFile, selFile);
+		wxString cmd = s_config.GetSpumuxCmd();
+		cmd.Replace(wxT("$FILE_CONF"), spuFile);
+		cmd.Replace(wxT("$STREAM"), wxString::Format(wxT("%d"), stIdx));
+		if (!Exec(cmd, mpegFile, menuFile)) {
+			Failed();
 			return false;
+		}
+		if (s_config.GetRemoveTempFiles() || stIdx + 1 < stCount) {
+			if ((!videoMenu || mpegFile != menu->GetBackground()) && !DeleteFile(mpegFile))
+				return false;
+			if (!DeleteFile(btFile) || !DeleteFile(hlFile) || !DeleteFile(selFile) || !DeleteFile(spuFile))
+				return false;
+		}
 	}
 	IncSubStep(50);
 
