@@ -3,7 +3,7 @@
 // Purpose:     Progress of generation/burning dialog
 // Author:      Alex Thuering
 // Created:     14.08.2004
-// RCS-ID:      $Id: ProgressDlg.h,v 1.26 2010/05/09 18:19:54 ntalex Exp $
+// RCS-ID:      $Id: ProgressDlg.h,v 1.29 2010/09/16 18:29:26 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -117,13 +117,13 @@ private:
 
 	void Run(BurnDlg* burnDlg, DVD* dvd);
 	
-	int GetMenuSubSteps(wxArrayPtrVoid& menuVobs, DVD* dvd, wxString dvdTmpDir);
+	int GetMenuSubSteps(wxArrayPtrVoid& menuVobs, wxArrayInt& menuWSTypes, DVD* dvd, wxString dvdTmpDir);
 	int GetTitleSubSteps(wxArrayPtrVoid& titleVobs, DVD* dvd, Cache* cache);
 	int GetSlideshowSubSteps(wxArrayPtrVoid& slideshowVobs, DVD* dvd, wxString dvdTmpDir);
 	int GetSubtitleSubSteps(wxArrayPtrVoid& subtitleVobs, DVD* dvd, wxString dvdTmpDir);
 	bool CleanTemp(const wxString& tmpDir, const wxString& dvdTmpDir, const wxString& dvdOutDir);
 	
-	bool GenerateMenus(wxArrayPtrVoid& menuVobs, int menuSubSteps, DVD* dvd);
+	bool GenerateMenus(wxArrayPtrVoid& menuVobs, wxArrayInt& menuWSTypes, int menuSubSteps, DVD* dvd);
 	bool Transcode(wxArrayPtrVoid& titleVobs, int titleSubSteps, DVD* dvd);
 	bool Multiplex(const wxString& videoFile, const wxArrayString& audioFiles, const wxString& vobFile);
 	bool GenerateSlideshow(wxArrayPtrVoid& slideshowVobs, int slideshowSubSteps, DVD* dvd);
@@ -135,11 +135,10 @@ private:
 	bool FormatDvd(BurnDlg* burnDlg);
 	bool BurnDvd(BurnDlg* burnDlg, DVD* dvd, const wxString& dvdOutDir, const wxString& tmpDir);
 	
-	bool GenerateMenu(Menu* menu, const wxString& menuFile, AudioFormat audioFormat, wxString audioFile,
-			int videoBitrate);
+	bool GenerateMenu(Menu* menu, WidescreenType widescreenType, const wxString& menuFile, AudioFormat audioFormat,
+			wxString audioFile, int videoBitrate);
     bool GenerateSlideshow(Slideshow* slideshow, const wxString& vobFile, AudioFormat audioFormat);
-	bool Transcode(const StreamArray& streams, const wxString& mpegFile,
-			const wxArrayString& audioFiles, const wxString& vobFile, int videoBitrate);
+	bool Transcode(Vob* vob, int videoBitrate, bool useMplex);
 	bool MultiplexSubtitles(const wxString& vobFile, TextSub* textSub, unsigned int streamIdx,
 			const wxString& resultFile);
 	/** removes given file and print error message if it failed */
