@@ -3,26 +3,24 @@
 // Purpose:     DVD title properties dialog
 // Author:      Alex Thuering
 // Created:     31.01.2004
-// RCS-ID:      $Id: TitlePropDlg.cpp,v 1.42 2010/04/18 15:47:04 ntalex Exp $
+// RCS-ID:      $Id: TitlePropDlg.cpp,v 1.47 2010/10/10 18:39:55 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
 
 #include "TitlePropDlg.h"
 #include "TitlesetManager.h"
-#include "AVPropDlg.h"
-#include "SubtitlePropDlg.h"
 #include "MPEG.h"
 #include "Config.h"
 #include <wxVillaLib/utils.h>
 #include <wx/tokenzr.h>
+#include <wx/regex.h>
 #include "rc/add.png.h"
 #include "rc/remove.png.h"
 #include "rc/preferences.png.h"
 
 enum {
-	AV_BT_ID = 7850,
-	DO_NOT_TRANSCODE_CB_ID,
+	DO_NOT_TRANSCODE_CB_ID = 7850,
 	VOB_BOX_ID,
 	ADD_BT_ID,
 	REMOVE_BT_ID,
@@ -34,7 +32,6 @@ BEGIN_EVENT_TABLE(TitlePropDlg, wxPropDlg)
 	EVT_BUTTON(ADD_BT_ID, TitlePropDlg::OnAddBt)
 	EVT_BUTTON(REMOVE_BT_ID, TitlePropDlg::OnRemoveBt)
 	EVT_BUTTON(PROP_BT_ID, TitlePropDlg::OnPropBt)
-	EVT_BUTTON(AV_BT_ID, TitlePropDlg::OnAV)
 	EVT_CHECKBOX(DO_NOT_TRANSCODE_CB_ID, TitlePropDlg::OnDoNotTranscodeCheck)
 END_EVENT_TABLE()
 
@@ -42,14 +39,9 @@ TitlePropDlg::TitlePropDlg(wxWindow* parent, DVD* dvd, int tsi, int pgci, int vo
 	m_dvd = dvd;
 	m_tsi = tsi;
 	m_pgci = pgci;
-	PgcArray& pgcs = m_dvd->GetPgcArray(tsi, false);
-	m_audioLangCodes = &pgcs.GetAudioLangCodes();
-	m_subtitleLangCodes = &pgcs.GetSubtitleLangCodes();
-	m_video = &pgcs.GetVideo();
-	m_pgc = pgcs[pgci];
+	m_pgcs = &m_dvd->GetPgcArray(tsi, false);
+	m_pgc = (*m_pgcs)[pgci];
 	m_vob = m_pgc->GetVobs()[vobi];
-	if (!m_vob->GetSlideshow())
-		m_vob = new Vob(*m_vob);
 	m_vobi = vobi;
 	m_vobListBox = NULL;
 	m_doNotTranscodeCheckBox = NULL;
@@ -90,7 +82,7 @@ void TitlePropDlg::CreatePropPanel(wxSizer* sizer) {
 		wxBoxSizer* vobPane = new wxBoxSizer(wxHORIZONTAL);
 		grpSizer->Add(vobPane, 1, wxEXPAND);
 		
-		m_vobListBox = new VobListBox(this, VOB_BOX_ID, m_vob, *m_audioLangCodes, *m_subtitleLangCodes, m_dvd);
+		m_vobListBox = new VobListBox(this, VOB_BOX_ID, m_vob, m_pgcs, m_dvd);
 		vobPane->Add(m_vobListBox, 1, wxEXPAND);
 		
 		wxBoxSizer* buttonPane = new wxBoxSizer(wxVERTICAL);
@@ -148,11 +140,11 @@ wxArrayString TitlePropDlg::GetPreCommands() {
 	for (int tsi=0; tsi<(int)m_dvd->GetTitlesets().Count(); tsi++) {
 		Titleset* ts = m_dvd->GetTitlesets()[tsi];
 		if (tsi == m_tsi) { // same titleset
+			if (ts->GetMenus().GetCount())
+				commands.Add(wxT("call menu;"));
 			StringSet entrySet = ts->GetMenus().GetEntries();
 			for (StringSet::const_iterator entry = entrySet.begin(); entry != entrySet.end(); entry++)
-				if (*entry == wxT("root"))
-					commands.Add(wxT("call menu;"));
-				else
+				if (*entry != wxT("root"))
 					commands.Add(wxString::Format(wxT("call menu entry %s;"), entry->c_str()));
 		}
 		if (m_dvd->IsJumppadsEnabled()) {
@@ -172,13 +164,17 @@ wxArrayString TitlePropDlg::GetPreCommands() {
 
 bool TitlePropDlg::SetValues() {
 	// check commands
+	wxString preCommands = GetString(4);
+	wxString postCommands = GetString(5);
 	DVDAction action(false, BUTTON_ACTION);
-	action.SetCustom(GetString(4));
-	if (!action.IsValid(m_dvd, m_tsi, m_pgci, false, wxT(""), true, false))
+	action.SetCustom(preCommands);
+	if (preCommands.length() && !action.IsValid(m_dvd, m_tsi, m_pgci, false, wxT(""), true, false)) {
 		return false;
-	action.SetCustom(GetString(5));
-	if (!action.IsValid(m_dvd, m_tsi, m_pgci, false, wxT(""), true, false))
+	}
+	action.SetCustom(postCommands);
+	if (postCommands.length() && !action.IsValid(m_dvd, m_tsi, m_pgci, false, wxT(""), true, false)) {
 		return false;
+	}
 	
 	int n = 0;
 	if (m_vob->GetSlideshow()) {
@@ -187,23 +183,28 @@ bool TitlePropDlg::SetValues() {
 		slideshow->SetAspectRatio((AspectRatio) (GetInt(n++)+1));
 		slideshow->SetDuration(GetInt(n++));
 	} else {
+		// check chapters
+		wxString chapters = GetString(1);
+		wxStringTokenizer tkz(chapters, _T(","));
+		while (tkz.HasMoreTokens()) {
+			wxString token = tkz.GetNextToken().Strip(wxString::both);
+			wxRegEx re(wxT("^(([[:digit:]]+:)?[[:digit:]][[:digit:]]?:)?[[:digit:]][[:digit:]]?(\\.[[:digit:]]+)?$"));
+			if (!re.Matches(token)) {
+				wxLogError(_("Invalid chapter timestamp '%s'"), token.c_str());
+				return false;
+			}
+		}
+		
 		// update vob
-		m_vobListBox->UpdateVob();
-		m_vob->SetDoNotTranscode(GetBool(n++) && m_vob->GetAudioFilenames().GetCount() == 0);
-		m_vob->SetChapters(GetString(n++));
-		m_vob->SetPause(GetInt(n++));
+		Vob* vob = m_vobListBox->SetValues();
+		vob->SetDoNotTranscode(GetBool(n++) && vob->GetAudioFilenames().GetCount() == 0);
+		vob->SetChapters(GetString(n++));
+		vob->SetPause(GetInt(n++));
 		// set new vob
-		m_pgc->GetVobs()[m_vobi] = m_vob;
-		// update audio language list
-		wxArrayString audioLangCodes = m_vobListBox->GetAudioLangCodes();
-		m_audioLangCodes->Clear();
-		for (wxArrayString::iterator it = audioLangCodes.begin(); it != audioLangCodes.end(); it++)
-			m_audioLangCodes->Add(*it);
-		// update subtitle language list
-		wxArrayString subtitleLangCodes = m_vobListBox->GetSubtitleLangCodes();
-		m_subtitleLangCodes->Clear();
-		for (wxArrayString::iterator it = subtitleLangCodes.begin(); it != subtitleLangCodes.end(); it++)
-			m_subtitleLangCodes->Add(*it);
+		m_pgc->GetVobs()[m_vobi] = vob;
+		m_vob = vob;
+		// update video parameters
+		m_pgcs->GetVideo().SetAspect(m_vobListBox->GetAspectRatio());
 	}
 	m_pgc->SetPalette(GetString(n++));
 	m_pgc->SetPreCommands(GetString(n++));
@@ -211,23 +212,9 @@ bool TitlePropDlg::SetValues() {
 	return true;
 }
 
-void TitlePropDlg::CreateButtonPane(wxSizer* sizer, bool resetButton, bool dontShowCheckbox) {
-	wxBoxSizer* buttonPane = new wxBoxSizer(wxHORIZONTAL);
-	wxButton* avBt = new wxButton(this, AV_BT_ID, _("Video..."));
-	buttonPane->Add(avBt, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL);
-	buttonPane->Add(10, 10, 1, wxEXPAND);
-	wxButton* okBt = new wxButton(this, wxID_OK, _("OK"));
-	okBt->SetDefault();
-	buttonPane->Add(okBt, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL);
-	buttonPane->Add(8, 0);
-	wxButton* cancelBt = new wxButton(this, wxID_CANCEL, _("Cancel"));
-	buttonPane->Add(cancelBt, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL);
-	sizer->Add(buttonPane, 0, wxEXPAND|wxLEFT|wxRIGHT|wxBOTTOM, 10);
-}
-
 void TitlePropDlg::OnSelectItem(wxCommandEvent& event) {
 	m_delBt->Enable(m_vobListBox->GetSelection() > 0);
-	m_propBt->Enable(m_vobListBox->GetSelection() >= 1 + (int) m_vob->GetAudioFilenames().GetCount());
+	m_propBt->Enable(m_vobListBox->HasPropDialog());
 }
 
 void TitlePropDlg::OnAddBt(wxCommandEvent& event) {
@@ -258,21 +245,8 @@ void TitlePropDlg::OnRemoveBt(wxCommandEvent& event) {
 }
 
 void TitlePropDlg::OnPropBt(wxCommandEvent& event) {
-	m_vobListBox->Update();
-	int idx = m_vobListBox->GetSelection() - 1 - m_vob->GetAudioFilenames().GetCount();
-	wxString subtitleLanguage = m_vobListBox->GetSubtitleLangCodes()[idx];
-	SubtitlePropDlg dialog(this, m_vob->GetSubtitles()[idx], &subtitleLanguage);
-	if (dialog.ShowModal() == wxID_OK) {
-		m_vobListBox->GetSubtitleLangCodes()[idx] = subtitleLanguage;
-	}
-}
-
-void TitlePropDlg::OnAV(wxCommandEvent& event) {
-	AVPropDlg propDlg(this, m_video, false);
-	if (propDlg.ShowModal() == wxID_OK) {
-		wxCommandEvent evt(EVT_COMMAND_DVD_CHANGED, this->GetId());
-		GetEventHandler()->ProcessEvent(evt);
-	}
+	m_vobListBox->ShowPropDialog();
+	UpdateDoNotTranscodeCheck();
 }
 
 void TitlePropDlg::OnDoNotTranscodeCheck(wxCommandEvent& event) {
@@ -282,7 +256,6 @@ void TitlePropDlg::OnDoNotTranscodeCheck(wxCommandEvent& event) {
 void TitlePropDlg::UpdateDoNotTranscodeCheck() {
 	if (m_doNotTranscodeCheckBox == NULL)
 		return;
-	if (m_vobListBox->HasAudioFiles())
-		m_doNotTranscodeCheckBox->SetValue(false);
+	m_doNotTranscodeCheckBox->SetValue(m_vobListBox->GetDoNotTranscode());
 	m_doNotTranscodeCheckBox->Enable(!m_vobListBox->HasAudioFiles() && MPEG::HasNavPacks(m_vob->GetFilename()));
 }
