@@ -3,7 +3,7 @@
 // Purpose:     DVD title properties dialog
 // Author:      Alex Thuering
 // Created:     31.01.2004
-// RCS-ID:      $Id: TitlePropDlg.h,v 1.15 2010/04/18 15:47:04 ntalex Exp $
+// RCS-ID:      $Id: TitlePropDlg.h,v 1.17 2010/10/10 18:39:55 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -25,9 +25,7 @@ private:
 	DVD* m_dvd;
 	int m_tsi;
 	int m_pgci;
-	wxArrayString* m_audioLangCodes;
-	wxArrayString* m_subtitleLangCodes;
-	Video* m_video;
+	PgcArray* m_pgcs;
 	Pgc* m_pgc;
 	Vob* m_vob;
 	int m_vobi;
@@ -38,14 +36,12 @@ private:
 	wxButton* m_propBt;
 	void CreatePropPanel(wxSizer* sizer);
 	bool SetValues();
-	void CreateButtonPane(wxSizer* sizer, bool resetButton = false, bool dontShowCheckbox = false);
 	/** Returns list of pre commands */
 	wxArrayString GetPreCommands();
 	void OnSelectItem(wxCommandEvent& event);
 	void OnAddBt(wxCommandEvent& event);
 	void OnRemoveBt(wxCommandEvent& event);
 	void OnPropBt(wxCommandEvent& event);
-	void OnAV(wxCommandEvent& event);
 	void OnDoNotTranscodeCheck(wxCommandEvent& event);
 	void UpdateDoNotTranscodeCheck();
 	DECLARE_EVENT_TABLE()
