@@ -3,7 +3,7 @@
 // Purpose:     Titleset Manager
 // Author:      Alex Thuering
 // Created:	27.01.2003
-// RCS-ID:      $Id: TitlesetManager.cpp,v 1.41 2010/05/09 18:19:22 ntalex Exp $
+// RCS-ID:      $Id: TitlesetManager.cpp,v 1.45 2010/09/05 10:56:08 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -69,7 +69,18 @@ public:
 			wxFfmpegMediaDecoder decoder;
 			if (!decoder.Load(filename))
 				return false;
-			m_pOwner->AddVideo(filename, defDrag == wxDragMove);
+			bool hasVideo = false;
+			bool hasAudio = false;
+			for (unsigned int i = 0; i < decoder.GetStreamCount(); i++) {
+				if (decoder.GetStreamType(i) == stVIDEO)
+					hasVideo = true;
+				else if (decoder.GetStreamType(i) == stAUDIO)
+					hasAudio = true;
+			}
+			if (hasVideo)
+				m_pOwner->AddVideo(filename, defDrag == wxDragMove);
+			else
+				m_pOwner->AddAudio(filename);
 			return true;
 		} else if (wxThumbnails::IsAudio(filename)) {
 			m_pOwner->AddAudio(filename);
@@ -186,10 +197,10 @@ wxBitmap TitlesetManager::GetThumbImage(wxThumb& thumb) {
 	if (!img.Ok()) {
 		int titleHeight = m_tTextHeight + 2;
 		if (vob->GetMenu()) {
-			img = wxBitmap(vob->GetMenu()->GetImage(m_tWidth, m_tHeight-titleHeight));
+			img = wxBitmap(vob->GetMenu()->GetImage(m_tWidth-8, m_tHeight-titleHeight));
 			thumb.SetBitmap(img);
 		} else if (vob->GetSlideshow()) {
-			img = wxBitmap(vob->GetSlideshow()->GetThumbImage(m_tWidth, m_tHeight-titleHeight));
+			img = wxBitmap(vob->GetSlideshow()->GetThumbImage(m_tWidth-8, m_tHeight-titleHeight));
 			thumb.SetBitmap(img);
 		} else
 			img = thumb.GetBitmap(this, m_tWidth, m_tHeight-titleHeight);
@@ -305,10 +316,10 @@ void TitlesetManager::DrawThumbnail(wxBitmap& bmp, wxThumb& thumb, int index) {
 	dc.SelectObject(wxNullBitmap);
 }
 
-int TitlesetManager::GetRowHeight(int begRow, int count) {
+int TitlesetManager::GetRowHeight(unsigned int begRow, unsigned int count) {
 	int capHeight = 0;
-	for (int i=begRow; i<begRow+count; i++)
-		if (i < (int) m_tCaptionHeight.GetCount())
+	for (unsigned int i=begRow; i<begRow+count; i++)
+		if (i < m_tCaptionHeight.GetCount())
 			capHeight += m_tCaptionHeight[i]>0 ? m_tCaptionHeight[i] : 1;
 	return (m_tHeight + m_tBorder)*count + capHeight*m_tTextHeight;
 }
@@ -450,7 +461,18 @@ bool TitlesetManager::AddVideo(const wxString& fname, bool createTitle, int tsi)
 		pgc->SetPostCommands(_T("call vmgm menu 1;"));
 		pgc->GetVobs().Add(vob);
 		pgcs.Add(pgc);
-		m_dvd->UpdateButtonImageFor(tsi, pgcs.GetCount() - 1);
+		if (!m_dvd->UpdateButtonImageFor(tsi, pgcs.GetCount() - 1) && pgcs.GetCount() > 4) {
+			// find menu to duplicate
+			PgcArray& menus = m_dvd->GetPgcArray(tsi, true);
+			for (unsigned int menuIdx = 0; menuIdx < menus.GetCount(); menuIdx++) {
+				Menu* menu = menus[menuIdx]->GetMenu();
+				if (menu && menu->GetButtonByJumpAction(-2, pgcs.GetCount() - 2)
+						&&  menu->GetButtonByJumpAction(-2, pgcs.GetCount() - 3)) {
+					DuplicateMenu(tsi, menuIdx);
+					break;
+				}
+			}
+		}
 	} else
 		pgc->GetVobs().Add(vob);
 	
@@ -637,6 +659,38 @@ void TitlesetManager::AddTitleset() {
 	AddMenu();
 }
 
+bool TitlesetManager::DuplicateMenu(int tsi, int pgci) {
+	Vob* vob = m_dvd->GetVob(tsi, true, pgci, 0);
+	if (!vob || !vob->GetMenu())
+		return false;
+	wxSvgXmlNode* menuXml = vob->GetMenu()->GetXML(DVDSTYLER_XML);
+	// add new menu
+	int pgci2 = m_dvd->AddMenu(tsi);
+	Menu* menu = m_dvd->GetMenu(tsi, pgci2);
+	menu->PutXML(menuXml);
+	delete menuXml;
+	// fix jump actions
+	int actionPgci = m_dvd->GetPgcArray(tsi, false).GetCount() - 1;
+	for (unsigned int obji = 0; obji < menu->GetObjectsCount(); obji++) {
+		MenuObject* obj = menu->GetObject(obji);
+		if (obj->IsButton() && !obj->GetAction().IsCustom() && obj->GetAction().GetTsi() == -2) {
+			if (!obj->GetAction().IsMenu() && !obj->GetAction().IsPlayAll()) {
+				obj->GetAction().SetPgci(actionPgci);
+				menu->UpdateButtonImageFor(-2, actionPgci, m_dvd);
+				actionPgci++;
+			} else if (obj->GetAction().IsMenu()) {
+				if (obj->GetAction().GetPgci() == pgci2)
+					obj->GetAction().SetPgci(pgci2 + 1);
+				else if (obj->GetAction().GetPgci() == 100)
+					obj->GetAction().SetPgci(pgci2 - 1);
+				else if (pgci2 > 2 && obj->GetAction().GetPgci() == pgci2 - 2)
+					obj->GetAction().SetPgci(pgci2 - 1);
+			}
+		}
+	}
+	return true;
+}
+
 void TitlesetManager::OnAddTitlesetUpdateUI(wxUpdateUIEvent& event) {
 	int cnt = m_dvd->GetTitlesets().Count();
 	event.Enable(cnt>0 && m_dvd->GetTitlesets()[cnt-1]->GetTitles().Count());
@@ -809,14 +863,15 @@ void TitlesetManager::OnProps(wxCommandEvent& WXUNUSED(event)) {
 	if (isMenu) { // menu
 		MenuPropDlg propDlg(this->GetParent(), m_dvd, tsi, pgci);
 		if (propDlg.ShowModal() == wxID_OK) {
-			SendMenuSelectedEvent(); // updated MenuEditor
+			SendMenuSelectedEvent(); // update MenuEditor
 			SendDvdChangedEvent();
+			UpdateItems();
 		}
 	} else { //title
 		TitlePropDlg propDlg(this->GetParent(), m_dvd, tsi, pgci, vobi);
 		if (propDlg.ShowModal() == wxID_OK) {
 			m_dvd->UpdateButtonImageFor(tsi, pgci);
-			SendMenuSelectedEvent(); // updated MenuEditor
+			SendMenuSelectedEvent(); // update MenuEditor
 			SendDvdChangedEvent();
 		}
 	}
@@ -837,12 +892,12 @@ void TitlesetManager::OnCopy(wxCommandEvent& WXUNUSED(event)) {
 
 void TitlesetManager::OnPaste(wxCommandEvent& WXUNUSED(event)) {
 	int tsi = GetSelected() >= 0 ? DVD::GetTsi(GetSelectedId()) : m_dvd->GetTitlesets().Count() - 1;
-	Menu* menu = new Menu(m_dvd, tsi, m_dvd->GetVideoFormat(), m_dvd->GetAspectRatio());
+	int pgci = m_dvd->AddMenu(tsi);
+	Menu* menu = m_dvd->GetMenu(tsi, pgci);
 	wxSvgXmlDocument doc;
 	if (!GetXmlFromClipboard(DATAFORMAT_MENU, doc))
 		return;
 	menu->PutXML(doc.GetRoot());
-	int pgci = m_dvd->AddMenu(menu, tsi);
 	UpdateItems();
 	SelectMenu(tsi, pgci);
 	SendDvdChangedEvent();
