@@ -3,7 +3,7 @@
 // Purpose:     Miscellaneous utilities
 // Author:      Alex Thuering
 // Created:		06.04.2008
-// RCS-ID:      $Id: Utils.h,v 1.2 2010/01/12 20:47:35 ntalex Exp $
+// RCS-ID:      $Id: Utils.h,v 1.3 2010/04/03 21:46:36 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -12,6 +12,7 @@
 #define UTILS_H
 
 #include <wxSVGXML/svgxml.h>
+#include <wx/arrstr.h>
 
 const wxString DATAFORMAT_ACTION =  wxT("application/dvdstyler-action");
 const wxString DATAFORMAT_MENU =  wxT("application/dvdstyler-menu");
@@ -31,5 +32,9 @@ bool GetXmlFromClipboard(wxString dataFormatId, wxSvgXmlDocument& doc);
  * Gets text from clipboard.
  */
 wxString GetTextFromClipboard();
+/**
+ * Sorts string arrays by value
+ */
+int Sort(wxArrayString& keys, wxArrayString& values, const wxString& selectedKey = wxEmptyString);
 
 #endif // UTILS_H
