@@ -3,13 +3,14 @@
 // Purpose:     The list box to display information about streams in given VOB
 // Author:      Alex Thuering
 // Created:     03.05.2009
-// RCS-ID:      $Id: VobListBox.cpp,v 1.10 2010/04/10 09:22:58 ntalex Exp $
+// RCS-ID:      $Id: VobListBox.cpp,v 1.14 2010/10/10 18:39:55 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
 
 #include "VobListBox.h"
 #include "Config.h"
+#include "VideoPropDlg.h"
 #include "SubtitlePropDlg.h"
 #include <wx/filedlg.h>
 #include <wx/arrimpl.cpp>
@@ -30,12 +31,11 @@ END_EVENT_TABLE()
 const int ITEM_HEIGHT = 50;
 #define ITEM_FONT wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT)
 
-VobListBox::VobListBox(wxWindow* parent, wxWindowID id, Vob* vob, wxArrayString audioLangCodes,
-		wxArrayString subtitleLangCodes, DVD* dvd): wxVListBox(parent, id) {
-	m_vob = vob;
-	m_audioLangCodes = audioLangCodes;
-	m_subtitleLangCodes = subtitleLangCodes;
+VobListBox::VobListBox(wxWindow* parent, wxWindowID id, Vob* vob, PgcArray* pgcs, DVD* dvd): wxVListBox(parent, id) {
+	m_vob = new Vob(*vob);
+	m_pgcs = pgcs;
 	m_dvd = dvd;
+	m_aspectRatio = pgcs->GetVideo().GetAspect();
 	m_videoImg = Scale(LoadFrame(m_vob->GetFilename()));
 	m_audioImg = Scale(wxBITMAP_FROM_MEMORY(audio).ConvertToImage());
 	m_subtitleImg = Scale(wxBITMAP_FROM_MEMORY(subtitle).ConvertToImage());
@@ -123,8 +123,8 @@ void VobListBox::RefreshInfo() {
 				y += AddChoiceCtrl(DVD::GetAudioFormatLabels(true, true), stream->GetAudioFormat(), rect, itemY,
 						choiceIdx, !m_vob->GetDoNotTranscode());
 				rect2.x += m_choiceList[choiceIdx-1]->GetSize().GetWidth() + 2;
-				int langIdx = DVD::GetAudioLanguageCodes().Index(audioLangIdx < m_audioLangCodes.GetCount()
-						? m_audioLangCodes[audioLangIdx] : s_config.GetDefAudioLanguage());
+				int langIdx = DVD::GetAudioLanguageCodes().Index(audioLangIdx < m_pgcs->GetAudioLangCodes().size()
+						? m_pgcs->GetAudioLangCodes()[audioLangIdx] : s_config.GetDefAudioLanguage());
 				audioLangIdx++;
 				AddChoiceCtrl(DVD::GetAudioLanguageCodes(), langIdx, rect2, itemY, choiceIdx, true);
 				break;
@@ -137,9 +137,12 @@ void VobListBox::RefreshInfo() {
 			}
 		}
 		itemY += y > ITEM_HEIGHT ? y + 1 : ITEM_HEIGHT + 1;
+		
+		if (n == 0)
+			m_audioChoiceIdx = choiceIdx;
 	}
 
-	m_subtitleChoiceIdx = m_choiceList.size();
+	m_subtitleChoiceIdx = choiceIdx;
 	for (unsigned int si = 0; si < m_vob->GetSubtitles().size(); si++) {
 		m_info.Add(wxArrayString());
 		m_infoRect.Add(RectList());
@@ -155,8 +158,8 @@ void VobListBox::RefreshInfo() {
 		y = AddInfo(_("Subtitle:") + wxString(wxT(" ")) + m_vob->GetSubtitles()[si]->GetFilename().AfterLast(wxT('.')),
 				n, dc, x, y);
 
-		int langIdx = DVD::GetAudioLanguageCodes().Index(si < m_subtitleLangCodes.GetCount()
-				? m_subtitleLangCodes[si] : s_config.GetDefSubtitleLanguage());
+		int langIdx = DVD::GetAudioLanguageCodes().Index(si < m_pgcs->GetSubPictures().size()
+				? m_pgcs->GetSubPictures()[si]->GetLangCode() : s_config.GetDefSubtitleLanguage());
 		wxRect& rect = m_infoRect[n][m_infoRect[n].size()-1];
 		y += AddChoiceCtrl(DVD::GetAudioLanguageCodes(), langIdx, rect, itemY, choiceIdx, true);
 		itemY += y > ITEM_HEIGHT ? y + 1 : ITEM_HEIGHT + 1;
@@ -175,10 +178,13 @@ int VobListBox::AddInfo(const wxString& s, int n, wxDC& dc, int x, int y) {
 int VobListBox::AddChoiceCtrl(wxArrayString formats, int selection, wxRect& rect, int itemY, int& choiceIdx,
 		bool enabled) {
 	wxChoice* ctrl = NULL;
-	if (choiceIdx >= (int) m_choiceList.size()) {
+	if (choiceIdx >= (int) m_choiceList.size() || m_choiceList[choiceIdx] == NULL) {
 		ctrl = new wxChoice(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, formats);
-		m_choiceList.Add(ctrl);
 		ctrl->SetSelection(selection);
+		if (choiceIdx >= (int) m_choiceList.size())
+			m_choiceList.push_back(ctrl);
+		else
+			m_choiceList[choiceIdx] = ctrl;
 	} else
 		ctrl = m_choiceList[choiceIdx];
 	choiceIdx++;
@@ -225,25 +231,24 @@ void VobListBox::RemoveItem(int index) {
 		// remove subtitle
 		int subtitleIdx = index - 1 - m_vob->GetAudioFilenames().size();
 		m_vob->GetSubtitles().RemoveAt(subtitleIdx);
-		m_subtitleLangCodes.RemoveAt(subtitleIdx);
 		// choice index = count of video stream + count of audio streams * 2 + count of audio subtitle streams
-		int choiceIdx = 1 + m_vob->GetAudioStreamCount()*2 + subtitleIdx;
+		int choiceIdx = m_subtitleChoiceIdx + subtitleIdx;
 		m_choiceList[choiceIdx]->Hide();
-		m_choiceList.RemoveAt(choiceIdx);
+		m_choiceList.erase(m_choiceList.begin() + choiceIdx);
 	} else {
 		// removed audio file
-		m_vob->RemoveAudioFile(index-1);
+		int audioIdx = index - 1;
+		m_vob->RemoveAudioFile(audioIdx);
 		// choice index = count of video stream + count of audio streams * 2
-		int choiceIdx = 1 + (m_vob->GetAudioStreamCount() - m_vob->GetAudioFilenames().size() + index - 1)*2;
+		int choiceIdx = m_audioChoiceIdx + audioIdx*2;
 		m_choiceList[choiceIdx]->Hide();
 		m_choiceList[choiceIdx + 1]->Hide();
-		m_choiceList.RemoveAt(choiceIdx);
-		m_choiceList.RemoveAt(choiceIdx);
+		m_choiceList.erase(m_choiceList.begin() + choiceIdx);
+		m_choiceList.erase(m_choiceList.begin() + choiceIdx);
 	}
 	RefreshInfo();
 	RefreshAll();
 }
-	
 
 void VobListBox::AddAudio(wxString filename) {
 	m_vob->AddAudioFile(filename);
@@ -256,6 +261,9 @@ void VobListBox::AddAudio(wxString filename) {
 		stream->SetDestinationFormat(m_dvd->GetAudioFormat());
 	else if (stream->GetSourceSampleRate() != 48000)
 		stream->SetDestinationFormat(stream->GetSourceAudioFormat());
+	// add choice controls
+	m_choiceList.insert(m_choiceList.begin() + m_subtitleChoiceIdx, NULL);
+	m_choiceList.insert(m_choiceList.begin() + m_subtitleChoiceIdx, NULL);
 	// update list box
 	RefreshInfo();
 	RefreshAll();
@@ -263,7 +271,6 @@ void VobListBox::AddAudio(wxString filename) {
 
 void VobListBox::AddSubtitle(wxString filename) {
 	m_vob->GetSubtitles().Add(new TextSub(filename));
-	m_subtitleLangCodes.Add(s_config.GetDefSubtitleLanguage());
 	// update list box
 	RefreshInfo();
 	RefreshAll();
@@ -292,13 +299,20 @@ void VobListBox::SetDoNotTranscode(bool value) {
 	}
 }
 
+/**
+ * Returns true if "do not transcode active ist"
+ */
+bool VobListBox::GetDoNotTranscode() {
+	return m_vob->GetDoNotTranscode();
+}
+
 bool VobListBox::HasAudioFiles() {
 	return m_vob->GetAudioFilenames().size() > 0;
 }
 
-Vob* VobListBox::UpdateVob() {
+Vob* VobListBox::SetValues() {
 	unsigned int choiceIdx = 0;
-	unsigned int audioLangIdx = 0;
+	unsigned int audioIdx = 0;
 	for (unsigned int stIdx = 0; stIdx < m_vob->GetStreams().size(); stIdx++) {
 		Stream* stream = m_vob->GetStreams()[stIdx];
 		switch (stream->GetType()) {
@@ -307,33 +321,74 @@ Vob* VobListBox::UpdateVob() {
 			break;
 		case stAUDIO:
 			stream->SetDestinationFormat(m_choiceList[choiceIdx++]->GetSelection());
-			if (audioLangIdx < m_audioLangCodes.GetCount())
-				m_audioLangCodes.RemoveAt(audioLangIdx);
-			m_audioLangCodes.Insert(m_choiceList[choiceIdx++]->GetStringSelection(), audioLangIdx++);
+			if (audioIdx >= m_pgcs->GetAudioLangCodes().size())
+				m_pgcs->GetAudioLangCodes().push_back(m_choiceList[choiceIdx++]->GetStringSelection());
+			else
+				m_pgcs->GetAudioLangCodes()[audioIdx] = m_choiceList[choiceIdx++]->GetStringSelection();
+			audioIdx++;
 			break;
 		default:
 			break;
 		}
 	}
-	m_subtitleLangCodes.clear();
 	for (unsigned int si = 0; si < m_vob->GetSubtitles().size(); si++) {
-		m_subtitleLangCodes.Add(m_choiceList[choiceIdx++]->GetStringSelection());
+		if (m_pgcs->GetSubPictures().size() <= si)
+			m_pgcs->GetSubPictures().push_back(new SubPicture(m_choiceList[choiceIdx++]->GetStringSelection()));
+		else
+			m_pgcs->GetSubPictures()[si]->SetLangCode(m_choiceList[choiceIdx++]->GetStringSelection());
 	}
 	return m_vob;
 }
 
+/**
+ * Get subtitle language code
+ */
+wxString VobListBox::GetSubtitleLangCode(int subtitleIndex) {
+	return m_choiceList[m_subtitleChoiceIdx + subtitleIndex]->GetStringSelection();
+}
+
+/**
+ * Sets subtitle language code
+ */
+void VobListBox::SetSubtitleLangCode(int subtitleIndex, wxString langCode) {
+	m_choiceList[m_subtitleChoiceIdx + subtitleIndex]->SetSelection(DVD::GetAudioLanguageCodes().Index(langCode));
+}
+
+/**
+ * Returns true if selected item has properties dialog
+ */
+bool VobListBox::HasPropDialog() {
+	return GetSelection() == 0 || GetSelection() >= 1 + (int) m_vob->GetAudioFilenames().GetCount();
+}
+
+/**
+ * Shows subtitle properties dialog  
+ */
+void VobListBox::ShowPropDialog() {
+	if (GetSelection() == 0) {
+		// video
+		SetValues(); // update destination format
+		VideoPropDlg dialog(this, m_vob, m_aspectRatio);
+		if (dialog.ShowModal() == wxID_OK) {
+			if (m_vob->GetStreams()[0]->GetDestinationFormat() > vfCOPY)
+				SetDoNotTranscode(false);
+			m_choiceList[0]->SetSelection(m_vob->GetStreams()[0]->GetDestinationFormat() - 1);
+			m_aspectRatio = dialog.GetAspectRatio();
+		}
+	} else if (GetSelection() >= 1 + (int) m_vob->GetAudioFilenames().GetCount()) {
+		// subtitle
+		int idx = GetSelection() - 1 - m_vob->GetAudioFilenames().GetCount();
+		TextSub* textsub = m_vob->GetSubtitles()[idx];
+		SubtitlePropDlg dialog(this, textsub, GetSubtitleLangCode(idx));
+		if (dialog.ShowModal() == wxID_OK) {
+			SetSubtitleLangCode(idx, dialog.GetLangCode());
+		}
+	}
+}
+
 /**
  * Processes a double click event
  */
 void VobListBox::OnDoubleClick(wxMouseEvent& evt) {
-	if (GetSelection() >= 1 + (int) m_vob->GetAudioFilenames().GetCount()) {
-		UpdateVob();
-		int idx = GetSelection() - 1 - m_vob->GetAudioFilenames().GetCount();
-		TextSub* textsub = m_vob->GetSubtitles()[idx];
-		SubtitlePropDlg dialog(this, textsub, &m_subtitleLangCodes[idx]);
-		if (dialog.ShowModal() == wxID_OK) {
-			// update subtitle language choice
-			// TODO use m_subtitleChoiceIdx
-		}
-	}
+	ShowPropDialog();
 }
