@@ -3,7 +3,7 @@
 // Purpose:     The list box to display information about streams in given VOB
 // Author:      Alex Thuering
 // Created:     03.05.2009
-// RCS-ID:      $Id: VobListBox.h,v 1.6 2010/04/03 21:52:48 ntalex Exp $
+// RCS-ID:      $Id: VobListBox.h,v 1.9 2010/10/10 18:39:55 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -16,11 +16,11 @@
 #include <wx/choice.h>
 #include <wx/dynarray.h>
 #include "DVD.h"
+#include <vector>
 
 WX_DECLARE_OBJARRAY(wxRect, RectList);
 WX_DECLARE_OBJARRAY(RectList, RectListOfList);
 WX_DECLARE_OBJARRAY(wxArrayString, StringListOfList);
-WX_DEFINE_ARRAY(wxChoice*, ChoiceList);
 
 /**
  * The list box to display information about streams in given VOB.
@@ -30,8 +30,7 @@ public:
 	/**
 	 * Constructor
 	 */
-	VobListBox(wxWindow* parent, wxWindowID id, Vob* vob, wxArrayString audioLangCodes,
-			wxArrayString subtitleLangCodes, DVD* dvd);
+	VobListBox(wxWindow* parent, wxWindowID id, Vob* vob, PgcArray* pgcs, DVD* dvd);
 	
 	/**
 	 * Returns the height of the specified item (in pixels)
@@ -63,42 +62,64 @@ public:
 	 */
 	void SetDoNotTranscode(bool value);
 	
+	/**
+	 * Returns true if "do not transcode active ist"
+	 */
+	bool GetDoNotTranscode();
+	
 	/**
 	 * Returns true if this vob contains audio files
 	 */
 	bool HasAudioFiles();
 	
 	/**
-	 * Updates vob object with values from controls
+	 * Updates vob and pgcs objects with values from controls
 	 */
-	Vob* UpdateVob();
+	Vob* SetValues();
 	
 	/**
-	 * Returns the list of audio language codes
+	 * Sets subtitle language code
 	 */
-	wxArrayString& GetAudioLangCodes() { return m_audioLangCodes; }
+	void SetSubtitleLangCode(int subtitleIndex, wxString langCode);
 	
 	/**
-	 * Returns the list of subtitle language codes
+	 * Get subtitle language code
 	 */
-	wxArrayString& GetSubtitleLangCodes() { return m_subtitleLangCodes; }
+	wxString GetSubtitleLangCode(int subtitleIndex);
+	
+	/**
+	 * Returns selected aspect ratio
+	 */
+	AspectRatio GetAspectRatio() { return m_aspectRatio; }
+	
+	/**
+	 * Returns true if selected item has properties dialog
+	 */
+	bool HasPropDialog();
+	
+	/**
+	 * Shows subtitle properties dialog  
+	 */
+	void ShowPropDialog();
 	
 private:
 	/** The DVD */
 	DVD* m_dvd;
-	/** The VOB to edit */
+	/** Copy of VOB object to edit */
 	Vob* m_vob;
-	/** The list of audio language codes */
-	wxArrayString m_audioLangCodes;
-	/** The list of subtitle language codes */
-	wxArrayString m_subtitleLangCodes;
+	/** Arrays of PGCs */
+	PgcArray* m_pgcs;
+	/** selected aspect ratio */
+	AspectRatio m_aspectRatio;
 	/** The list of information string list for every file in VOB */
 	StringListOfList m_info;
 	/** The list of rectangle list. Each rectangle stores coordinates and size of one information string. */
 	RectListOfList m_infoRect;
 	/** The list of choice controls */
-	ChoiceList m_choiceList;
-	/** Index of first subtitle choice */
+	vector<wxChoice*> m_choiceList;
+	/** Index of first choice for audio file */
+	int m_audioChoiceIdx;
+	/** Index of first choice for subtitle */
 	int m_subtitleChoiceIdx;
 	
 	wxBitmap m_videoImg;
