@@ -3,7 +3,7 @@
 // Purpose:     DVD Authoring Application
 // Author:      Alex Thuering
 // Created:	10.10.2003
-// RCS-ID:      $Id: dvdstyler.cpp,v 1.44 2010/05/22 14:13:38 ntalex Exp $
+// RCS-ID:      $Id: dvdstyler.cpp,v 1.48 2010/07/05 19:17:09 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -33,6 +33,11 @@
 
 #define LOCALE_DIR wxFindDataDirectory(_T("..") + wxString(wxFILE_SEP_PATH) + _T("locale"))
 
+#ifdef __WXMAC__
+#include <fontconfig/fontconfig.h>
+#define FONTS_CONF wxFindDataFile(wxT("Resources/fonts.conf"))
+#endif
+
 class DVDStyler : public wxApp {
 public:
 	bool OnInit();
@@ -47,6 +52,7 @@ bool DVDStyler::OnInit() {
 	// fix config
 	s_config.Init();
 	wxString version = s_config.GetVersion();
+	bool firstStart = version != APP_VERSION;
 	if (version.length() < 3 || version.Mid(0, 3) != APP_VERSION.Mid(0, 3)) {
 		wxLogNull log;
 		s_config.SetVersion(APP_VERSION);
@@ -76,8 +82,7 @@ bool DVDStyler::OnInit() {
 
 	// locale
 	int lang = s_config.GetLanguage();
-	bool firstStart = lang == wxLANGUAGE_UNKNOWN;
-	if (firstStart) {
+	if (lang == wxLANGUAGE_UNKNOWN) {
 		wxString languageCode = s_config.GetLanguageCdoe();
 		if (languageCode.length() > 0 && wxLocale::FindLanguageInfo(languageCode)) {
 			lang = wxLocale::FindLanguageInfo(languageCode)->Language;
@@ -94,18 +99,42 @@ bool DVDStyler::OnInit() {
 		m_locale.AddCatalogLookupPathPrefix(LOCALE_DIR);
 		m_locale.AddCatalog(wxT("dvdstyler"));
 	}
-
+	
+	// set default video format
+	if (firstStart) {
+#ifdef __WXMSW__
+		wxString ntscCountriesArr[3] = { wxT("USA"), wxT("Canada"), wxT("Japan") };
+		wxArrayString ntscCountries(3, ntscCountriesArr);
+		wxRegKey key(wxT("HKEY_CURRENT_USER\\Control Panel\\International"));
+		wxString country;
+		if (key.QueryValue(wxT("sCountry"), country) && ntscCountries.Index(country, false) >= 0)
+			s_config.SetDefVideoFormat(vfNTSC);
+#else
+		if (wxDateTime::GetCountry() == wxDateTime::USA)
+			s_config.SetDefVideoFormat(vfNTSC);
+#endif
+	}
+	
 #ifndef __WXWINCE__
 	setlocale(LC_NUMERIC, "C");
 #endif
-
+	
 	wxFfmpegMediaDecoder::Init();
-#ifndef __WXMSW__
-	if (firstStart) {
-		// check if mplex exists
-		wxLogNull log;
-		bool mplex = wxExecute(wxT("mplex"), wxEXEC_SYNC | wxEXEC_NODISABLE) == 1;
-		s_config.SetUseMplex(mplex);
+	
+#ifdef __WXMAC__
+	// init font config
+	wxString fontConfigFile;
+	if (!wxGetEnv(wxT("FONTCONFIG_FILE"), &fontConfigFile)
+			&& !wxFile::Exists(wxGetHomeDir() + wxFILE_SEP_PATH + wxT(".fonts.conf"))) {
+		// load fonts.conf
+		FcConfig* fc = FcConfigCreate();
+		if (fc == NULL) {
+			wxLogError(wxString(wxT("SubtitlePropDlg::UpdateFontList(): ")) + wxString(wxT("FcConfigCreate failed.")));
+			return false;
+		}
+		FcConfigParseAndLoad(fc, (const FcChar8*) (const char*) FONTS_CONF.ToUTF8(), FcTrue);
+		FcConfigBuildFonts(fc);
+		FcConfigSetCurrent(fc);
 	}
 #endif
 	
@@ -119,10 +148,10 @@ bool DVDStyler::OnInit() {
 	
 	// parse command line
 	static const wxCmdLineEntryDesc cmdLineDesc[] = {
-		{ wxCMD_LINE_SWITCH, wxString(wxT("s")), wxString(wxT("start")), _("automatically starts the generation and burning") },
-		{ wxCMD_LINE_SWITCH, wxString(wxT("e")), wxString(wxT("stderr")), _("write log messages to standard error") },
-		{ wxCMD_LINE_SWITCH, wxString(wxT("v")), wxString(wxT("version")), _("show version of program") },
-		{ wxCMD_LINE_SWITCH, wxString(wxT("h")), wxString(wxT("help")), _("show summary of options") },
+		{ wxCMD_LINE_SWITCH, wxT("s"), wxT("start"), _("automatically starts the generation and burning") },
+		{ wxCMD_LINE_SWITCH, wxT("e"), wxT("stderr"), _("write log messages to standard error") },
+		{ wxCMD_LINE_SWITCH, wxT("v"), wxT("version"), _("show version of program") },
+		{ wxCMD_LINE_SWITCH, wxT("h"), wxT("help"), _("show summary of options") },
 		{ wxCMD_LINE_PARAM, NULL, NULL, _("input file"), wxCMD_LINE_VAL_STRING,
 				wxCMD_LINE_PARAM_OPTIONAL | wxCMD_LINE_PARAM_MULTIPLE },
 		{ wxCMD_LINE_NONE }
