@@ -3,7 +3,7 @@
 // Purpose:     Media Encoder Interface
 // Author:      Alex Thuering
 // Created:     24.02.2007
-// RCS-ID:      $Id: mediaenc.h,v 1.5 2008/12/31 16:02:59 ntalex Exp $
+// RCS-ID:      $Id: mediaenc.h,v 1.6 2010/05/03 06:06:48 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -17,7 +17,13 @@ enum VideoFormat {
 	vfNONE = 0,
 	vfCOPY,
 	vfPAL,
-	vfNTSC
+	vfNTSC,
+	vfPAL_CROPPED,
+	vfNTSC_CROPPED,
+	vfPAL_HALF_D1,
+	vfNTSC_HALF_D1,
+	vfPAL_VCD,
+	vfNTSC_VCD
 };
 
 enum AudioFormat {
@@ -39,6 +45,11 @@ enum AspectRatio {
 	ar16_9
 };
 
+/** Returns frame size of given video format */
+wxSize GetFrameSize(VideoFormat format);
+/** Returns true if video format is NTSC */
+bool isNTSC(VideoFormat format);
+
 class wxMediaEncoder {
   public:
 	virtual ~wxMediaEncoder() {}
