@@ -3,7 +3,7 @@
 // Purpose:     FFMPEG Media Encoder
 // Author:      Alex Thuering
 // Created:     04.08.2007
-// RCS-ID:      $Id: mediaenc_ffmpeg.cpp,v 1.19 2010/05/03 06:06:48 ntalex Exp $
+// RCS-ID:      $Id: mediaenc_ffmpeg.cpp,v 1.20 2010/06/30 20:08:56 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -59,12 +59,16 @@ wxString wxFfmpegMediaEncoder::GetBackendVersion() {
 			LIBAVUTIL_VERSION_INT>>16, LIBAVUTIL_VERSION_INT>>8&0xFF, LIBAVUTIL_VERSION_INT&0xFF);
 }
 
+#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(52, 45, 0)
+#define av_guess_format guess_format
+#endif
+
 bool wxFfmpegMediaEncoder::BeginEncode(const wxString& fileName, VideoFormat videoFormat,
 		AudioFormat audioFormat, AspectRatio aspectRatio, int videoBitrate) {
     EndEncode();
-    AVOutputFormat* outputFormat = guess_format("dvd", NULL, NULL);
+    AVOutputFormat* outputFormat = av_guess_format("dvd", NULL, NULL);
     if (audioFormat == afNONE)
-    	outputFormat = guess_format(NULL, fileName.mb_str(), NULL);
+    	outputFormat = av_guess_format(NULL, fileName.mb_str(), NULL);
     if (!outputFormat) {
       wxLogError(wxT("Cannot open output format"));
       return false;
@@ -288,15 +292,11 @@ void wxFfmpegMediaEncoder::CloseVideoEncoder() {
     avcodec_close(m_videoStm->codec);
     if (m_imgConvertCtx)
         sws_freeContext(m_imgConvertCtx);
-    if (m_picture)
-    {
-        av_free(m_picture->data[0]);
-        av_free(m_picture);
-        m_picture = NULL;
+    if (m_picture) {
+        av_freep(&m_picture->data[0]);
+        av_freep(&m_picture);
     }
-    if (m_videoOutbuf)
-        av_free(m_videoOutbuf);
-    m_videoOutbuf = NULL;
+    av_freep(&m_videoOutbuf);
     m_videoStm = NULL;
 }
 
@@ -380,14 +380,15 @@ bool wxFfmpegMediaEncoder::writeVideoFrame() {
 }
 
 void wxFfmpegMediaEncoder::EndEncode() {
-    CloseVideoEncoder();
-    CloseAudioEncoder();
-    
     if (!m_outputCtx)
         return;
     
+    CloseVideoEncoder();
+    CloseAudioEncoder();
+    
     // write the trailer
-    av_write_trailer(m_outputCtx);
+    if (m_outputCtx->nb_streams)
+    	av_write_trailer(m_outputCtx);
 
     // free the streams
     for (unsigned int i = 0; i < m_outputCtx->nb_streams; i++) {
@@ -403,6 +404,5 @@ void wxFfmpegMediaEncoder::EndEncode() {
 #endif
     
     // free the stream
-    av_free(m_outputCtx);
-    m_outputCtx = NULL;
+    av_freep(&m_outputCtx);
 }
