@@ -3,7 +3,7 @@
 // Purpose:     Properties dialog (base class for all properties dialogs)
 // Author:      Alex Thuering
 // Created:     18.11.2003
-// RCS-ID:      $Id: PropDlg.cpp,v 1.21 2010/04/04 10:13:24 ntalex Exp $
+// RCS-ID:      $Id: PropDlg.cpp,v 1.24 2010/10/10 18:37:52 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -222,16 +222,14 @@ void wxPropDlg::AddText(wxSizer* sizer, wxString text, int flag) {
 		sizer->Add(10, 10);
 }
 
-void wxPropDlg::AddTextProp(wxSizer* sizer, const wxString& label, const wxString& value,
-  bool readonly, int width)
-{
-  if (!sizer) // only update value
-  {
+wxSizer* wxPropDlg::AddTextProp(wxSizer* sizer, const wxString& label, const wxString& value,
+		bool readonly, int width, bool addSpacer) {
+  if (!sizer) { // only update value
 	wxTextCtrl* ctrl = (wxTextCtrl*) m_controls[m_updateIndex];
 	ctrl->SetValue(value);
 	ctrl->SetEditable(!readonly);
 	m_updateIndex++;
-	return;
+	return NULL;
   }
   // add label
   if (label.length())
@@ -240,22 +238,23 @@ void wxPropDlg::AddTextProp(wxSizer* sizer, const wxString& label, const wxStrin
   wxSize size = width > 0 ? wxSize(width, -1) : wxDefaultSize;
   wxTextCtrl* ctrl = new wxTextCtrl(propWindow, -1, value, wxDefaultPosition, size);
   ctrl->SetEditable(!readonly);
+  wxBoxSizer* pane = NULL;
   if (width > 0) {
-    wxBoxSizer* pane = new wxBoxSizer(wxHORIZONTAL);
+    pane = new wxBoxSizer(wxHORIZONTAL);
     pane->Add(ctrl);
-    pane->Add(10, 10, 1, wxEXPAND);
+	if (addSpacer)
+		pane->Add(0, 0, 1, wxEXPAND);
     sizer->Add(pane, 1, wxEXPAND, 0);
   } else
     sizer->Add(ctrl, 1, wxEXPAND, 0);
   m_types.Add(propTEXT);
   m_controls.Add(ctrl);
   m_groupIds.Add(m_currGroupId);
+  return pane;
 }
 
-void wxPropDlg::AddStaticTextProp(wxSizer* sizer, const wxString& label, const wxString& value)
-{
-  if (!sizer) // only update value
-  {
+void wxPropDlg::AddStaticTextProp(wxSizer* sizer, const wxString& label, const wxString& value) {
+  if (!sizer) { // only update value
 	((wxStaticText*)m_controls[m_updateIndex])->SetLabel(value);
 	m_updateIndex++;
 	return;
@@ -314,11 +313,8 @@ wxSizer* wxPropDlg::AddSpinProp(wxSizer* sizer, const wxString& label, int value
 	return pane;
 }
 
-void wxPropDlg::AddCheckProp(wxSizer* sizer, const wxString& label, bool value,
-  bool readonly, int id)
-{
-  if (!sizer) // only update value
-  {
+void wxPropDlg::AddCheckProp(wxSizer* sizer, const wxString& label, bool value, bool readonly, int id) {
+  if (!sizer) { // only update value
 	wxCheckBox* ctrl = (wxCheckBox*)m_controls[m_updateIndex];
 	ctrl->SetValue(value);
 	ctrl->Enable(!readonly);
@@ -336,11 +332,8 @@ void wxPropDlg::AddCheckProp(wxSizer* sizer, const wxString& label, bool value,
   m_groupIds.Add(m_currGroupId);
 }
 
-void wxPropDlg::AddRadioProp(wxSizer* sizer, const wxString& label, bool value,
-	  long style, bool readonly, int id)
-{
-  if (!sizer) // only update value
-  {
+void wxPropDlg::AddRadioProp(wxSizer* sizer, const wxString& label, bool value, long style, bool readonly, int id) {
+  if (!sizer) { // only update value
 	wxRadioButton* ctrl = (wxRadioButton*)m_controls[m_updateIndex];
 	ctrl->SetValue(value);
 	ctrl->Enable(!readonly);
@@ -359,14 +352,10 @@ void wxPropDlg::AddRadioProp(wxSizer* sizer, const wxString& label, bool value,
   m_groupIds.Add(m_currGroupId);
 }
 
-void wxPropDlg::AddRadioGroupProp(wxSizer* sizer, const wxArrayString& text,
-	int value, bool readonly)
-{
-  if (!sizer) // only update value
-  {
+void wxPropDlg::AddRadioGroupProp(wxSizer* sizer, const wxArrayString& text, int value, bool readonly) {
+  if (!sizer) { // only update value
   	wxArrayPtrVoid* radioControls = (wxArrayPtrVoid*) m_controls[m_updateIndex];
-  	for (int i=0; i<(int)radioControls->GetCount(); i++)
-  	{
+  	for (int i=0; i<(int)radioControls->GetCount(); i++) {
 	  wxRadioButton* ctrl = (wxRadioButton*)(*radioControls)[i];
 	  ctrl->SetValue(i == value);
 	  ctrl->Enable(!readonly);
@@ -394,8 +383,6 @@ wxSizer* wxPropDlg::AddComboProp(wxSizer* sizer, const wxString& label, const wx
 		const wxArrayString& choices, long style, int width, bool addSpacer, int id) {
 	if (!sizer) { // only update value
 		wxComboBox* ctrl = (wxComboBox*)m_controls[m_updateIndex];
-		ctrl->Clear();
-		ctrl->Append(choices);
 		ctrl->SetStringSelection(value);
 		m_updateIndex++;
 		return NULL;
@@ -424,9 +411,6 @@ wxSizer* wxPropDlg::AddBitmapComboProp(wxSizer* sizer, const wxString& label, co
 		const wxArrayString& choices, const BitmapArray& bitmaps, long style, int width, bool addSpacer, int id) {
 	if (!sizer) { // only update value
 		wxBitmapComboBox* ctrl = (wxBitmapComboBox*)m_controls[m_updateIndex];
-		ctrl->Clear();
-		for (unsigned int i = 0; i < choices.size(); i ++)
-			ctrl->Append(choices[i], bitmaps[i]);
 		ctrl->SetStringSelection(value);
 		m_updateIndex++;
 		return NULL;
@@ -436,7 +420,7 @@ wxSizer* wxPropDlg::AddBitmapComboProp(wxSizer* sizer, const wxString& label, co
 		sizer->Add(new wxStaticText(propWindow, -1, label), 0, wxALIGN_CENTER_VERTICAL);
 	wxBitmapComboBox* ctrl =
 		new wxBitmapComboBox(propWindow, id, value, wxDefaultPosition, wxDefaultSize, choices, style);
-	for (unsigned int i = 0; i < bitmaps.size(); i ++)
+	for (unsigned int i = 0; i < bitmaps.size(); i++)
 		ctrl->SetItemBitmap(i, bitmaps[i]);
 	ctrl->SetStringSelection(value);
 	wxBoxSizer* pane = NULL;
@@ -458,18 +442,14 @@ wxSizer* wxPropDlg::AddChoiceProp(wxSizer* sizer, const wxString& label, const w
 		const wxArrayString& choices, int width, bool addSpacer, int id) {
 	if (!sizer) { // only update value
 		wxChoice* ctrl = (wxChoice*)m_controls[m_updateIndex];
-		ctrl->Clear();
-		SetChoiceData(ctrl, value, choices);
+		ctrl->SetStringSelection(value);
 		m_updateIndex++;
 		return NULL;
 	}
 	// add combobox control
 	if (label.length())
 		sizer->Add(new wxStaticText(propWindow, -1, label), 0, wxALIGN_CENTER_VERTICAL);
-	wxString strings[1];
-	wxChoice* ctrl = new wxChoice(propWindow, id,
-			wxDefaultPosition, wxDefaultSize, 0, strings);
-	SetChoiceData(ctrl, value, choices);
+	wxChoice* ctrl = AddChoiceProp(value, choices, id);
 	wxBoxSizer* pane = NULL;
   	if (width >= 0) {
   		pane = new wxBoxSizer(wxHORIZONTAL);
@@ -481,12 +461,23 @@ wxSizer* wxPropDlg::AddChoiceProp(wxSizer* sizer, const wxString& label, const w
   			ctrl->SetMinSize(wxSize(width, -1));
   	} else
   		sizer->Add(ctrl, 1, wxEXPAND, 0);
+	return pane;
+}
+
+/**
+ * Creates choice control.
+ * @returns Choice control
+ */
+wxChoice* wxPropDlg::AddChoiceProp(const wxString& value, const wxArrayString& choices, int id) {
+	wxChoice* ctrl = new wxChoice(propWindow, id, wxDefaultPosition, wxDefaultSize, choices);
+	ctrl->SetStringSelection(value);
 	m_types.Add(propCHOICE);
 	m_controls.Add(ctrl);
 	m_groupIds.Add(m_currGroupId);
-	return pane;
+	return ctrl;
 }
 
+
 /**
  * Creates label and choice control with item 'Custom'.
  * @returns subSizer if width > 0, null otherwise
@@ -504,16 +495,6 @@ void wxPropDlg::SetLastControlCustom(int choiceCtrlIdx, bool enable) {
 	GetLastControl()->Enable(enable);
 }
 
-void wxPropDlg::SetChoiceData(wxChoice* ctrl, const wxString& value, const wxArrayString& choices) {
-	int sel = -1;
-	for (int i = 0; i < (int) choices.GetCount(); i++) {
-		ctrl->Append(choices[i]);
-		if (choices[i] == value)
-			sel = i;
-	}
-	ctrl->SetSelection(sel);
-}
-
 void wxPropDlg::OnSelectCustomChoice(wxCommandEvent& evt) {
 	wxString name = ((wxWindow*)evt.GetEventObject())->GetName();
 	long idx = 0;
@@ -525,11 +506,8 @@ void wxPropDlg::OnSelectCustomChoice(wxCommandEvent& evt) {
 }
 
 //--- Grid property ---//
-void wxPropDlg::AddGridProp(wxSizer* sizer, const wxArrayPtrVoid& data,
-  const wxString& rowTitle, bool editable)
-{
-  if (!sizer) // only update value
-  {
+void wxPropDlg::AddGridProp(wxSizer* sizer, const wxArrayPtrVoid& data, const wxString& rowTitle, bool editable) {
+  if (!sizer) { // only update value
 	wxGrid* grid = (wxGrid*) m_controls[m_updateIndex];
 	grid->ClearGrid();
 	SetGridData(grid, data, rowTitle, editable);
@@ -558,9 +536,7 @@ void wxPropDlg::AddGridProp(wxSizer* sizer, const wxArrayPtrVoid& data,
   m_groupIds.Add(m_currGroupId);
 }
 
-void wxPropDlg::SetGridData(wxGrid* grid, const wxArrayPtrVoid& data,
-  const wxString& rowTitle, bool editable)
-{
+void wxPropDlg::SetGridData(wxGrid* grid, const wxArrayPtrVoid& data, const wxString& rowTitle, bool editable) {
   grid->EnableEditing(editable);
 
   wxArrayString* cols = (wxArrayString*) data.Item(0);
