@@ -3,7 +3,7 @@
 // Purpose:     Base class for property dialogs
 // Author:      Alex Thuering
 // Created:     18.11.2003
-// RCS-ID:      $Id: PropDlg.h,v 1.18 2010/04/03 21:45:53 ntalex Exp $
+// RCS-ID:      $Id: PropDlg.h,v 1.21 2010/10/10 18:37:52 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -73,8 +73,8 @@ protected:
 
 	void AddText(wxSizer* sizer, wxString text, int flag = wxALIGN_CENTER_VERTICAL);
 
-	void AddTextProp(wxSizer* sizer, const wxString& label, const wxString& value,
-	  bool readonly = false, int width = -1);
+	wxSizer* AddTextProp(wxSizer* sizer, const wxString& label, const wxString& value,
+	  bool readonly = false, int width = -1, bool addSpacer = true);
 	void AddStaticTextProp(wxSizer* sizer, const wxString& label, const wxString& value);
 	/**
 	 * Adds a static line with given caption to the window
@@ -110,6 +110,11 @@ protected:
 	 */
 	wxSizer* AddChoiceProp(wxSizer* sizer, const wxString& label, const wxString& value,
 	  const wxArrayString& choices, int width = -1, bool addSpacer = true, int id = -1);
+	/**
+	 * Creates choice control.
+	 * @returns Choice control
+	 */
+	wxChoice* AddChoiceProp(const wxString& value, const wxArrayString& choices, int id = -1);
 	/**
 	 * Creates label and choice control with item 'Custom'.
 	 * @returns subSizer if width > 0, null otherwise
@@ -118,7 +123,6 @@ protected:
 	  const wxArrayString& choices, int customItemIdx, int width = -1, bool addSpacer = true);
 	/** Sets the last control as control to enter the custom value. */
 	void SetLastControlCustom(int choiceCtrlIdx, bool enable);
-	void SetChoiceData(wxChoice* ctrl, const wxString& value, const wxArrayString& choices);
 	void AddGridProp(wxSizer* sizer, const wxArrayPtrVoid& data,
 	  const wxString& rowTitle, bool editable);
 	void SetGridData(wxGrid* grid, const wxArrayPtrVoid& data,
