@@ -3,7 +3,7 @@
 // Purpose:     wxThumbnailFactory class
 // Author:      Alex Thuering
 // Created:		15.02.2003
-// RCS-ID:      $Id: ThumbnailFactory.cpp,v 1.16 2009/11/08 18:14:16 ntalex Exp $
+// RCS-ID:      $Id: ThumbnailFactory.cpp,v 1.17 2010/09/05 14:35:58 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -337,12 +337,22 @@ wxImage wxThumbnailFactory::GenerateThumbnail(ThumbInfo& info, bool save) {
 		} else
 			gnome_thumbnail_factory_create_failed_thumbnail(thumbnail_factory, info.uri, info.mtime);
 		return img;
-#elif defined(WX_SVG) || defined(FFMPEG)
-		wxFfmpegMediaDecoder ffmpeg;
-		if (ffmpeg.Load(info.filename) && ffmpeg.BeginDecode(96, 80)) {
-			if (ffmpeg.GetDuration()> 0)
-			ffmpeg.SetPosition(ffmpeg.GetDuration() * 0.05);
-			img = ffmpeg.GetNextFrame();
+#elif defined(WX_SVG) || defined(decoder)
+		wxFfmpegMediaDecoder decoder;
+		if (decoder.Load(info.filename) && decoder.BeginDecode(96, 80)) {
+			if (decoder.GetDuration() > 0) {
+				double dpos = decoder.GetDuration() * 0.05;
+				decoder.SetPosition(dpos > 0.5 ? dpos - 0.5 : dpos);
+				for (int i = 0; i < 100; i++) {
+					img = decoder.GetNextFrame();
+					double dpos1 = decoder.GetPosition();
+					if (dpos1 >= dpos || dpos1 < 0)
+						break;
+				}
+			} else {
+				for (int i = 0; i < 15; i++)
+					img = decoder.GetNextFrame();
+			}
 		}
 		if (img.Ok()) {
 			unsigned char* data = img.GetData();
