@@ -3,7 +3,7 @@
 // Purpose:     wxThumbnails class
 // Author:      Alex Thuering
 // Created:		2.02.2003
-// RCS-ID:      $Id: Thumbnails.cpp,v 1.28 2010/02/22 20:19:08 ntalex Exp $
+// RCS-ID:      $Id: Thumbnails.cpp,v 1.30 2010/06/29 18:41:10 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -220,7 +220,7 @@ void wxThumbnails::SetSelected(int value) {
 }
 
 wxString wxThumbnails::GetAudioVideoExtWildcard() {
-	return wxT("*.asf;*.avi;*.dv;*.flv;*.m2p;*.mkv;*.mov;*.mp4;*.mpeg;*.mpg;*.ogg;*.vob;*.wmv;");
+	return wxT("*.asf;*.avi;*.dv;*.flv;*.m2p;*.mkv;*.mov;*.mp4;*.mpeg;*.mpg;*.ogg;*.ogm;*.vob;*.webm;*.wmv;");
 }
 
 wxString wxThumbnails::GetVideoExtWildcard() {
@@ -298,19 +298,19 @@ void wxThumbnails::UpdateItemsInt() {
 		ShowDir(m_dir);
 }
 
-int wxThumbnails::GetRowHeight(int begRow, int count) {
+int wxThumbnails::GetRowHeight(unsigned int begRow, unsigned int count) {
 	// border
 	bool drawShadow = m_tOutline == wxTHUMB_OUTLINE_IMAGE;
 	int border = m_tBorder + (m_tOutline != wxTHUMB_OUTLINE_NONE ? 4 : 0) + (drawShadow ? 4 : 0) + 1;
 	// image height
 	int imgHeight = 0;
-	for (int i = begRow; i < begRow + count; i++)
-		if (i < (int) m_tImageHeight.GetCount())
+	for (unsigned int i = begRow; i < begRow + count; i++)
+		if (i < m_tImageHeight.GetCount())
 			imgHeight += m_tImageHeight[i];
 	// caption height
 	int capHeight = 0;
-	for (int i = begRow; i < begRow + count; i++)
-		if (i < (int) m_tCaptionHeight.GetCount())
+	for (unsigned int i = begRow; i < begRow + count; i++)
+		if (i < m_tCaptionHeight.GetCount())
 			capHeight += m_tCaptionHeight[i];
 	return imgHeight + border*count + capHeight*m_tTextHeight;
 }
